/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * COPSE Client Application
 * Classe:  ClientTest
\******************************************************************************/



import java.io.*;
import java.util.*;



import java.net.*;
import COPSE.*;

class ConnectionManager
    extends Thread
{
//    UserSessionManager      pUserSessionManager
    int     mUserId;

    Socket  pSocket;
    ObjectInputStream pInput;
    ObjectOutputStream pOutput;

    boolean mExit;

    public ConnectionManager( String serverHost, String serverPort )
        throws UnknownHostException, IOException
    {
        super();

        System.out.println( "DEBUG: Connection Manager - Creating Socket ..." );
        pSocket = new Socket( serverHost, new Integer(serverPort).intValue() );

        System.out.println( "DEBUG: Connection Manager - Getting Streams ..." );
        try {
            InputStream in = pSocket.getInputStream();
            OutputStream out = pSocket.getOutputStream();

            System.out.println( "DEBUG: Connection Manager - Creating Object Streams ..." );
            pInput = new ObjectInputStream( in );
            pOutput = new ObjectOutputStream( out );
        }

        catch( Exception e )
        {
            System.out.println( "DEBUG: Connection Manager - Error..." );
        }

        System.out.println( "DEBUG: Connection Manager - Ok" );
        mUserId = 0;
        mExit = false;
    }

    public void run()
    {
        while( !mExit )
        {
            try{
                //if( pInput.available() != 0 )
                //{
                    System.out.println( "DEBUG: Receiving Event ...");
                    CEvent event = (CEvent) pInput.readObject();
                    //pUserSessionManager.ReceiveEvent(event);

                    // Temporary:
                    System.out.println( "DEBUG: Received event #" + event.GetType() );

                    switch( event.GetType() )
                    {
                        case CEvent.EvUserConnection_Open:
                            {
                                Integer i = (Integer)event.GetArgumentAt(0);
                                if( mUserId == 0 )
                                    mUserId = i.intValue();
                                else
                                    // TODO: Add User to Logged User List...
                                    System.out.println( "User "+i+" logged on!" );
                            }
                            break;

                        case CEvent.EvUserConnection_Close:
                            {
                                Integer i = (Integer)event.GetArgumentAt(0);
                                if( mUserId == i.intValue() )
                                {
                                    mExit = true;
                                }
                                else
                                    // TODO: Remove User to Logged User List...
                                    System.out.println( "User "+i+" logged out!" );
                            }

                    } // end switch

                //} // end if
            } catch(IOException ioe)
            {
                // TODO: Verify if the connection is still alive
            } catch(Exception e)
            {}
        } // while( !mExit )

        //pSocket.close();
        //Exit();
    }

    public synchronized void Send( CEvent event )
        throws IOException
    {
        pOutput.writeObject( event );
    }

    public void Exit()
    {
        try{
            pSocket.close();
        } catch( IOException ioe ) {};
        stop();
    }

}







public class ClientTest
{

    ConnectionManager   pConMgr;

    public static void main( String[] args )
    {
        new ClientTest().run( args );
    }


    public ClientTest()
    {
        pConMgr = null;
    }


    public void run( String[] args )
    {
        System.out.println( "COPSE Client - Test" );

        if( args.length < 2 )
        {
            System.out.println( "Parameters:\n   <port_number> <user_login>" );
            return;
        }

        try{
            System.out.println( "DEBUG: Creating Connection Manager" );
            pConMgr = null;
            pConMgr = new ConnectionManager( "localhost", args[0] );
        }
        catch (UnknownHostException uhe)
        {   System.out.println( "Error: Unknown Host!" );
            return;
        }
        catch (IOException ioe )
        {   System.out.println( "DEBUG: IOException" );
            ioe.printStackTrace( System.out );
            return;
        }

        if( pConMgr != null )
            _Execute(args[1]);
    }

    private void _Execute( String login )
    {
        System.out.println( "DEBUG: Starting Connection Manager" );
        pConMgr.start();

        CEvent event = new CEvent( CEvent.EvUserConnection_Open );
        event.AddArgument( login );
        event.AddArgument( login );

        try {
            System.out.println( "DEBUG: Sending OpenConnection Command..." );
            pConMgr.Send( event );
        } catch(IOException ioe )
        {
            System.out.println( "DEBUG: Error on sending 'OPENCONNECTION' command..." );
            pConMgr.Exit();
            System.exit(0);
        }


        System.out.print(":");

        BufferedReader in = new BufferedReader(new InputStreamReader(System.in));

        String command;
        do {
            command = "";
            try {
                StringTokenizer st = new StringTokenizer(in.readLine());
                if( st.hasMoreTokens() )
                {
                    command = st.nextToken();

                    if( command.equals( "exit" ) )
                    {
                        System.out.println( "DEBUG: Sending Close Command ...");
                        event = new CEvent( CEvent.EvUserConnection_Close );
                        pConMgr.Send( event );
                        System.out.println( "DEBUG: Sent");
                    }
                    else
                        System.out.print(":");
                }
            } catch (IOException ioe )
            {
                System.out.println( "DEBUG: Error os String Tokenizer ...");
            }

        } while( !command.equals("exit") );

        try {
            pConMgr.join();
        } catch (Exception e ) {};
    }
}
