/**
 * UFRGS - II - PPGC - MESTRADO EM CIENCIA DA COMPUTACAO
 * AUTORA MANUELE KIRSCH PINHEIRO
 * CLASSE CL_MONITOR
 * PACOTE BW.CONTROL
 * DATA DE CRIACAO 16 / 07 / 2000
 */

package BW.control;

import java.util.Hashtable;
import java.util.Vector;

import BW.storage.FC_ST_Monitor;
import BW.typeIdDefault;
import BW.kernel.*;

/**
 * classe CL_Monitor e responsavel pelo monitoramento
 * dentro do BW
 * @autor Manuele K P
 * @version 1.0
 */
public class CL_Monitor {

    ///////////////////////////////// ATRIBUTOS //////////////


	/** mais alto identificador de objeto (objId) tratado dentro desta classe */
	int lastId;

	/** ativa ou desativa o controle dos identificadors de objetos para os eventos */
	boolean changeId;

	/** lista dos participantes ativos no grupo */
	//transferida essa responsbilidade para a classe CL_Awareness, por
	//se tratar isso de uma especie de awareness - presenca
	//Hashtable actives;

	/** grupo utilizado, normalmente o default,
	 * um objeto BW_Group com typeId=5 e
	 * objId=5 (BW.typeIdDefault.BW_GROUP)*/
	BW_Group group;

	/** eventos registrados para monitoramento,
	 * normalemente representada pelo objeto default,
	 * um objeto BW_Register de typeId=3
	 * e objId=3 (BW.typeIdDefault.BW_REGISTER) */
	BW_Register register;

	//FALTA: conexao com a classe para percepcao
	/** objeto responsavel pela contextualizacao */
	CL_Awareness awareness;
	
	//FALTA: conexao com a fachada de armazenamento
	/** Acesso a camada de armazenamento das informacoes */
	FC_ST_Monitor storage;
	
        //FALTA: relativo aos papeis
        //??? - estao dentro do grupo



    //////////////////////////////// CONSTRUTORES ////////////

	/** construtor default, com registro e grupo defautlts.
	 * O registro default e um objeto BW_Register de typeId=3
	 * e objId=3 (BW.typeIdDefault.BW_REGISTER), enquanto o
	 * grupo default e um objeto BW_Group com typeId=5 e
	 * objId=5 (BW.typeIdDefault.BW_GROUP).
	 */
	public CL_Monitor () {
		//super (typeIdDefault.CL_MONITOR,typeIdDefault.CL_MONITOR);

		//inicia os atributos
		lastId = typeIdDefault.OBJID;
		changeId = true;
		//actives = new Hashtable();
		register = new BW_Register();
                register.setTypeId(typeIdDefault.BW_REGISTER);
                register.setObjId(typeIdDefault.BW_REGISTER);

		//inicializa grupo default
		group = new BW_Group (typeIdDefault.BW_GROUP,typeIdDefault.BW_GROUP);

		//busca informacoes na base da dados
                //this.recall();
	}

	/** construtor passando o grupo desejado, e setando o registro de eventos
	 * como o default
	 * @see #register
	 * @param 	BW_Group grp	grupo a ser usado
	 */
	public CL_Monitor (BW_Group grp) {
		//super (typeIdDefault.CL_MONITOR,typeIdDefault.CL_MONITOR);

		//inicia os atributos
		lastId = typeIdDefault.OBJID;
		changeId = true;
		//actives = new Hashtable();
		register = new BW_Register();
                register.setTypeId(typeIdDefault.BW_REGISTER);
                register.setObjId(typeIdDefault.BW_REGISTER);

		this.group = grp;

		//busca informacoes na base da dados
                //this.recall();
	}



       /** finaliza as operacoes */
	/*protected void finalize () throws Throwable {
         	this.saveall();
         	//super.finalize();
	} */

	
    /////////////////////////////// METODOS //////////////////


    /////////////////// ACESSO A BASE DE DADOS	

	/** resgata informacoes da base de dados */
	public void recall () {
	
		//LE OBJETO REGISTER ARMAZENADO
		BW_Register newR = this.storage.read (register);
		if (newR!=null)
			register = newR;
		
		//LE OBJETO GRUPO DEFAULT ARMAZENADO
		BW_Group newG = this.storage.read (group);
		if (newG != null)
			group = newG;		
			
         	//System.out.println ("[DBG]CL_Monitor.recall: loading information "+register+"/"+group.getName());
        }

	
	/** envia para a base de dados todas as informacoes */
	public void saveall () {
	
		System.out.println ("[DBG]CL_Monitor.saveall: saving information");
         	//SALVA REGISTRO
		this.storage.save (register);
		//SALVA GRUPO
		this.storage.save (group);
		
	}

	
        /** salva a ocorrencia de um evento na base de dados
         * @param	BW_Event evt	evento a ser enviado a base de dados
         * @return	boolean    	true se a operacao ocorreu com sucesso
         */
	protected boolean saveEvent (BW_Event evt) {
         	//SALVA O EVENTO OCORRIDO
         	return this.storage.save (evt);
	}

        /** salva a ocorrencia de um conjunto de evento na base de dados
         * @param	Vector evts	vetor de BW_Events a serem enviados a base de dados
         * @return	boolean  	true se a operacao ocorreu com sucesso
         */
	protected boolean saveEvent (Vector evts) {		
         	//SALVA OS EVENTOS OCORRIDOS
         	return this.storage.save (evts);
	}
	
	

	///////////////////////// REGISTRO E EVENTOS
	

	/** registra um novo tipo de evento
	 * @param 	BW_Event newevt	novo tipo de evento
	 / @return 	boolean true se fez a insercao
	 */
	public boolean eventRegister (BW_Event newevt) {
		//if (changeId) newevt.setObjId(lastId++);
        	return register.insertEvent (newevt);
	}


	/** remove do registro um tipo de evento
	 * @param 	BW_Event evt	tipo de evento
	 * @return 	boolean true se ermoevu o tipo de evento
	 */
	public boolean eventRemove (BW_Event evt) {
		return register.removeEvent (evt);
	}
		
			
	/** alerta quanto a ocorrencia de um evento especifico.
	 * @param 	BW_Event evt	evento ocorrido
	 * @return	boolean		true, se o registro da ocorrencia do evento
	 *				      ocorreu sem problemas;
	 *				false, se o tipo de evento nao estava registrado,
	 *				       ou se nao foi possivel armazenar sua ocorrencia.
	 */
	public boolean eventOccurrence (BW_Event evt) {
		boolean ret = false;
         	if (register.verifyEvent (evt)) {
         		if (changeId) evt.setObjId(lastId++);
			ret = this.saveEvent(evt);
		}	
		return ret;
	}

	
        /** Informa sobre a ocorrencia de diversos eventos em uma unica chamada.
	 * @param	Vector evts	Vetor de BW_Events ocorridos
	 * @return	boolean 	true se a ocorrencia destes eventos foi registrada
	 *				sem problemas, false caso contrario.
         */
        public boolean eventOccurence (Vector evts) {
        	//informa a ocorrencia dos eventos
      		boolean ret = false;
      		Vector evts_ok = new Vector();
      		
      		//percorre o vetor, separa so os eventos registrados
      		for (int i=0;i<evts.size();i++) {
      			BW_Event event = (BW_Event) evts.elementAt(i);
      			if (register.verifyEvent (event)) {
      				if (changeId) event.setObjId(lastId++);
      				evts_ok.addElement (event);
      			}
      			
      		}
      		
      		//envia para a base de dados os OK
      		if (!evts_ok.isEmpty())
	      		ret = this.saveEvent (evts_ok);
	      		
      		return ret;
        }


	//////////////////////////// MEMBROS
	
	/** avisa o monitor sobre o ingresso de um usuario
	 * especifico no sistema
	 * @param	int uid	identificador do usuario
  	 * @return 	BW_Member objeto sobre o usuario ou
	 *		null se o membro nao pertencer ao grupo
	 */
	public BW_Member memberLogin(int uid) {
	    BW_Member mb = null;	
	    mb = awareness.memberLogin (uid);
	    return mb;
	}


	/** avisa o monitor que um usuario deixou o sistema
	 * @param 	BW_Member memb	membro q deixou o sistema
	 */
	public void memberLogout (BW_Member memb) {
		awareness.memberLogout (memb);
	}
	
        /** adiciona um novo membro ao grupo
         * @param 	BW_Member newmb	novo membro do grupo
	 * @return	boolean true se a insercao ocorreu sem problema
	 */
	public boolean addMember (BW_Member newmb) {
		return (group.insertMember (newmb));	
	}
	
        /** remove um membro do grupo
         * @param 	BW_Member memb	membro do grupo a ser removido
	 */
	public void removeMember (BW_Member memb) {
		group.removeMember (memb);	
	}


	////////////////////// METODOS "GET"S	
	
	/** retorna a lista dos membros ativos classificados por uid
	 *  @return Hashtable	membros ativos
	 */
	public Hashtable getActives () {
         	return awareness.getActiveUsers();
	}

	/** retorna o ultimo identificador de objeto usado
	 * @return	int	ultimo object id usado
	 */
	public int getLastId () {
         	return lastId;
	}

	/** retorna o grupo ativo no momento
	  * @return BW_Groupgrupo ativo
	 */
	public BW_Group getGroup () {
         	return group;
	}

	/** Retorna o registro de eventos utilizado
	 * @return	BW_Register	 objeto com a lista de eventos registrados
	 */
	public BW_Register getRegister () {
		return this.register;
	}

	
	
	///////////////////// METODOS "SET"S
	
	/** Altera o grupo ativo no momento.
	 * Este grupo sera atualizado com relacao ao que esta armazenado na base de dados.
	 * 	
	 * @param BW_Group newgrp 	novo grupo ativo
	 */
	public void setGroup (BW_Group newgrp) {
		//salva antes o grupo
		this.storage.save(group);
		//seta para o novo
         	this.group = newgrp;
         	//busca as informacoes do novo grupo
         	BW_Group grpR = this.storage.read (group);
         	if (grpR!=null)
         		this.group = grpR;
	}

	/** Seta um novo objeto de registro para a camada de controle.
	 * Este objeto sera atualizado com relacao ao que esta armazenado na base de dados
	 * @param BW_Register regist	novo registro
	 */
	public void setRegister (BW_Register regist) {
		//salva antes
		this.storage.save (register);
		//seta
		this.register = regist;
		//le e se nao nulo, seta
		BW_Register rgtR = this.storage.read (register);
		if (rgtR!=null)
			this.register = rgtR;
	}
	
	/** ativa ou desativa o controle dos Identificadores
	 * de objetos para os eventos
	 * @param	boolean active	true ativa o controle	
	 */
	public void setChangeId (boolean active) {
         	this.changeId = active;
	}
	
       /** Seta o proximo identificador de objeto disponivel
        * @param	int lastId	proximo identificador de objeto a ser usado
        */
        public void setLastId (int last) {
        	this.lastId = last;
        }

        /** seta a conexao com a camada de armazenamento */
        public void setFCSTMonitor (FC_ST_Monitor mon) {
		this.storage = mon;
        }

        /** seta a conexao com a classe responsavel pela contextualizacao */
        public void setCLAwareness (CL_Awareness aware) {
        	this.awareness = aware;
        }

        	
	///////////////////////// PAPEIS
	
	/** inclui um novo papel no grupo
	 * @param  BW_Paper newpap	novo papel incluido
	 */
	public void addPaper (BW_Paper newpap) {
		this.group.insertPaper (newpap);
	}
	
	/** remove um papel do grupo
	 * @param BW_Paper pap	papel a ser removido
	 * @see	BW.kernel.BW_Group#removePaper
	 */
	public void removePaper (BW_Paper pap) {
		this.group.removePaper (pap);	
	}
	
	
	////////////////////////////
	
	/** termina suas atividades */
	public void terminate () {
		try {
			//this.finalize();
			this.saveall();
		} catch (Throwable t) {
			System.out.println ("problemas...");
		}	
	}
	
}
