/**
 * UFRGS - II - PPGC - MESTRADO EM CIENCIA DA COMPUTACAO
 * AUTORA MANUELE KIRSCH PINHEIRO
 * CLASSE FC_CL_Context
 * PACOTE CONTROL
 * DATA DE CRIACAO 09 / 01 / 2001
 */

package BW.control;

import BW.kernel.*;
import BW.control.*;
import BW.Interface.UI_Interface;


//import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;



/**
 * A classe FC_CL_Context serve de fachada para contextualizacao
 * da camada de controle (pacote control) do framework BW.
 * Atraves desta classe que se da a comunicacao entre a interface
 * com o usuario e o framework BW.
 * @author Manuele Kirsch Pinheiro
 * @version 1.0
 */
public class FC_CL_Context {

      ////////////////////////// ATRIBUTOS ////////////////////////////

      /** Gerente de contextualizacao da camada de controle */
      CL_Awareness awareness;

      /** gerenet de apresentacao da camada de interface */
      UI_Interface ui;

      ///////////////////////// CONTRUTOR ////////////////////////////

      public FC_CL_Context () {
      		awareness=null;
      		ui = null;
      }


      ///////////////////////// METODOS //////////////////////////////

       /** seta o gerente de contextualizacao da camada de controle */
       public void setCLAwareness (CL_Awareness aware) {
       		this.awareness = aware;
       }

       /** seta o gerente de apresentacao da camada de interface */
       public void setUI (UI_Interface ui) {
       		this.ui = ui;
       }

       /** retorna o gerente de contextualizacao da camada de controle */
       public CL_Awareness  getCLAwareness () {
       		return this.awareness;
       }

       /** retorna o gerente de apresentacao da camada de interface */
       public UI_Interface getUI () {
       		return this.ui;
       }

       /** informa sobre a entrada de um usuario no sistema
       	* @param	BW_Member memb	membro que ingressou no sistema
       	* @return	BW_Member	informacoes atualizadas sobre este
       	*				mesmo membro. Null se o membro nao
       	*				existe.
       	*/
       public BW_Member memberLogin (BW_Member memb) {
       	    BW_Member ret = memb;
       	    try {
        	ret = this.awareness.memberLogin (memb.getObjId());
            } catch (Exception e) {
                ret = null;
            }	
            return ret;
       }

       /** informa sobre a saida de um usuario do sistema */
       public void memberLogout (BW_Member memb) {
            try {
            	if (memb!=null)
            		this.awareness.memberLogout(memb);
            } catch (Exception e) {

            }
       }

       /** retorna os usuarios vistos como ativos
        * @return Hashtable	hash no formato (uid,member) com os membros vistos como ativos
        */
       public Hashtable getActiveUsers () {
       		return this.awareness.getActiveUsers();
       }

       /** Retorna a informacao atualizada de um membro qualquer do grupo
        * @param	BW_Member memb	objeto member com o UID e o tipo do usuario que
        *				se esta procurando informacoes
        * @return	BW_Member	objeto member com informacoes atualizadas
        */
       public BW_Member getMemberInfo (BW_Member memb) {       		
       	   BW_Member ret = null;
       	   try {
       	   	if (memb!=null)
       	   		ret = this.awareness.getMemberInfo (memb);
       	   } catch (Exception e) {
       	
       	   }
       	   return ret;
       }

       /** atualiza junto ao grupo as informacoes do membro
        *  @param	BW_Member	membro com as informacoes atualizadas
        */
       public void setMemberInfo (BW_Member memb) {
       		try {
       			this.awareness.setMemberInfo (memb);
       		}catch (Exception e) {
       		
       		}
       }

       /** retorna o objeto representativo do grupo ativo
        * @return	BW_Group	grupo atual
        */
       public BW_Group getGroup () {
       		BW_Group grp = null;
        	try {
        		grp = this.awareness.getGroup();
        	}catch (Exception e) {
        	
        	}
        	return grp;
       }


       /** Solicita os eventos para a contextualizacao de um usuario.
        * Baseado nas informacoes dos profiles do usuario, so serao retornados
        * aqueles eventos ditos de interesse do usuario.
        * @param	BW_Member memb	membro que se deseja contextualizar
        * @return	Vector		vetor de BW_Event com todos os eventos
        *				que ocorreram, dentro dos interesses
        *				do usuario.
        */
       public Vector getEvents (BW_Member memb) {
       		Vector ret = null;
       		try {       			
       			ret = this.awareness.getEvents (memb);
       		} catch (Exception e) {
       		
       		}
       		return ret;
       }

       /** Notifica a camada de interface sobre novos eventos ocorridos
        * E feito um repasse da notificacao recebida da camada de controle
        * para a controle de interface, alertando que novas informacoes
        * estao disponiveis.
        */
       public void Notify () {
	  try {
	  	ui.Notify();
	  } catch (Exception npe) {}
	
       }


  	/** termina suas atividades */
	public void terminate () {
		System.out.println ("[DBG]FC_CL_Context.terminate: finalizing activities");
		this.awareness.terminate();
	}
}


