/**
 * UFRGS - II - PPGC - MESTRADO EM CIENCIA DA COMPUTACAO
 * AUTORA MANUELE KIRSCH PINHEIRO
 * CLASSE BW_Member
 * PACOTE BW.KERNEL
 * DATA DE CRIACAO 14 / 07 / 2000
 */

package BW.kernel;

import java.util.Hashtable;
import java.util.Enumeration;

import BW.typeIdDefault;

/**
 * classe BW_Member representa a ideia de membro do
 * grupo dentro do BW
 * @author Manuele K P
 * @version 1.2
 */
public class BW_Member extends BW_Element {

      ///////////////////////////////// ATRIBUTOS //////////////

	/** nome do membro */
	String name;

	/** login */
	String login;

	/** endereco de mail para contato */
	String mail;

	/** pagina pessoal */
	String homepage;

	/** maquina onde ocorreu o ultimo login */
	String machine;

	/** intervalo de tempo do ultimo login do usuario
	 * Atencao! Pode ser null
	 */
	BW_TimeInterval lastlogin;

	
	
	/** conjunto de papeis em potencial que podem ser desempenhados pelo membro,
	 * disposto no formato (nome do papel, objeto BW_Paper)
	 */
	Hashtable papers;
	
	/** papel ativo no momento
	 * Atencao! pode ser null
	 */
	BW_Paper activepaper;
	
	/** Awareness profile do usuario - reune todos os profiles pessoais e
	 * por papel do usuario.
	 * E este atributo que mantem todos os profiles pessoais por papel do
	 * membro. Um set descuidado neste parametro podera tornar o objeto
	 * inconsistente.
	 * @see BW_AwarenessProfile
	 */
	 BW_AwarenessProfile profiles;
	


      //////////////////////////////// CONSTRUTORES ////////////

	/** construtor recebe apenas o login do
	 *  usuario e seu uid (identificador numerico, que
	 *  servira de id deste objeto
	 * @param	String login	login do usuario
	 * @param 	int uid		user id
	 */
	public BW_Member (String login, int uid) {
        	super (typeIdDefault.BW_MEMBER, uid);
		this.login = new String (login);

		//informacoes pessoais adevem ser setadas
		//"a mao" pelos metodos "set", se for de
		//interesse da aplicacao
		this.mail = new String();
		this.homepage = new String();
		this.name = new String();
		this.machine = new String();

		//CUIDADO! NullPoiterException deve ser tratada!
		this.lastlogin = null;
		this.activepaper = null;
		
		this.profiles = new BW_AwarenessProfile ();
		this.papers = new Hashtable();		
	}

	/** construtor recebe apenas o identificador do tipo
	 *  e o uid do usuario (identificador numerico, que
	 *  servira de id deste objeto)
	 * @param	int typeid	identificador de tipo da classe
	 * @param 	int uid		user id
	 */
	public BW_Member (int typeid, int uid) {
        	super (typeid, uid);

		//informacoes pessoais adevem ser setadas
		//"a mao" pelos metodos "set", se for de
		//interesse da aplicacao
		this.login = new String ();
		this.mail = new String();
		this.homepage = new String();
		this.name = new String();
		this.machine = new String();

		//CUIDADO! NullPoiterException deve ser tratada!
		this.lastlogin = null;
		this.activepaper = null;
		
		this.profiles = new BW_AwarenessProfile ();
		this.papers = new Hashtable();		
	}
	
	/** construtor default recebe o login vazio e usa
	 *  um uid (identificador numerico, que
	 *  servira de id deste objeto) default
	 */
	public BW_Member () {
        	super (typeIdDefault.BW_MEMBER, typeIdDefault.BW_MEMBER);
		
		//informacoes pessoais adevem ser setadas
		//"a mao" pelos metodos "set", se for de
		//interesse da aplicacao
		this.login = new String();
		this.mail = new String();
		this.homepage = new String();
		this.name = new String();
		this.machine = new String();

		//CUIDADO! NullPoiterException deve ser tratada!
		this.lastlogin = null;
		this.activepaper = null;
		
		this.profiles = new BW_AwarenessProfile ();
		this.papers = new Hashtable();		
	}
	
	

      /////////////////////////////// METODOS //////////////////
      	
      	/**
      	 * atribui um novo login para o usuario
      	 * @param String newlogin
      	 */
      	public void setLogin (String newlogin) {
      		this.login = new String (newlogin);      	
      	}
      	
	/**
	 * atribui um novo nome para o usuario
	 * @param String newname	novo nome
	 */
        public void setName (String newname) {
		this.name = new String (newname);
	}

	/**
	 * atribui um novo mail para o usuario
	 * @param String newmail	novo mail
	 */
        public void setMail (String newmail) {
        	this.mail = new String (newmail);
	}

	/**
	 * atribui uma nova homepage para o usuario
	 * @param String newhp	nova homepage
	 */
        public void setHomePage (String newhp) {
        	this.homepage = new String (newhp);
	}

	/**
	 * seta a ultima maquina pela qual o usuario
	 * ingressou no sistema
	 * @param String machine	maquina usada
	 */
        public void setMachine (String machine) {
        	this.machine = new String (machine);
	}

	/**
	 * seta a ultima vez (periodo) que o usuario
	 * ingressou no sistema
	 * @param TimeInterval interval	periodo do ultimo login
	 * @see TimeInterval
	 */
        public void setLastLogin (BW_TimeInterval interval) {
              try {
        	this.lastlogin = new BW_TimeInterval (interval);
              } catch (NullPointerException npe) {
              	this.lastlogin = null;
              }
	}

	
	/**
	 * recupera o login do usuario
	 * @return	String login do usuario
	 */
	public String getLogin () {
          return (new String (login));
	}

   	/**
	 * recupera o nome do usuario
	 * @return	String nome do usuario
	 */
	public String getName () {
          return (new String (name));
	}

   	/**
	 * recupera o mail do usuario
	 * @return	String mail do usuario
	 */
	public String getMail () {
          return (new String (mail));
	}


   	/**
	 * recupera o endereco da homepage do usuario
	 * @return	String homepage do usuario
	 */
	public String getHomePage () {
          return (new String (homepage));
	}

   	/**
	 * recupera a maquina onde o usuario logou pela ultima vez
	 * @return	String maquina
	 */
	public String getMachine () {
          return (new String (machine));
	}

   	/**
	 * recupera o ultimo periodo de login do usuario
	 * @return	BW_TimeInterval intervalo de tempo
	 */
	public BW_TimeInterval getLastLogin () {
	  try { 
             return (new BW_TimeInterval (lastlogin));
	  } catch (NullPointerException npe) {
	     return null;
	  }
	}

	
	
	
	////////////////// metodos para manipulacao do AWARENESS PRODFILE
	
	/** retorna o awareness profile do usuario, que descreve todos
	 * seus profiles pessoais e pessoais por papel
	 * @return	BW_AwarenessProfile	profiles do usuario
	 * @see 	BW_AwarenessProfile
	 */
	public BW_AwarenessProfile getAwarenessProfile () {
	 	return (BW_AwarenessProfile)profiles.clone();	
	}	
	
	/** seta o awareness profile do usuario para um
	 * novo objeto
	 * Atencao!! Sempre usar este metodo antes de usar um setPapers!
	 * O Awareness Profile do usuario controla tambem os profiiles
	 * pessoais por papel do membro, e uma manipulacao descuidada
	 * pode torna-lo insconsistente!
	 * @param 	BW_AwarenessProfile newprofile
	 * @see 	BW_AwarenessProfile
	 * @see		#setPapers
	 */
	public void setAwarenessProfile (BW_AwarenessProfile newprofile) {
		this.profiles = (BW_AwarenessProfile)newprofile.clone();
	}
	
	
        /** metodo insertPersonalProfile insere um profile pessoal
         * no conjunto de profiles do membro
         * @param	BW_Profile newprof	novo profile a ser acrescentado
         */
        public void insertPersonalProfile (BW_Profile newprof) {
        	profiles.insertPersonalProfile (newprof);

        }


        /** metodo removePersonalProfile remove um profile pessoal
         * no conjunto de profiles do membro
         * @param	BW_Profile prof	profile a ser removido
         */
        public void removePersonalProfile (BW_Profile prof) {
        	profiles.removePersonalProfile (prof);

        }
	


	////////////////// metodos para manipulacao dos PAPEIS
		
	/** monta a chave para a hash que mantem os papeis do usuario
	 * @see #papers
	 */
	Object mountKey (BW_Paper pap) {
		return new String (pap.getName());	
	}
	
	/** monta a chave para a hash que mantem os papeis do usuario
	 * @see #papers
	 */
	Object mountKey (String pap) {
		return new String (pap);	
	}

	
	/** insere um novo papel, com seu respectivo profile pessoal,
	 * ao conjunto de papeis potenciais do usuario
	 * @param 	BW_Paper newpaper	novo papel potencial do usuario
	 * @param	BW_Profile ppprofile	profile pessoal do papel
	 */			
	public void insertPaper (BW_Paper newpaper, BW_Profile ppprofile) {
		Object key = this.mountKey (newpaper);
		papers.put (key,newpaper);
		profiles.insertPersonalPaperProfile (ppprofile,newpaper);	
	}

		
        /** remove um papel do conjunto de papeis em potencial do membro
         * @param	BW_Paper pap	papel a ser removido
         */
        public void removePaper (BW_Paper pap) {
        	Object key = this.mountKey (pap);
                papers.remove (key);
                profiles.removePersonalPaperProfile (pap);
        }

        /** remove um papel do conjunto de papeis em potencial do membro
         * @param	String papname	nome do papel a ser removido
         */
        public void removePaper (String papname) {
        	Object key = this.mountKey (papname);
                BW_Paper pap = (BW_Paper) papers.remove (key);
                if (pap!=null)
                	profiles.removePersonalPaperProfile (pap);
        }


        /** verifica se um determinado papel esta entre
         * os papeis em potencial do membro
         * @param 	BW_Paper pap	papel a ser verificado
         * @erturn 	boolean	true	se o papel esta no conjunto
         *			false	caso contrario	
         */
        public boolean verifyPaper (BW_Paper pap) {
        	Object key = this.mountKey (pap);
        	return papers.containsKey (key);
        }

        /** verifica se um determinado papel esta entre
         * os papeis em potencial do membro
         * @param 	String papname	nome do papel a ser verificado
         * @erturn 	boolean	true	se o papel esta no conjunto
         *			false	caso contrario	
         */
        public boolean verifyPaper (String papname) {
        	Object key = this.mountKey (papname);
        	return papers.containsKey (key);
        }



	/** retorna o papel, do conjunto de papeis potenciais do
	 * membro, cujo nome foi passado como parametro
	 * @param 	String papername	nome do papel a ser resgatado
	 * @return	BW_Paper	papel correspondente ao nome dado
	 * retorna null caso nao exista no conjunto de papeis do membro
	 * nenhum papel com o nome dado
	 */
	public BW_Paper getPaper (String papername) {
		Object key = this.mountKey (papername);
		return (BW_Paper) papers.get (key);	
	}
	
	
	/** retorna o conjunto completo dos papeis potenciais do membro,
	 * relacionados a seus respectivos profiles pessoais - tupla
	 *(BW_Paper, BW_Profile)
	 * @return	Hashtable hash com os papeis (BW_Paper) e profiles (BW_Profile)
	 */
	public Hashtable getPapers () {
		Hashtable ht = new Hashtable ();
		Enumeration en = papers.elements();
		while (en.hasMoreElements()) {
			BW_Paper pap = (BW_Paper) en.nextElement();
			BW_Profile prof = profiles.getPersonalPaperProfile(pap);
			if (prof!=null)
				ht.put (pap,prof);	
		}	
			
	        return ht;
	}
	
	/** seta o conjunto completo de papeis potenciais do usuario,
	 * ja com seus respectivos profiles pessoais por papel
	 * @param Hashtable pap_prof hash com os papeis relacionados a seus profiles pessoais
	 * @see #getPapers
	 */
	 public void setPapers (Hashtable pap_prof) {
	 	this.papers = new Hashtable();
	 	Enumeration en = pap_prof.keys();
	 	while (en.hasMoreElements()) {
	 		BW_Paper pap = (BW_Paper)en.nextElement();
	 		papers.put (this.mountKey(pap),pap);
	 	}
	 	profiles.setPersonalPaperProfile (pap_prof);	 	
	 }
	

	 	
	 /** retorna o papel que esta sendo desempenhado pelo membro neste
	  * momento, tambem chamado de papel ativo do membro
	  * @return BW_Paper	papel ativo
	  */
	  public BW_Paper getActivePaper () {
	       /*try {
	  	  return (BW_Paper)activepaper.clone();	  		
	  	} catch (NullPointerException npe){
	  	  return null;
	  	}*/
	  	return activepaper;
	  }
	
	 /** seta o papel que esta sendo desempenhado pelo membro neste
	  * momento, tambem chamado de papel ativo do membro
	  * Atencao: se for passado como parametro um Null, este metodo
	  * setara o papel ativo para null e retornara true.
	  * @param BW_Paper active	papel ativo
	  * @return boolean 	true se o papel ativo pode ser setado
	  * 		    	false se o papel especificado nao pertence ao conjunto de papeis do usuario
	  */
	  public boolean setActivePaper (BW_Paper active) {
	  	boolean resp = false;
	  	try {
	  	   if (this.verifyPaper(active)) {
	  		activepaper = (BW_Paper)papers.get (this.mountKey(active));
	  		resp = true;
	  	   }
	  	} catch (NullPointerException npe) {
	  	   activepaper = null;
	  	   //resp = true;
	  	}
	  	return resp;
	  }
	
	
	
	  /** retorna o profile pessoal para um determinado papel
	   * @param	BW_Paper pap 	papel para o qual se quer o profile pessoal por papel
	   * @return	BW_Profile	profile pessoal do papel, null se o papel nao consta
	   *				no conjunto de papeis do membro
	   */
	  public BW_Profile getPersonalPaperProfile (BW_Paper pap) {
	  	BW_Profile prof = null;
	  	if (this.verifyPaper(pap)) {
	  		prof = profiles.getPersonalPaperProfile (pap);
	  	}
	        return prof;
	  }
	
	  /** seta o profile pessoal para o papel passado como parametro.
	   * Retorna false se o membro nao possui o papel passado como parametro	
	   * @param	BW_Paper pap	papel para o qual se deseja setar o profile pessoal
	   * @param	BW_Profile prof	profile pessoal para o papel
	   * @return	boolean	false se o papel nao esta no conjunto de papeis potenciais
	   * 			true se conseguiu setar o profile pessoal do papel
	   */
	  public boolean setPersonalPaperProfile (BW_Paper pap, BW_Profile prof) {
	  	boolean resp = false;
	  	if (this.verifyPaper(pap)) {
	  		profiles.removePersonalPaperProfile (pap);
	  		profiles.insertPersonalPaperProfile (prof,pap);
	  		resp = true;
	  	}
	        return resp;
	  }
	
	
	
	
	///////////////// metodos padrao
	
        /** retorna o valor de todos os campos da variavel
         * Atencao! Nao insere os campos com valor null
         */
	public Hashtable getFields () {
             Hashtable ht = super.getFields ();
                //campos que podem ser vazios, mas nao chegam a ser nulos
	     	ht.put ("login",this.getLogin());
		ht.put ("name",this.getName());
		ht.put ("mail",this.getMail());
		ht.put ("homepage",this.getHomePage());
		ht.put ("machine",this.getMachine());
		
		ht.put ("profiles",this.getAwarenessProfile());
		ht.put ("papers",this.getPapers());
		
		//nao coloca o campo se ele for nulo
		if (activepaper != null) ht.put ("activepaper",this.getActivePaper().clone());
		if (lastlogin != null) ht.put ("lastlogin",this.getLastLogin());
		//else ht.put ("lastlogin","unknown");
		
		return ht;
	}


	/** metodo setFields seta os atributos do cobjeto
	 * com base em uma Hashtable no mesmo formato retornado
	 * pelo metodo getFields. Retorna true se tudo correu
	 * bem, e false caso contrario.
	 * Atencao! Para setar um membro e obrigatorio a presenca dos campos
	 * "login","name","mail","machine","profile" e "papers", e facultativa
	 * a presenca dos campos "lastlogin" e "activepaper". Havendo a presenca
	 * dos campos obrigatorios na hashtable, o objeto sera setado. 
	 * @param	Hashtable fields	atributos a serem preenchidos
	 * @return	boolean	true - se foi possivel ajustar os atribuitos
	 *			false - caso contrario
	 * @see	#getFields
	 */
       public boolean setFields (Hashtable fields) {
		Hashtable bkp = super.getFields();
       	boolean ret = super.setFields (fields);
       		
         	if (ret && fields.containsKey ("login") && fields.containsKey ("name")
       			&& fields.containsKey ("mail") && fields.containsKey ("homepage") && fields.containsKey ("machine")
       			&& fields.containsKey ("profiles") && fields.containsKey ("papers")) {

				//System.out.println ("[DBG]BW_Member.setFields: seting ...");

       			this.setLogin ((String) fields.get ("login"));       					
       			this.setName ((String) fields.get ("name"));
       			this.setMail ((String) fields.get ("mail"));
       			this.setHomePage ((String) fields.get ("homepage"));
       			this.setMachine ((String) fields.get ("machine"));
       			this.setPapers ((Hashtable) fields.get ("papers"));
       			this.setAwarenessProfile ((BW_AwarenessProfile) fields.get ("profiles"));
       			
       			
       			//campos nao obrigatorios
       			if (fields.containsKey ("lastlogin"))
       				this.setLastLogin ((BW_TimeInterval) fields.get ("lastlogin"));
       			else	this.lastlogin = null;
       			
       			if (fields.containsKey ("activepaper"))
       				this.setActivePaper ((BW_Paper) fields.get ("activepaper"));
       			else 	activepaper = null;
       			      			       			       			
       			ret = true;       			
       		}
		else  {
				ret = false;
				super.setFields (bkp);
			}
       		
       		return ret;       	
	}
	
	
	
	
	/**
	* metodo clone retorna uma outra instancia de
	* BW_Member com os mesmos atributos desta
	*/
	public Object clone() {
		BW_Member nm = new BW_Member (this.getLogin(),this.getObjId());
		nm.setName (this.getName());
		nm.setMail (this.getMail());
		nm.setHomePage(this.getHomePage());
		nm.setMachine(this.getMachine());		
		nm.setLastLogin(this.getLastLogin());
		
		nm.setAwarenessProfile (this.getAwarenessProfile());
		nm.setPapers (this.getPapers());
		
		if (this.getActivePaper()!=null)
			nm.setActivePaper (this.getActivePaper());
		
		return (Object)nm;
	}	
	
	
	public String toString () {
   		//pega os campos definidos na superclasse
   		String s = super.getFields().toString();
   		StringBuffer sb = new StringBuffer();
   	
   		//tira o } final       		
       		int i = s.length();
       		char c = s.charAt (i-1);
       		sb.append (s.substring(0,i-1));

   		//vai inserindo os campos
   		sb.append (", Login=");		sb.append (this.getLogin());
   		sb.append (", Name=");		sb.append (this.getName());
   		sb.append (", Mail=");		sb.append (this.getMail());
   		sb.append (", HomePage=");	sb.append (this.getHomePage());
   		sb.append (", Machine=");	sb.append (this.getMachine());
   		sb.append (", Profiles=");	sb.append (this.getAwarenessProfile());
   		sb.append (", Papers=");	sb.append (this.papers);
   		   		
   		if (this.getLastLogin() != null)
	   		sb.append (", LastLogin=");	sb.append (this.getLastLogin());   		
	   		   		
   		sb.append (c);
   		return sb.toString();   		
   		
   }


   /** seta o uid - identificador do usuario
    * @param 	int uid		novo indentificador
    */
   public void setUID (int uid) {
   	this.setObjId(uid);
   }

   /** retorna o uid - identificador do usuario
    * @return	int	uid
    */
   public int getUID() {
   	return this.getObjId();
   }

}
