/**
 * UFRGS - II - PPGC - MESTRADO EM CIENCIA DA COMPUTACAO
 * AUTORA MANUELE KIRSCH PINHEIRO
 * Classe BW_Profile
 * PACOTE BW.KERNEL
 * DATA DE CRIACAO 11 / 11 / 2000
 */

package BW.kernel;

import java.util.Calendar;
import java.util.Hashtable;
import java.util.Enumeration;

import BW.typeIdDefault;

/**
 * Classe BW_Profile: descreve a nocao de profile basico
 * dentro do framework BW
 * @author Manuele Kirsch Pinheiro
 * @version 1.2
 */
public class BW_Profile extends BW_Element {

     ///////////////////////////////// ATRIBUTOS //////////////

     /** intervalo de interesse do profile */
     BW_TimeInterval    interval;

     /** eventos registrados no profile.
      * chave da Hash eh formada pelo nome da classe do evento e o typeId
      * e o objeto armazenado eh o proprio evento
      */
     Hashtable	events;

     ///////////////////////////////// CONSTRUTOR //////////////

     /**
      * construtor padrao - passa os identificadores de tipo e do objeto
      * @param	int typeId	identificador de tipo
      * @param	int objId	identificador do objeto
      */
      public BW_Profile (int typeId, int objId) {
      		super (typeId,objId);      		
      		events = new Hashtable();
      		interval = new BW_TimeInterval(typeIdDefault.BW_TIMEINTERVAL,typeIdDefault.OBJID);
      		Calendar inicio = Calendar.getInstance();
      		inicio.set (1999,0,1,0,0,0);
      		interval.setBegin (inicio);
      }

      /**
       * construtor simples
       */
       public BW_Profile () {
       		super (typeIdDefault.BW_PROFILE,typeIdDefault.OBJID + typeIdDefault.BW_PROFILE + 1);
      		events = new Hashtable();
       		interval = new BW_TimeInterval(typeIdDefault.BW_TIMEINTERVAL,typeIdDefault.OBJID);
      		Calendar inicio = Calendar.getInstance();
      		inicio.set (1999,0,1,0,0,0);
      		interval.setBegin (inicio);      		
       }


     ///////////////////////////////// METODOS //////////////

     /**
      * metodo setInterval ajusta o intervalo de interesse do profile
      * @param BW_TimeInterval newinterval	novo intervalo do profile
      */
      public void setInterval (BW_TimeInterval newinterval) {
      		this.interval = (BW_TimeInterval) newinterval.clone();

      }

      /**
       * metodo setEvents seta todos os eventos registrados no profile
       * de uma unica vez.
       * @param Hashtable Events	eventos para registro no profile
       */
       public void setEvents (Hashtable Events) {
       		this.events = (Hashtable) Events.clone();

       }

      /**
       * metodo getInterval retorna o intervalo de interesse do profile
       * @return	BW_TimeInterval	intervalo de interesse
       */
       public BW_TimeInterval getInterval () {
       		return (BW_TimeInterval)this.interval.clone();
       	
       }

       /**
        * metodo getEvents retorna um vetor com todos os eventos de
        * interesse do profile.
        * @see #mountKey
        * @return 	Hashtable	eventos de interesse do profile
        */
        public Hashtable getEvents () {
        	return (Hashtable) this.events.clone();	
        }


	/** monta a chave usada na hash que mantem os eventos de interesse
	 * A chave e uma string formada pelo nome da
	 * classe do evento mais o sinal de "-" (hifem), mais
	 * o typeId da classe;
	 * e o valor e o proprio typeId do evento
	 */
	public String mountKey (BW_Event obj) {
		   Class classe = obj.getClass();
		   StringBuffer key = new StringBuffer();
		   key.append ((obj.getClass().getName()));
		   key.append("-");
		   key.append(obj.getTypeId());	         	
		   return key.toString();
	}

	/** inclui um tipo de evento para o pool de eventos de interesse,
	 * a partir do objeto passado como parametro.
	 * Retorna true se insercao foi realizada, e false caso contrario
	 * @param 	BW_Event evt	objeto cujo tipo se deseja registrar no profile
	 * @return	boolean 	true - insercao ok
	 */
	public boolean insertEvent (BW_Event evt){
		boolean r=false;
         	try {
		   events.put (this.mountKey(evt),evt);
		   r=true;
		} catch (NullPointerException np) {
                   //caso obj = null, retorna false
		   r=false;
		}
		return r;
	}

	/** verifica se um tipo de evento e de interesse do profile
	 *  @param	BW_Event evt	evento cujo tipo esta ou nao ergistrado
	 *  @return	boolean	 retorna true se o tipo de evento esta registrado
	 */
	public boolean verifyEvent (BW_Event evt) {
	     boolean r=false;
             try {	
		if (events.containsKey(this.mountKey(evt))) {
			r=true;
		}
	     } catch (NullPointerException np) {
              	r=false;
	     }
	     return r;
	}


        /** remove um tipo de evento do profile
	 * @param 	BW_Event evt	evento sendo retirado do profile
	 * @return	boolean		true - tipo foi removido corretamente
	 */
	public boolean removeEvent (BW_Event evt) {
		boolean r=false;
         	try {
                  events.remove (this.mountKey(evt));
		  r=true;
		} catch (NullPointerException np){
			r=false;
		}
		return r;
	}	

	
        public Object clone () {
        	BW_Profile copy = new BW_Profile (this.getTypeId(),this.getObjId());
        	copy.setInterval (this.getInterval());
        	copy.setEvents (this.getEvents());
        	return (Object)copy;

        }


   	/** metodo getFields retorna um Hashtable com os
	 * pares (atributo,valor) do objeto
   	 * @return Hashtable  - atributo,valor
    	 */
	public Hashtable getFields () {
		Hashtable ht = super.getFields();
		ht.put ("events",(Hashtable)events.clone());
		ht.put ("interval",(BW_TimeInterval)interval.clone());
		return ht;			
	}

	
	/** metodo setFields seta os atributos do cobjeto
	 * com base em uma Hashtable no mesmo formato retornado
	 * pelo metodo getFields. Retorna true se tudo correu
	 * bem, e false caso contrario
	 *
	 * @param	Hashtable fields	atributos a serem preenchidos
	 * @return	boolean	true - se foi possivel ajustar os atribuitos
	 *			false - caso contrario
	 * @see	#getFields
	 */
	 public boolean setFields (Hashtable fields) {
		Hashtable bkp = super.getFields ();
	 	boolean ret = super.setFields(fields);
	 	//System.out.println ("[DEBUG] - SUPER.GETFIELDS "+ret);
	 	try {
	 		if (ret && fields.containsKey ("events")  && fields.containsKey("interval")) {
	 			//System.out.println ("[DEBUG] - EVENTS "+fields.get ("events"));
	 			this.setEvents((Hashtable) fields.get ("events"));
	 			//System.out.println ("[DEBUG] - INTERVAL "+fields.get ("interval"));
	 			this.setInterval((BW_TimeInterval) fields.get ("interval"));
	 			ret = true;
	 	
	 		}
	 		else  {
				ret = false;
				super.setFields (bkp);
			}

	 	} catch  (NullPointerException npe){
				ret = false;
				super.setFields (bkp);
	 	}		
	 	
	 	return ret;
	 } 	
	
	 public String toString () {
	  	StringBuffer sb = new StringBuffer();
       		
       		sb.append ("{ ");
       		sb.append ("typeId=");
       		sb.append (this.getTypeId());
       		sb.append (", objId=");
       		sb.append (this.getObjId());
       		sb.append (", Events=[");
       		Enumeration e = this.getEvents().keys();
       		while (e.hasMoreElements()) {
       		    sb.append (e.nextElement().toString());
       		    sb.append (" ");
       		}       		
       		sb.append ("]");
       		
       		sb.append (", Interval={");
       		sb.append (this.getInterval().toString());
       		sb.append ("} }");
       		
	        return sb.toString();
	 }
}
