/**
 * UFRGS - II - PPGC - MESTRADO EM CIENCIA DA COMPUTACAO
 * AUTORA MANUELE KIRSCH PINHEIRO
 * CLASSE BW_TimeInterval
 * PACOTE BW.KERNEL
 * DATA DE CRIACAO 11 / 07 / 2000
 */
package BW.kernel;

import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;
import java.util.Locale;
import java.util.Hashtable;
import java.util.StringTokenizer;

import BW.typeIdDefault;


/**
 * Classe BW_TimeInterval descreve um intervalo de tempo
 * @author Manuele K P
 * @version 1.3
 */
 public class BW_TimeInterval extends BW_Element {

    ///////////////////////////////// ATRIBUTOS //////////////

      /** inicio do intervalo */
      Calendar begin;

      /** final do intervalo */
      Calendar end;

      /** timezone utilizado */
      TimeZone tmz;

      /** locale utilizado */
      Locale lcl;

    //////////////////////////////// CONSTRUTORES ////////////

      /**
       *  construtor - parametro e um outro objeto TimeInterval
       * de onde sao retirados as informacoes. Por default seta
       * o identificador do objeto como sendo o do fonte (passado
       * como parametro) mais 1
       *  @param  TimeInterval source	fonte dos dados
       */
      public BW_TimeInterval (BW_TimeInterval source) {
      	super (source.getTypeId(),source.getObjId()+1);
        tmz = source.getTimeZone();
	lcl = source.getLocale();
	begin = source.getBegin ();
	end = source.getEnd();
      }

      /**
       *  construtor - parametros sao os identificadores do
       *  tipo do objeto e do proprio objeto
       *  @param  int typeId	id do tipo do objeto
       *  @param  int objId	id do objeto	
       */
     public BW_TimeInterval (int typeId, int objId) {
      	super (typeId,objId);
        tmz = TimeZone.getDefault();
	lcl = Locale.getDefault();
	begin = Calendar.getInstance(tmz,lcl);
	end = Calendar.getInstance(tmz,lcl);
      }

      /**
       *  construtor - parametros indicados sao, alem dos
       *  identificadores do tipo do objeto e do proprio objeto,
       *  o timezone e o locale do intervalo.
       *  @param  int typeId	id do tipo do objeto
       *  @param  int objId	id do objeto	
       *  @param  TimeZone tz	timezone a ser usado	
       *  @param  locale lc	locale a ser usado
       */
     public BW_TimeInterval (int typeId, int objId, TimeZone tz, Locale lc) {
      	super (typeId,objId);
        tmz = (TimeZone) tz.clone();
	lcl = (Locale) lc.clone();
	begin = Calendar.getInstance(tmz,lcl);
	end = Calendar.getInstance(tmz,lcl);
      }

     /**
       *  construtor - parametros indicados sao, alem dos
       *  identificadores do tipo do objeto e do proprio objeto,
       *  a data de inicio e a de fim do intervalo.
       *  @param  int typeId	id do tipo do objeto
       *  @param  int objId	id do objeto	
       *  @param  Calendar bg	inicio do intervalo
       *  @param  Calendar ed	final do intervalo
       */
     public BW_TimeInterval (int typeId, int objId, Calendar bg, Calendar ed) {
      	super (typeId,objId);
        tmz = TimeZone.getDefault();
	lcl = Locale.getDefault();
        if (bg.after(ed)) {
	   begin = (Calendar) ed.clone();
	   end = (Calendar) bg.clone();
        } else {
 	   begin = (Calendar) bg.clone();
	   end = (Calendar) ed.clone();
        }
      }

     /**
       *  construtor - parametros indicados sao
       *  a data de inicio e a de fim do intervalo.
       *  @param  Calendar bg	inicio do intervalo
       *  @param  Calendar ed	final do intervalo
       */
     public BW_TimeInterval (Calendar bg, Calendar ed) {
      	super (typeIdDefault.BW_TIMEINTERVAL,(typeIdDefault.BW_TIMEINTERVAL + typeIdDefault.OBJID)* 10);
        tmz = TimeZone.getDefault();
	lcl = Locale.getDefault();
        if (bg.after(ed)) {
	   begin = (Calendar) ed.clone();
	   end = (Calendar) bg.clone();
        } else {
 	   begin = (Calendar) bg.clone();
	   end = (Calendar) ed.clone();
        }
      }


      /**
       *  construtor - sem parametros
       *  so com valores default para o objeto
       */
     public BW_TimeInterval () {
      	super (typeIdDefault.BW_TIMEINTERVAL,(typeIdDefault.BW_TIMEINTERVAL + typeIdDefault.OBJID)* 10);
        tmz = TimeZone.getDefault();
	lcl = Locale.getDefault();
	begin = Calendar.getInstance(tmz,lcl);
	end = Calendar.getInstance(tmz,lcl);
      }



     /////////////////////////////// METODOS ////////////

     /**
      * metodo setBegin altera o valor do inicio do intervalo
      * @param 	Calendar newbegin	novo inicio
      */
     public void setBegin (Calendar newbegin) {
        if (!newbegin.after(this.end)) {
     	  this.begin = (Calendar) newbegin.clone();
        } else {
	  //System.out.println ("EU");
          this.begin = (Calendar) this.end.clone();
 	  this.end = (Calendar) newbegin.clone();
	  
        }
	
     }

     /**
      * metodo setBegin altera o valor do inicio do intervalo,
      * a partir de uma contagem em milisegundos desde 1/1/1970.
      * @param 	long newbegin	novo inicio
      */
     public void setBegin (long newbegin) {
	if (newbegin < this.getEndMillis())
        {
     	  this.begin.setTime(new Date(newbegin));
	  //this.begin.setTimeInMillis (newbegin);
        } else {
          this.begin = (Calendar) this.end.clone();
 	  //this.end.setTimeInMillis(newbegin);
     	  this.end.setTime(new Date(newbegin));
        }
     }

     /**
      * metodo setBegin altera o valor do inicio do intervalo
      * @param 	Date nb		novo inicio
      */
     public void setBegin (Date nb) {
	Calendar newbegin = Calendar.getInstance (tmz,lcl);
	newbegin.setTime (nb);
        if (!newbegin.after(this.end)) {
     	  this.begin = (Calendar) newbegin.clone();
        } else {
          this.begin = (Calendar) this.end.clone();
 	  this.end = (Calendar) newbegin.clone();
        }
     }

     /**
      * metodo setEnd altera o valor do fim do intervalo
      * @param 	Date nf		novo fim
      */
     public void setEnd (Date nf) {
	Calendar newbegin = Calendar.getInstance (tmz,lcl);
	newbegin.setTime (nf);
        if (newbegin.after(this.end)) {
     	  this.end = (Calendar) newbegin.clone();
        } else {
          this.end = (Calendar) this.begin.clone();
 	  this.begin = (Calendar) newbegin.clone();
        }
     }

     /**
      * metodo setEnd altera o valor do fim do intervalo
      * @param 	Calendar newend		novo fim
      */
     public void setEnd (Calendar newend) {
        if (newend.after (this.begin)) {
     	    this.end = (Calendar) newend.clone();
	} else {
	    this.end = (Calendar) this.begin.clone();
            this.begin = (Calendar) newend.clone();
	}
     }

     /**
      * metodo setEnd altera o valor do fim do intervalo
      * a partir de uma contagem em milisegundos desde 1/1/1970.
      * @param 	long newend		novo fim
      */
     public void setEnd (long newend) {
        if (newend > this.getBeginMillis()) {
     	    //this.end.setTimeInMillis(newend);
     	    this.end.setTime(new Date(newend));
	} else {
	    this.end = (Calendar) this.begin.clone();
            //this.begin.setTimeInMillis (newend);
     	    this.begin.setTime(new Date(newend));
	}
     }

     /** metodo setInterval seta o inicio e o fim do intervalo
      * em uma unica chamada.
      * @param Calendar newbg	novo inicio
      * @param Calendar newed	novo fim
      */
     public void setInterval (Calendar newbg, Calendar newed) {
		if (newbg.after(newed)) {
			this.begin = (Calendar) newed.clone();
			this.end = (Calendar) newbg.clone();
		}
		else {
			this.begin = (Calendar) newbg.clone();
			this.end = (Calendar) newed.clone();
		}
     }
     

     /** metodo setInterval seta o inicio e o fim do intervalo
      * em uma unica operacao.
      * @param	long newbg	novo inicio
      * @param	long newed	novo fim
      */
     public void setInterval (long newbg, long newed) {
     	if (newbg<=newed) {
     		this.begin.setTime (new Date (newbg));
     		this.end.setTime (new Date (newed));
     	} else {
     		this.begin.setTime (new Date (newed));
     		this.end.setTime (new Date (newbg));
     	}
     }

     /**
      * metodo setTimeZone altera o valor do TimeZone utilizado
      * @param  TimeZone newtz	novo timezone
      */
     public void setTimeZone (TimeZone newtz) {
        this.tmz = (TimeZone) newtz.clone();
        begin.setTimeZone(tmz);
	end.setTimeZone(tmz);
     }

     /** 
      * metodo setTimeZone altera o valor do Timezone utilizado
      * @param String newtz	novo timezone
      */
      public void setTimeZone (String newtz) {
	TimeZone ntz = TimeZone.getTimeZone (newtz);
	if (ntz != null) {
		tmz = (TimeZone) ntz.clone();
		begin.setTimeZone (tmz);
		end.setTimeZone (tmz);
	}
      }


     /**
      * metodo setLocale altera o valor de Locale utilizado,
      * a partir da saida do metodo toString da classe Locale.
      * Se o valor for invalido nao faz nenhuma alteracao.
      * @param String newlocale		novo Locale
      */
      public void setLocale (String newlocale) {
          StringTokenizer stk = new StringTokenizer (newlocale,"_");
	  Locale newlcl = null;
	  int i = stk.countTokens();
	  if (i==2) newlcl = new Locale ((String)stk.nextElement(),(String)stk.nextElement());
	  else if (i==3) newlcl = new Locale ((String)stk.nextElement(),(String)stk.nextElement(),(String)stk.nextElement());
          else newlcl = this.lcl;

	  lcl = newlcl;
	  Calendar newcal = Calendar.getInstance(this.tmz,this.lcl);
	  newcal.setTime (begin.getTime());
	  begin = (Calendar) newcal.clone();
	  newcal.setTime (end.getTime());
	  end = (Calendar) newcal.clone();
      }

     /** metodo setLocale altera o valor de Locale utilizado.
      * @param Locale newlocale 	novo locale
      */  		 	
     public void setLocale (Locale newlocale) {
      	this.setLocale (newlocale.toString());
     }


     /**
      * metodo getBegin retorna o inicio do intervalo
      * @return  Calendar	inicio do intervalo
      */
     public Calendar getBegin () {
        return ((Calendar) begin.clone());
     }

     /**
      * metodo getEnd retorna o fim do intervalo
      * @return Calendar	fim do intervalo
      */
     public Calendar getEnd () {
        return ((Calendar)end.clone());
     }

   /** metodo getBeginMillis retorna o valor de Begin
    *  em milesegundos contados desde 01/01/1970.
    */
   public long getBeginMillis () {
    	return begin.getTime().getTime();
   }

   /** metodo getEndMillis retorna o valor de End
    *  em milesegundos contados desde 01/01/1970.
    */
   public long getEndMillis () {
    	return end.getTime().getTime();
   }

     /**
      * metodo getTimeZone retorna o TimeZone utilizado
      * @return TimeZone	timezone usado
      */
     public TimeZone getTimeZone () {
      	return ((TimeZone)tmz.clone());
      }

    /**
     * retorna uma String com a identificacao do TimeZone
     * @return	String	Id do TimeZone usado
     */
    public String getTimeZoneString() {
    	return tmz.getID();
    }

    /**
     * retorna a string de identificaca do locale
     * @return	String	locale usado
     */
    public String getLocaleString () {
    	return lcl.toString();
    }

     /**
      * metodo getLocale retorna o Locale utilizado
      * @return Locale	locale utilizado
      */
     public Locale getLocale () {
     	return ((Locale)lcl.clone());
     }

     /**
      * metodo getDuraction retorna a duracao do intervalo
      * dividida nos seguintes campos: YEAR MONTH DAY HOUR MINUTE SECOND,
      * colocados em uma hashtable com seus respectivos valores
      * @return	Hastable	duracao do periodo
      */
     public Hashtable getDuration () {
      	Hashtable ht = new Hashtable();

        int d = (end.get(Calendar.DAY_OF_MONTH) >= begin.get(Calendar.DAY_OF_MONTH))?end.get(Calendar.DAY_OF_MONTH) - begin.get(Calendar.DAY_OF_MONTH):begin.get(Calendar.DAY_OF_MONTH) - end.get(Calendar.DAY_OF_MONTH);
      	int m = (end.get(Calendar.MONTH) >= begin.get(Calendar.MONTH))?end.get(Calendar.MONTH) - begin.get(Calendar.MONTH):begin.get(Calendar.MONTH) - end.get(Calendar.MONTH);
      	int y = (end.get(Calendar.MONTH) >= begin.get(Calendar.MONTH))?end.get(Calendar.YEAR) - begin.get(Calendar.YEAR):begin.get(Calendar.YEAR) - end.get(Calendar.YEAR) ;
      	int mm = (end.get(Calendar.MINUTE) >= begin.get(Calendar.MINUTE))?end.get(Calendar.MINUTE) - begin.get(Calendar.MINUTE):begin.get(Calendar.MINUTE) - end.get(Calendar.MINUTE);
      	int h = (end.get(Calendar.HOUR) >= begin.get(Calendar.HOUR))?end.get(Calendar.HOUR) - begin.get(Calendar.HOUR):begin.get(Calendar.HOUR) - end.get(Calendar.HOUR);
      	int s = (end.get(Calendar.SECOND) >= begin.get(Calendar.SECOND))?end.get(Calendar.SECOND) - begin.get(Calendar.SECOND):begin.get(Calendar.SECOND) - end.get(Calendar.SECOND);

        ht.put ("YEAR",new Integer (y));
        ht.put ("MONTH",new Integer (m));
 	ht.put ("DAY",new Integer (d));
	ht.put ("HOUR",new Integer (h));
	ht.put ("MINUTE",new Integer (mm));
	ht.put ("SECOND",new Integer (s));

	return (ht);
     }

   /** metodo getDuractionMillis retorna a duracao
    * do evento em milesegundos.
    */
    public long getDuractionMillis () {
     	long dur = (this.getEndMillis() > this.getBeginMillis())?this.getEndMillis() - this.getBeginMillis():this.getBeginMillis() - this.getEndMillis();
  	return dur;
    }


   /** metodo getFields retorna um Hashtable com os
    * pares (atributo,valor) do objeto
    * @return Hashtable  - atributo,valor
    */
    public Hashtable getFields () {
	Hashtable ht = super.getFields ();
	ht.put ("bg",this.getBegin());
 	ht.put ("ed",this.getEnd());
	ht.put ("tmz",this.getTimeZone());
	ht.put ("lcl",this.getLocale());
	return (ht);
    }

   /** metodo setFields seta os atributos do objeto
    * a partir de um Hashtable nos mesmos moldes da
    * retornada pelo metodo getFields.
    * @param 	Hashtable fields	hash com os atributos
    * @rerurn	boolean	true - se conseguiu setar os atributos
    *		        false - caso contrario
    * @see #getFields
    */
    public boolean setFields (Hashtable fields) {
	Hashtable bkp = super.getFields();
     	boolean ret = super.setFields (fields);

     	//se o getFields da superclasse retornou ok - segue o processamento
     	if (ret) {
     		//eh imperativo haver os campos "bg" e "ed" para o begin e o end
		if (fields.containsKey("bg") && fields.containsKey("ed")) {
			this.begin = (Calendar)((Calendar) fields.get ("bg")).clone();
			this.end = (Calendar) ((Calendar) fields.get ("ed")).clone();			

			//se nao houverem os campos tmz e lcl, seta pelo default
			if (fields.containsKey("tmz"))
				this.setTimeZone((TimeZone)fields.get ("tmz"));
			else
				this.setTimeZone(TimeZone.getDefault());

			if (fields.containsKey ("lcl"))
				this.setLocale ((Locale) fields.get ("lcl"));
			else
				this.setLocale (Locale.getDefault());
				
			ret = true;
		}
		else  {
				ret = false;
				super.setFields (bkp);
			}	
     	}
     	return ret;
    }


   /** metodo clone retorna uma nova instancia com
    * os mesmos atributos desta, exceto o objId que
    * e acrescido de 1
    */
    public Object clone () {
   	BW_TimeInterval nti = new BW_TimeInterval (this.getTypeId(),this.getObjId(),this.getTimeZone(),this.getLocale());
   	//nti.setBegin (this.getBegin());
	//nti.setEnd (this.getEnd());
	nti.setInterval (this.getBegin(),this.getEnd());
	//System.out.println ("[DEBUG] BW_TimeInterval.clone: clone "+nti);
        return ((Object)nti);
   }

   public String toString () {
        StringBuffer sb = new StringBuffer();

        sb.append (begin.get(Calendar.DAY_OF_MONTH));
        sb.append ("/");
        sb.append (begin.get(Calendar.MONTH));
        sb.append ("/");
        sb.append (begin.get(Calendar.YEAR));
        sb.append ("-");
        //sb.append (begin.get(Calendar.HOUR));
        sb.append (begin.get(Calendar.HOUR_OF_DAY));
        sb.append (":");
        sb.append (begin.get(Calendar.MINUTE));
        sb.append (":");
        sb.append (begin.get(Calendar.SECOND));
        sb.append (":");
        sb.append (begin.get(Calendar.MILLISECOND));

        sb.append (" TO ");
        sb.append (end.get(Calendar.DAY_OF_MONTH));
        sb.append ("/");
        sb.append (end.get(Calendar.MONTH));
        sb.append ("/");
        sb.append (end.get(Calendar.YEAR));
        sb.append ("-");
        //sb.append (end.get(Calendar.HOUR));
        sb.append (end.get(Calendar.HOUR_OF_DAY));
        sb.append (":");
        sb.append (end.get(Calendar.MINUTE));
        sb.append (":");
        sb.append (end.get(Calendar.SECOND));
        sb.append (":");
        sb.append (end.get(Calendar.MILLISECOND));	
	
	sb.append (" TMZ ");
	sb.append (tmz.getID());
	sb.append (" LCL ");
	sb.append (lcl.toString());
        return sb.toString();
	
  }

  /** metodo allanOperations realiza a operacao de Allen passada por parametro
   * com o segundo intervalo tb passado por parametro
   * As operacoes possiveis sao:
   *	"<"	-	this before S		">"	- 	S before this
   * 	"o"	-       this overlaps S		"oi"	-	S overlaps this
   * 	"m"	-       this meets S		"mi"	-	S meets this
   * 	"d"	-       this during S		"di"	-	S during this
   * 	"s"	-       this starts S		"si"	-	S starts this
   * 	"f"	-       this finishes S		"fi"	-	S finishes this
   * 	"="	-       this equals S		
   * @param 	String oper	operacao de Allen desejada.
   * @param	BW_TimeInterval	S	segundo intervalo para comparacao
   * @return	boolean	true	operacao e verdadeira
   *			false	operacao e falsa
   */
   public boolean allanOperations (String oper, BW_TimeInterval S) {
   	//operacao =
       if (oper.equals ("=")) {
        	return this.equals(S);
       }

       //operacao "<"
       if (oper.equals ("<")) {
       		return this.before (S);
       }
      //operacao ">"
       if (oper.equals (">")) {
       		return S.before (this);
       }

 	//operacao "o"
 	if (oper.equals ("o")) {
 		return this.overlaps (S);
 	}
 	//operacao "oi"
 	if (oper.equals ("oi")) {
 		return S.overlaps (this);
 	}
 	
   	//operacao "m"
   	if (oper.equals ("m")) {
   		return this.meets (S);
   	}
   	//operacao "mi"
   	if (oper.equals ("mi")) {
   		return S.meets (this);
   	}
   	
   	//operacao "d"
   	if (oper.equals ("d")) {
   		return this.during (S);
   	}
   	//operacao "di"
   	if (oper.equals ("di")) {
   		return S.during (this);
   	}

	//operacao "s"
   	if (oper.equals ("s")) {
   		return this.starts (S);
   	}
   	//operacao "si"
   	if (oper.equals ("si")) {
   		return S.starts (this);
   	}

   	//operacao "f"
   	if (oper.equals ("f")) {
   		return this.finishes (S);
   	}
   	//operacao "fi"
   	if (oper.equals ("fi")) {
   		return S.finishes (this);
   	}
   	
   	return false;
   }


   public boolean equals (BW_TimeInterval S) {
   	Calendar S_bg = S.getBegin();
   	Calendar S_ed = S.getEnd();
   	if (this.begin.equals (S_bg) && this.end.equals (S_ed))
   		return true;
   	else 	return false;
   }

   public boolean before (BW_TimeInterval S) {
   	Calendar S_bg = S.getBegin();
   	if (this.end.before (S_bg))
   		return true;
   	else 	return false;
   }

   public boolean overlaps (BW_TimeInterval S) {
        Calendar S_ed = S.getEnd ();
   	Calendar S_bg = S.getBegin();   	
   	if (this.begin.before (S_bg) && this.end.before (S_ed) && this.end.after (S_bg))
   		return true;
   	else 	return false;
   }

   public boolean meets (BW_TimeInterval S) {
   	Calendar S_bg = S.getBegin();
   	if (this.end.equals (S_bg))
   		return true;
   	else	return false;

   }

   public boolean during (BW_TimeInterval S) {
   	Calendar S_bg = S.getBegin();
   	Calendar S_ed = S.getEnd ();
   	if ( (this.begin.after (S_bg) && ( this.end.equals(S_ed) || this.end.before (S_ed) ) ) ||
   				( (this.begin.after (S_bg) || this.begin.equals(S_bg)) && this.end.before (S_ed) ) )
   		return true;
   	else	return false;

   }

   public boolean starts (BW_TimeInterval S) {
   	Calendar S_bg = S.getBegin ();
   	Calendar S_ed = S.getEnd ();   	
   	if ( this.begin.equals (S_bg) && this.end.before (S_ed))
   		return true;
   	else	return false;

   }

   public boolean finishes (BW_TimeInterval S) {
   	Calendar S_bg = S.getBegin ();
   	Calendar S_ed = S.getEnd ();   	
   	if ( this.begin.after (S_bg) && this.end.equals (S_ed))
   		return true;
   	else	return false;

   }



   /** metodo merge retorna um novo intervalo de tempo com englobando o
    * intervalo deste objeto e o passado como parametro.
    * ATENCAO: este metodo assume que os timezones e locales ja estao setados
    * @param BW_TimeInterval interval	intervalo a ser agrupado
    * @return BW_TimeInterval	intervalo resultante
    */
   public BW_TimeInterval merge (BW_TimeInterval interval) {
   	//BW_TimeInterval it = (BW_TimeInterval)interval.clone();
   	BW_TimeInterval it = interval;
   	//nao adianta nada - o Java nao altera nada - eh so um indicativo
   	//it.setTimeZone (this.getTimeZone());
   	//it.setLocale (this.getLocale());
   	
   	Calendar bg,bg1,bg2;
   	Calendar ed,ed1,ed2;
   	
   	bg1 = this.getBegin();
   	bg2 = it.getBegin();
   	bg = (bg1.before(bg2) || bg1.equals(bg2)) ? bg1 : bg2;
   	
   	ed1 = this.getEnd ();
   	ed2 = it.getEnd();
   	ed = (ed1.after(ed2) || ed1.equals(ed2)) ? ed1 : ed2;
   	
   	BW_TimeInterval nti = new BW_TimeInterval (this.getTypeId(),this.getObjId(),this.getTimeZone(),this.getLocale());
   	//nti.setBegin (bg);
   	//nti.setEnd (ed);
        nti.setInterval (bg,ed);

        //System.out.println ("[DBG]BW_TimeInterval.merge: merging "+this.toString()+" and "+it+" is "+nti);

   	return nti;
   }
}

