/**
 * UFRGS - II - PPGC - MESTRADO EM CIENCIA DA COMPUTACAO
 * AUTORA MANUELE KIRSCH PINHEIRO
 * CLASSE ST_DATABASEIMPLEMENTOR
 * PACOTE BW.STORAGE
 * DATA DE CRIACAO 17 / 01 / 2001
 * DATA DE ALTERACAO 16/ 05 / 2001
 */

package BW.storage;

import BW.kernel.*;
import BW.storage.*;
import BW.typeIdDefault;

import java.sql.*;
import java.util.*;
import java.net.URL;


/**
 * classe ST_DatabaseImplementor implementa o
 * armazenamento dos dados sobre uma base em
 * PostgreSQL, utilizando acesso via JDBC
 * @autor Manuele K P
 * @version 2.0
 */
public class ST_DatabaseImplementor extends ST_Implementor {

   /////////////////////////////// ATRIBUTOS ///////////////////////////////

   	//String query;
   	//Statement stmt;
   	//PreparedStatement  pstmt;
   	//ResultSet  results;
   	
   	Connection con;   	
   	
   	String url;
	String dbname;
	String host;
        String port;
	String user;
	String passwd;
	
	/** indica se a conexao com o servidor esta aberta */
	boolean status;
	
   //////////
   //////////////////// CONTRUTOR ///////////////////////////////

/* ------------------------------------------------------------------------- */

   /** Construtor padrao. Recebe o host onde se localiza a base como parametro
    * @param 	String host	servidor da base de dados
    */
   public ST_DatabaseImplementor (String host) {
   	this.host = new String (host);
   	this.url= new String("jdbc:postgresql:");
        this.dbname = new String ("bw");
        this.port="";
        this.user="BW";
        this.passwd="";

        url += "//";
        url += host;
        url+= "/";
        url += dbname;

        initDB (url,user,passwd);
   }

/* ------------------------------------------------------------------------- */

   /** Construtor recebe o host, onde se localiza a base, e o user
    * e a senha com a qual deve conectar.
    * @param 	String host	servidor da base de dados
    * @param	String user	nome do usuario
    * @param	String passwd	senha do usuario
    */
   public ST_DatabaseImplementor (String host, String user, String passwd) {
   	this.host = new String (host);
   	this.url = new String("jdbc:postgresql:");
        this.dbname = new String ("bw");
        this.port="";
        this.user = new String (user);
        this.passwd = new String (passwd);

        url += "//";
        url += host;
        url += "/";
        url += dbname;

        initDB (url,user,passwd);
   }

/* ------------------------------------------------------------------------- */

   /** Construtor recebe o host e porta, onde se localiza a base, e o user
    * e a senha com a qual deve conectar.
    * @param 	String host	servidor da base de dados
    * @param	String port	porta do servidor
    * @param	String base	nome da base de dados
    * @param	String user	nome do usuario
    * @param	String passwd	senha do usuario
    */
   public ST_DatabaseImplementor (String host, String port, String base, String user, String passwd) {
   	this.host = new String (host);
   	this.url = new String("jdbc:postgresql:");
        this.dbname = new String (base);
        this.port = new String (port);
        this.user = new String (user);
        this.passwd = new String (passwd);

        url += "//";
        url += host;
        url += ":";
        url += port;
        url += "/";
        url += dbname;

        initDB (url,user,passwd);
   }

 /* ------------------------------------------------------------------------- */



   /////////////////////////////// MANIPULACAO DA BASE DE DADOS ////////////


/* ------------------------------------------------------------------------- */

  /**
   * Metodo initDB inicializa a conexao com o Banco de dados
   * @param	String	url	URL de acesso ao Banco de Dados
   * @param	String 	user	nome do usuario na base da dados
   * @param	String 	passwd	senha a ser usada para conexao
   */
   public void initDB(String url, String user, String passwd) {
       	try {
                    //Le o Drive para o Postgres
                    Class.forName("postgresql.Driver");

                    //tenta conexao
		    System.out.println ("Conecting to "+url+" as "+user+":"+passwd);	
                    this.con = DriverManager.getConnection (url, user , passwd);

                    System.out.println ("Conected");
                    this.status = true;

         } catch (SQLException ex) {
          	 this.status = false;
          	 this.catchSQLException("initDB",ex);		
	} catch (java.lang.Exception ex) {			
          	 this.status = false;          	
          	 this.catchException("initDB",ex);		          	
	}
	 		
   } //initDB

 /* ------------------------------------------------------------------------- */

   public boolean _close () {
      boolean ret = false;
      try {
   	//conclui as operacoes ainda pendentes
	this.con.commit();
	
	this.showWarnings ("_close");
	
        //fecha a conexao
        this.con.close();

        ret = true;

     } catch (SQLException ex) {
        ret = false;
        this.catchSQLException("_close",ex);		
     } catch (java.lang.Exception ex) {			         	
        ret = false;
        this.catchException("_close",ex);
    }


     this.status = false;
     return ret;

   }

/* ------------------------------------------------------------------------- */

   ///////////////////////////// METODOS PARA CADA TIPO DE DADO ////////////////////////


   //////////////// para os intervalos de tempo -----------------------------------

/* ------------------------------------------------------------------------- */

   /** faz a leitura do inicio do intervalo e retorna seu valor em segundos
    * contados desde 1/1/1970.
    * @param	BW_Param interval	objeto de onde serao lidos os dados
    * @return	long	inicio do intervalo, em segundos
    */
   protected long readBgTimeInterval (BW_TimeInterval interval) {
      return interval.getBeginMillis()/1000;
   }

/* ------------------------------------------------------------------------- */

   /** faz a leitura do fim do intervalo e retorna seu valor em segundos
    * contados desde 1/1/1970.
    * @param	BW_Param interval	objeto de onde serao lidos os dados
    * @return	long	fim do intervalo, em segundos
    */
   protected long readEdTimeInterval (BW_TimeInterval interval) {
      return interval.getEndMillis()/1000;
   }


/* ------------------------------------------------------------------------- */

   /** faz a leitura do timezone do intervalo
    * @param	BW_Param interval	objeto de onde serao lidos os dados
    * @return	String	 string com o timezone do intervalo
    */
   protected String readTmzTimeInterval (BW_TimeInterval interval) {
       return interval.getTimeZoneString();
   }


/* ------------------------------------------------------------------------- */

   /** faz a leitura do locale do intervalo
    * @param	BW_Param interval	objeto de onde serao lidos os dados
    * @return	String	 string com o locale do intervalo
    */
   protected String readLclTimeInterval (BW_TimeInterval interval) {
       return interval.getLocaleString();
   }


/* ------------------------------------------------------------------------- */

   /** le uma entidade BW_TimeInterval do ResultSet passado por parametro
    *  @param	ResultSet source	fonte dos dados
    *  @return	BW_TimeInterval		objeto lido
    */
   protected BW_TimeInterval readTimeInterval (ResultSet source) throws SQLException {
    	//tempo eh armazenado em SEGUNDOS
   	long lbg = source.getLong ("bg") * 1000;
   	long led = source.getLong ("ed") * 1000;
   	String tmz = source.getString ("tmz");
   	String lcl = source.getString ("lcl");
   	
   	BW_TimeInterval lx = new BW_TimeInterval();
   	if (!lcl.equals(""))lx.setLocale(lcl);
   	else lx.setLocale(Locale.getDefault());
   	if (!tmz.equals(""))lx.setTimeZone(tmz);
   	else lx.setTimeZone (TimeZone.getDefault());
   	
   	BW_TimeInterval ret = new BW_TimeInterval (typeIdDefault.BW_TIMEINTERVAL,
   						   typeIdDefault.BW_TIMEINTERVAL,
   						   lx.getTimeZone(),
   						   lx.getLocale());
   	//ret.setBegin (lbg);
   	//ret.setEnd (led);
   	ret.setInterval (lbg,led);
   	//System.out.println ("[DBG] ST_DatabaseImplementor.readTimeInterval: readind interval "+ret);
   	return ret;
   }


/* ------------------------------------------------------------------------- */


   //////////////// para os eventos -----------------------------------------------


/* ------------------------------------------------------------------------- */

	/** faz a leitura dos eventos, seguindo uma determinada String SQL
	 * @param	String SQLText
	 * @return	Vector 	vetor de BW_Event lidos
	 */
	public Vector readEvents (String SQLtext) {
		Vector ret = null;		
		try {		
    			//garantindo seguranca?
    			con.setAutoCommit (false);	

			Statement stmt = con.createStatement();			
			ResultSet results = stmt.executeQuery (SQLtext);
			this.showWarnings ("readEvents");    			
    					
  			Vector buf = new Vector();    						
			boolean more = results.next ();											
			while (more) {
				int typeid = results.getInt ("typeid");
				int objid = results.getInt("objid");
				String name = results.getString ("name");
				String description = results.getString ("description");
				String details = results.getString ("details");
				
				BW_TimeInterval ti = this.readTimeInterval(results);
				
				int mtypeid = results.getInt ("member_typeid");
				int mobjid = results.getInt ("member_objid");	
				BW_Member mb = new BW_Member (mtypeid,mobjid);				
				//LEVA MUITO TEMPO LENDO TODO O MEMBRO!!
				//LE SO O NOME E O LOGIN QUE SAO OS DADOS IMPORTANTES PARA OS EVENTOS!								
				//mb.setTypeId(mtypeid);
				//mb = this.readMember (mb,mobjid);
    				Vector vmember = this.read ("SELECT login,name FROM t_member "+
    								" WHERE typeid = "+mtypeid+
    								" and objid = "+mobjid );
    				Hashtable hmember = (Hashtable) vmember.firstElement();
    				String mlogin = (String) hmember.get("login");
    				if (mlogin!=null)  mb.setLogin (mlogin);
    				String mname  = (String) hmember.get ("name");
    				if (mname!=null)   mb.setName(mname);
								
				BW_Event evt = new BW_Event (typeid,objid);
				evt.setName (name);
				evt.setDescription (description);
				evt.setDetails (details);
				if (ti!=null) evt.setInterval (ti);
				if (mb!=null) evt.setResponsible (mb);
				
				buf.addElement (evt);
				
    				//passa para o proximo registro
    				 more = results.next ();    				    				
    			} 	
    		    			
    		  //garantindo seguranca??
    		  con.commit();
   		  con.setAutoCommit (true); 	
   		
   		  Enumeration en = buf.elements();   		
   		  ret = new Vector();
   		  while (en.hasMoreElements()) {
   		  	ret.addElement (en.nextElement());
   		  }
		System.out.println ("[DEBUG]ST_DatabaseImplementor.readEvent: events reads"+ret.size());
   		
    		} catch ( SQLException ex ) {
         		this.catchSQLException("readEvents",ex);
     		} catch (java.lang.Exception ex) {	
     			this.catchException("readEvents",ex);		         	
    		}          	

      		    		
    		return ret;    						
	} 	

/* ------------------------------------------------------------------------- */
	
       /** faz a leitura dos eventos de um certo tipo, dentro de um periodo
        *  na base de dados.
	* @param	BW_Event what		tipo de evento a ser lido
	* @param	BW_TimeInterval when	intervalo a ser lido
	* @return 	Vector	vetor com os eventos lidos
    	*/
    	public Vector readEvents (BW_Event what, BW_TimeInterval when) {
    		Vector ret = null;
    		    		
    		String SQLtext = "SELECT * FROM t_event WHERE ";
		SQLtext += (" typeid = "+what.getTypeId());
		SQLtext += (" and bg >= "+this.readBgTimeInterval(when));
		SQLtext += (" and ed <= "+this.readEdTimeInterval(when));
		SQLtext += (" ORDER BY ed DESC");
			
                ret = this.readEvents (SQLtext);

                return ret;
    	}

/* ------------------------------------------------------------------------- */


    	/** le todos os eventos de um tipo, indicado pelo tipo da
	 * instancia exemplo.
    	 * @param	BW_Event what	instancia exemplo do tipo a ser lido
    	 * ATENCAO: o typeId do parametro "what" deve estar setado corretamente
    	 */
    	public Vector readEvents (BW_Event what) {
    		Vector ret = null;
   		
   		String SQLtext = "SELECT * FROM t_event WHERE ";
		SQLtext += (" typeid = "+what.getTypeId());
		SQLtext += (" ORDER BY ed DESC");
		
		ret = this.readEvents (SQLtext);
		
      		return ret;
    	}
    	

/* ------------------------------------------------------------------------- */

    	
    	/** le um unico evento - o mais novo com o typeid e objid indicado
    	 * @param	BW_Event what	instancia exemplo para leitura
    	 * @param	int objid	identificador da instancia especifica desejada
    	 * ATENCAO: o typeid deve estar corretamente setado no parametro "what"
    	 */
    	public BW_Event readEvents (BW_Event what, int obj) {
    		BW_Event ret = null;
    		
    		try {
    			String SQLtext = "SELECT * FROM t_event WHERE ";
			SQLtext += (" typeid = "+what.getTypeId());
			SQLtext += (" and objid = "+obj);
			SQLtext += (" ORDER BY ed DESC");
			
			Vector v = this.readEvents (SQLtext);
			ret = (BW_Event) v.firstElement (); 			
		     		
     		} catch (java.lang.Exception ex) {	
     			this.catchException("readEvents",ex);       	
    		}          	
    		
    		return ret;
    	}

/* ------------------------------------------------------------------------- */

    	
    	/** Grava um vetor de eventos na base de dados */
        public boolean writeEvents (Vector what) {
        	boolean ret = true;
        	
   		System.out.println ("[DEBUG] ST_DatabaseImplementor.writeEvents: BEGIN writing ");

        	try {
        		//garantindo seguranca??
       			con.setAutoCommit (false);
       			
    			String SQLtext = "INSERT INTO t_event (typeid, objid, name, description, ";
    			SQLtext += " details, member_typeid, member_objid, bg, ed, tmz, lcl ) ";
    			SQLtext += " VALUES ( ? , ? , ? , ? , ? , ? , ? , ? , ? , ? , ? )";
    			PreparedStatement pstmt = con.prepareStatement(SQLtext);
    			
    			for (int i=0; i<what.size(); i++) {

    				BW_Event evt = (BW_Event) what.elementAt (i);    					
    				pstmt.setInt (1,evt.getTypeId());    				
    				pstmt.setInt (2,evt.getObjId());
    				pstmt.setString (3,evt.getName());
    				pstmt.setString (4,evt.getDescription());
    				pstmt.setString (5,evt.getDetails());

   				System.out.println ("[DBG]ST_DatabaseImplementor.writeEvents: writing ("+evt.getTypeId()+","+evt.getObjId()+")");
   				
    				BW_Member mb = (BW_Member) evt.getResponsible();
    				if (mb!=null) {
    					pstmt.setInt (6,mb.getTypeId());
    					pstmt.setInt (7,mb.getObjId());
    				} else {
    					pstmt.setInt (6,typeIdDefault.UNSET);
    					pstmt.setInt (7,typeIdDefault.UNSET);
    				}
    				
    				BW_TimeInterval ti = (BW_TimeInterval) evt.getInterval();
    				if (ti!=null) {
    					pstmt.setLong (8,this.readBgTimeInterval(ti));
    					pstmt.setLong (9,this.readEdTimeInterval(ti));
    					pstmt.setString (10,this.readTmzTimeInterval(ti));
    					pstmt.setString (11,this.readLclTimeInterval(ti));
    				}
    				else {
    					ti = new BW_TimeInterval();
    					pstmt.setLong (8,this.readBgTimeInterval(ti));
    					pstmt.setLong (9,this.readEdTimeInterval(ti));
    					pstmt.setString (10,this.readTmzTimeInterval(ti));
    					pstmt.setString (11,this.readLclTimeInterval(ti));    					
    				}
    				    					
    				int ResultCode = pstmt.executeUpdate();
	    			if (ResultCode<=0)
    					ret = ret && false;
    				else
    					ret = ret && true;
    				this.showWarnings ("writeEvents");		    						
	        	   //vai para o proximo elemento no vetor
    		        }
    		
    		 //seguranca??
    		 con.commit();
    		 con.setAutoCommit (true);
    		
    		} catch (SQLException ex) {
    		  	ret = false;
    		  	this.catchSQLException ("writeEvents",ex);
    		} catch (Exception ex) {
			ret = false;
			this.catchException ("writeEvents",ex);
    		}
        	
        	return ret;
        }


/* ------------------------------------------------------------------------- */


        //////////////// para os membros ---------------------------------------

/* ------------------------------------------------------------------------- */

	/** le os membros de acordo com a string SQL passada por parametro
	 * @param	String SQLtext	consulta SQL a ser realizada
	 * @return	Vector		vetor de BW_Member com o resultado da consulta SQL
	 */
	public Vector readMember (String SQLtext) {

        	Vector ret = null;
    		try {
			Statement stmt = con.createStatement();			
			ResultSet member = stmt.executeQuery (SQLtext);
			
        	   	Vector v = new Vector();
			boolean more = member.next();
			while (more) {
				BW_Member mb = new BW_Member();
				
				mb.setTypeId (member.getInt ("typeid"));
				mb.setObjId (member.getInt("objid"));
				mb.setName (member.getString("name"));
				mb.setMail (member.getString("mail"));
				mb.setHomePage (member.getString("homepage"));
				mb.setMachine (member.getString("Machine"));
				mb.setLogin (member.getString("login"));
				BW_TimeInterval ti = this.readTimeInterval (member);
				mb.setLastLogin (ti);
				
				int ptype = member.getInt("profile_typeid");
                                int pobj = member.getInt("profile_objid");
				BW_AwarenessProfile ap = (BW_AwarenessProfile) this.read (new BW_AwarenessProfile (ptype,pobj), pobj);
				mb.setAwarenessProfile (ap);
				mb.setPapers (ap.getPersonalPaperProfile());
				
				v.addElement (mb);
				more = member.next();
    			}
		
		   	if (!v.isEmpty())
		   		ret = v;
		   	else
		     		ret = null;
    		} catch ( SQLException ex ) {    			
         		this.catchSQLException("readMember",ex);
     		} catch (java.lang.Exception ex) {	
     			this.catchException("readMember",ex);		         	
    		}          	
        	
		return ret;		

	}


/* ------------------------------------------------------------------------- */


        /** le todos os membros que estiveram ativos no sistema
         * dentro de um determinado intervalo de tempo.
         * @param	BW_Member memb	tipo de membro a ser lido
         * @param	BW_TimeInterval time	intervalo de login desejado
         * @return	Vector	vetor com os BW_Member lidos
         */
        public Vector readMember (BW_Member memb, BW_TimeInterval time) {
        	Vector ret = null;
    		String SQLtext = "SELECT * FROM t_member  WHERE ";
		SQLtext += (" typeid = "+memb.getTypeId());
		SQLtext += (" and bg >= "+ this.readBgTimeInterval(time));
		SQLtext += (" and ed <= "+ this.readEdTimeInterval(time));
			
		ret = this.readMember (SQLtext);		
        	return ret;
        }

/* ------------------------------------------------------------------------- */


        /** le todas as instancias de BW_Member armazenadas.
	 * ATENCAO: o typeId da instancia exemplo passada como parametro
	 * deve estar corretamente setado.
         * @param	BW_Member memb	tipo de membro a ser lido
         * @return	Vector	vetor com as instancias lidas
         */
        public Vector readMember (BW_Member memb) {
        	Vector ret = null;
    		String SQLtext = "SELECT * FROM t_member  WHERE ";
		SQLtext += (" typeid = "+memb.getTypeId());									
		//System.out.println ("[DBG]ST_DatabaseImplementor.readMember: "+SQLtext);
		ret = this.readMember (SQLtext);
        	         	
        	return ret;
        }


/* ------------------------------------------------------------------------- */

        /** le uma determinada instancia de BW_Member
         * @param BW_Member what	modelo para o membro a ser lido, com o typeid setado
         * @param int objid		UID do membro a ser lido
         * @return BW_Member		instancia lida da base de dados,
         *				null se o membro nao existe
         */
        public BW_Member readMember (BW_Member what, int objid) {
        	BW_Member ret = null;
        	
    		try {
    			String SQLtext = "SELECT * FROM t_member  WHERE ";
			SQLtext += (" typeid = "+what.getTypeId());
			SQLtext += (" and objid = "+objid);									
			Vector v = this.readMember (SQLtext);
			ret = (BW_Member) v.firstElement();	
		     		
     		} catch (java.lang.Exception ex) {	
			//nao leu nenhum membro - nao faz nada e retorna null
     			//this.catchException("readMember",ex);		         	
    		}          	
        	        	
        	return ret;
        }


/* ------------------------------------------------------------------------- */

        /** Grava um vetor de instancias de BW_Member
         * @param Vector	vetor de BW_Member a ser salvo na base de dados
         * @return boolean	true se a operacao ocorreu sem problemas
         */
        public boolean writeMember (Vector what) {
            boolean ret = true;
        	
            try {
        	
        	String SQLtext1 = "UPDATE t_member SET name = ?, mail = ?, homepage = ?,"+
        				" login = ?, machine = ?, bg = ?, ed = ?, tmz = ?, lcl = ?, "+
        				" profile_typeid = ?, profile_objid = ? "+
        				" WHERE typeid = ? and objid = ?";
        	PreparedStatement pstmt = con.prepareStatement (SQLtext1);
        	        	
        	for (int i=0; i<what.size();i++) {
        		BW_Member mb = (BW_Member) what.elementAt (i);
        		BW_AwarenessProfile ap = mb.getAwarenessProfile();        		
        		Enumeration pp = mb.getPapers().keys();
        		BW_TimeInterval ti = mb.getLastLogin();        		
        		if (ti==null) {        		
        			ti = new BW_TimeInterval();
        		}	
        		
        		//grava na tabela t_member
        		pstmt.setString	(1,mb.getName());
        		pstmt.setString	(2,mb.getMail());
        		pstmt.setString	(3,mb.getHomePage());        		
        		pstmt.setString	(4,mb.getLogin());
        		pstmt.setString	(5,mb.getMachine());
        		
        		pstmt.setLong 	(6,this.readBgTimeInterval(ti));
        		pstmt.setLong 	(7,this.readEdTimeInterval(ti));
        		pstmt.setString (8,this.readTmzTimeInterval(ti));
        		pstmt.setString (9,this.readLclTimeInterval(ti));
        			        		
        		pstmt.setInt	(10,ap.getTypeId());
        	        pstmt.setInt	(11,ap.getObjId());
        		pstmt.setInt	(12,mb.getTypeId());
        	        pstmt.setInt	(13,mb.getObjId());
        	
        	        int Result = pstmt.executeUpdate();
        	
        	        if (Result <= 0) {
        	        	String SQLtext2 = "INSERT INTO t_member ( name, mail, homepage, login, machine, "+
        	        			  " bg, ed, tmz, lcl, profile_typeid, profile_objid, typeid, objid ) "+
        	        			  " VALUES ( ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ? )";
        	 		PreparedStatement pstmt2 = con.prepareStatement (SQLtext2);
        			pstmt2.setString	(1,mb.getName());
        			pstmt2.setString	(2,mb.getMail());
        			pstmt2.setString	(3,mb.getHomePage());        		
        			pstmt2.setString	(4,mb.getLogin());
        			pstmt2.setString	(5,mb.getMachine());
        			if (ti==null) {
        				ti = new BW_TimeInterval();
        			}
        			
        			pstmt2.setLong 	(6,this.readBgTimeInterval(ti));
        			pstmt2.setLong 	(7,this.readEdTimeInterval(ti));
        			pstmt2.setString(8,this.readTmzTimeInterval(ti));
        			pstmt2.setString(9,this.readLclTimeInterval(ti));
        			
        			pstmt2.setInt	(10,ap.getTypeId());
        	        	pstmt2.setInt	(11,ap.getObjId());
        			pstmt2.setInt	(12,mb.getTypeId());
        	        	pstmt2.setInt	(13,mb.getObjId());
        	 		
        	        	Result = pstmt2.executeUpdate();
        	 		if (Result <=0)
        	 			ret = false;
        	 		//ret = true;
        	        } else ret = ret && true;
        	
        	        //grava o profile        	
        	        Vector vap = new Vector();
        	        vap.addElement (ap);
        	        boolean r1 = this.writeAwarenessProfile (vap);        	
        	        ret = ret && r1;
        	}
        	        	        	
            } catch (SQLException ex) {
    		  	ret = false;
    		  	this.catchSQLException("writeMember",ex);
    	    } catch (Exception ex) {
			ret = false;
			this.catchException("writeMember",ex);
    	    }
        	
        	
        	return ret;
        }


        //////////////// para os papeis ---------------------------------------

/* ------------------------------------------------------------------------- */

	/** Le instancias de BW_Paper de acordo com a string SQL passada
	 * @param	String SQLtext	consulta SQL
	 * @return	Vector		vetor de BW_Paper com as instancas lida
	 */
	public Vector readPaper (String SQLtext) {
        	Vector ret = null;
        	
    		try {
			Statement stmt = con.createStatement();			
			ResultSet paper = stmt.executeQuery (SQLtext);
			
			Vector pps = new Vector();        	   	
			
			boolean more = paper.next();
			while (more) {			
				BW_Paper p = new BW_Paper();
				p.setTypeId (paper.getInt ("typeid"));
				p.setObjId (paper.getInt("objid"));
				p.setLevel (paper.getInt("level"));
				p.setName (paper.getString("name"));
				p.setDescription (paper.getString("description"));
				int ptype = paper.getInt("profile_typeid");
                                int pobj = paper.getInt("profile_objid");
				BW_Profile prof = (BW_Profile) this.read (new BW_Profile (ptype,pobj), pobj);
				p.setProfile (prof);
				
				pps.addElement (p);
				more = paper.next();
    			}
		
		   if (!pps.isEmpty()) {
		   	ret = pps;
		   }
		     		
    		} catch ( SQLException ex ) {
         		this.catchSQLException("readPaper",ex);
     		} catch (java.lang.Exception ex) {	
     			this.catchException("readPaper",ex);		         	

		}

		return ret;
	}

/* ------------------------------------------------------------------------- */

        /** le todas as instancias de BW_Paper armazenadas
         * @param	BW_Paper what	tipo de papel a ser lido
         * @return	Vector	vetor com as instancias lidas
         */
        public Vector readPaper (BW_Paper what) {
        	Vector ret = null;
    		String SQLtext = "SELECT * FROM t_paper  WHERE ";
		SQLtext += (" typeid = "+what.getTypeId());
			
        	ret = this.readPaper (SQLtext);	
        	return ret;
        }

/* ------------------------------------------------------------------------- */

        /** le uma determinada instancia de BW_Paper
         * @param BW_Paper what	modelo do papel a ser lido, com typeid setado
         * @param int objid	identificado do objeto a ser lido
         * @return BW_Paper	papel lido
         */
        public BW_Paper readPaper (BW_Paper what, int objid) {
        	BW_Paper ret = null;
        	
    		try {
    			String SQLtext = "SELECT * FROM t_paper  WHERE ";
			SQLtext += (" typeid = "+what.getTypeId());
			SQLtext += (" and objid = "+objid);									
			Vector v = this.readPaper (SQLtext);
			ret = (BW_Paper) v.firstElement();
	
     		} catch (java.lang.Exception ex) {	
     			//this.catchException("readPaper",ex);		         	
			//null pointer - se nao leu nada, retorna null
    		}          	
        	
        	return ret;
        }


/* ------------------------------------------------------------------------- */


        /** grava um vetor de instancias de BW_Paper
         * @param	Vector	instancias de BW_Paper a serem salvas
         * @return	boolean	true se a operacao foi realizada com sucesso
         */
        public boolean writePaper (Vector what) {
            boolean ret = true;

            try {
        	
        	String SQLtext1 = "UPDATE t_paper SET level = ?, name = ?, description = ?, "+
        				" profile_typeid = ?, profile_objid = ? "+
        				" WHERE typeid = ? and objid = ?";
        	PreparedStatement pstmt = con.prepareStatement (SQLtext1);
        	        	        	
        	for (int i=0; i<what.size();i++) {
        		BW_Paper paper = (BW_Paper) what.elementAt (i);
        		BW_Profile pro = paper.getProfile();        		
        		pstmt.setInt 	(1,paper.getLevel());
        		pstmt.setString	(2,paper.getName());
        		pstmt.setString	(3,paper.getDescription());        		
        		pstmt.setInt	(4,pro.getTypeId());
        		pstmt.setInt	(5,pro.getObjId());
        		pstmt.setInt	(6,paper.getTypeId());
        	        pstmt.setInt	(7,paper.getObjId());
        	        int Result = pstmt.executeUpdate();
        	        //System.out.println ("[DBG]ST_Database_Implementor.writePaper: UPDATING PAPER: "+Result);
        	
        	        if (Result <= 0) {
        	        	String SQLtext2 = "INSERT INTO t_paper ( level, name, description, "+
        	        			  " profile_typeid, profile_objid, typeid, objid ) "+
        	        			  " VALUES ( ?, ?, ?, ?, ?, ?, ? )";
        	 		PreparedStatement pstmt2 = con.prepareStatement (SQLtext2);
        			pstmt2.setInt 	(1,paper.getLevel());
        			pstmt2.setString(2,paper.getName());
        			pstmt2.setString(3,paper.getDescription());        		
        			pstmt2.setInt	(4,pro.getTypeId());
        			pstmt2.setInt	(5,pro.getObjId());
        			pstmt2.setInt	(6,paper.getTypeId());
        	        	pstmt2.setInt	(7,paper.getObjId());
        	        	Result = pstmt2.executeUpdate();
        	        	//System.out.println ("[DBG]ST_Database_Implementor.writePaper: INSERTING PAPER: "+Result);
        	 		if (Result <=0)
        	 			ret = false;
        	 		else ret = true && ret;
        	        } else ret = true && ret;
        	
        	        System.out.println ("[DBG]ST_Database_Implementor.writePaper: WRITE PROFILE: ("+pro.getTypeId()+","+pro.getObjId()+")");
        	        //ret = ret && this.write (paper.getProfile());
        	        boolean r1 = this.write ((BW_Profile) pro);
        	        /*Vector vpro = new Vector ();
        	        vpro.addElement (pro);
        	        boolean r1 = this.writeProfile (vpro);
        	        */
        	        ret = r1 && ret;
        	}
        	        	        	
            } catch (SQLException ex) {
    		  	ret = false;
    		  	this.catchSQLException("writePaper",ex);
    	    } catch (Exception ex) {
			ret = false;
			this.catchException("writePaper",ex);
    	    }
        		
           return ret;
            	
        }


/* ------------------------------------------------------------------------- */



        //////////////// para o grupo -----------------------------------------

/* ------------------------------------------------------------------------- */

   public boolean readGroupPaperMember (BW_Group ret) throws SQLException, NullPointerException {
        boolean r = true;
	String SQLtext21 = "SELECT * FROM ";
	String SQLtext22 = " WHERE group_typeid = ? and group_objid = ?";    			
	PreparedStatement stmt;
	  			
  	//le os papeis do grupo
    	stmt = con.prepareStatement (SQLtext21+"t_group_paper"+SQLtext22);
	stmt.setInt (1,ret.getTypeId());
	stmt.setInt (2,ret.getObjId());    				
    	ResultSet results2 = stmt.executeQuery();
    	boolean more2 = results2.next();
    	while (more2) {
    		int ptype = results2.getInt("paper_typeid");
    		int pobj = results2.getInt("paper_objid");
    		
    		System.out.println ("[DBG]ST_DatabaseImplementor.readGroupMember: reading paper ("+ptype+","+pobj+")");
    		BW_Paper paper = this.readPaper (new BW_Paper (ptype,pobj),pobj);
    		if (paper != null)
    			ret.insertPaper (paper);
    		else r = r && false;	
    		more2 = results2.next();
    	}
    	
	//le os membros do grupo
    	stmt = con.prepareStatement (SQLtext21+"t_group_member"+SQLtext22);
	stmt.setInt (1,ret.getTypeId());
	stmt.setInt (2,ret.getObjId());    				
    	results2 = stmt.executeQuery();
    	more2 = results2.next();
    	while (more2) {
    		int ptype = results2.getInt("member_typeid");
    		int pobj = results2.getInt("member_objid");
    		BW_Member memb = new BW_Member ();
    		memb.setTypeId (ptype); memb.setObjId(pobj);
    		    		
    		memb = this.readMember (memb,pobj);
		//System.out.println ("[DBG] ST_DatabaseImplementor.readGroupMember: reading member ("+ptype+","+pobj+"): "+memb);    		
		System.out.println ("[DBG]ST_DatabaseImplementor.readGroupMember: reading member ("+ptype+","+pobj+")");
		
    		if (memb != null)
    			ret.insertMember (memb);
    		else
    			r = r && false;	
    		more2 = results2.next();
    	}
	    	
    	return r;
  }

/* ------------------------------------------------------------------------- */


   public boolean writeGroupPaperMember (BW_Group grp) throws SQLException, NullPointerException {
        boolean r = true;
    	
        String SQLtext21 = "DELETE FROM ";
        String SQLtext22 = " WHERE group_typeid = ? and group_objid = ?";    			
    	String SQLtext31 = "INSERT INTO ";
    	String SQLtext32 = " ( group_typeid , group_objid , ";
    	String SQLtext33 = " , ";
    	String SQLtext34 = " )  VALUES ( ? , ? , ? , ?) ";
    	    			

    	//System.out.println ("[DBG]ST_DatabaseImplementor.writeGroupPaperMember: queries: "+SQLtext2+" and "+SQLtext3);
    	
        PreparedStatement pstmt ;

        				
    	//insere os papeis
    	Vector v = grp.getPapers();
    	boolean r1 = this.writePaper (v);
    	
    	//if (v!=null && r1)
    	if (v!=null)
    	{
    	   //System.out.println ("[DBG]ST_DatabaseImplementor.writeGroupPaperMember: preparing");
    	
    	   pstmt= con.prepareStatement (SQLtext21+" t_group_paper "+SQLtext22);
    	   pstmt.setInt (1,grp.getTypeId());
    	   pstmt.setInt (2,grp.getObjId());
    	   int ResultCode = pstmt.executeUpdate();
    	
    	   //System.out.println ("[DBG]ST_DatabaseImplementor.writeGroupPaperMember: deleting group_paper "+ResultCode);
    	
    	   pstmt= con.prepareStatement (SQLtext31+"t_group_paper"+SQLtext32+"paper_typeid"+SQLtext33+"paper_objid"+SQLtext34);
    	   pstmt.setInt (1,grp.getTypeId());
    	   pstmt.setInt (2,grp.getObjId());    					
    			       					       					   	
    	   for (int j=0; j<v.size();j++) {
    	   	BW_Paper pap = (BW_Paper) v.elementAt(j);
    	   	pstmt.setInt (3,pap.getTypeId());
    	   	pstmt.setInt (4,pap.getObjId());    					   	
    	   	ResultCode = pstmt.executeUpdate();
    	   	//System.out.println ("[DBG]ST_DatabaseImplementor.writeGroupPaperMember: inserting group_paper ("+pap.getTypeId()+","+pap.getObjId()+")");
    		r1 = r1 && (ResultCode>0);
    	   }	
    	}    					    					
    					
    					
    	//insere os membros
    	v = new Vector();
    	Enumeration e = grp.getMembers().elements();
    	while (e.hasMoreElements()) {
    		v.addElement (e.nextElement());
    	}
    					
    	boolean r2 = this.writeMember(v);
    	//if (r2) {
    	   pstmt= con.prepareStatement (SQLtext21+"t_group_member"+SQLtext22);
    	   pstmt.setInt (1,grp.getTypeId());
    	   pstmt.setInt (2,grp.getObjId());
    	   int ResultCode = pstmt.executeUpdate();
    	
    	   //System.out.println ("[DBG]ST_DatabaseImplementor.writeGroupPaperMember: deleting group_member "+ResultCode);
    	
    	   pstmt= con.prepareStatement (SQLtext31+"t_group_member"+SQLtext32+"member_typeid"+SQLtext33+"member_objid"+SQLtext34); 			
    	   pstmt.setInt (1,grp.getTypeId());
    	   pstmt.setInt (2,grp.getObjId());    					
    					       					       					   	
    	   for (int j = 0; j<v.size();j++) {
    	   	BW_Member mb = (BW_Member) v.elementAt(j);
    	   	pstmt.setInt (3,mb.getTypeId());
    	   	pstmt.setInt (4,mb.getObjId());    					   	
    	   	ResultCode = pstmt.executeUpdate();
    	   	//System.out.println ("[DBG]ST_DatabaseImplementor.writeGroupPaperMember: inserting group_member ("+mb.getTypeId()+","+mb.getObjId()+")");
    		r2 = r2 && (ResultCode>0);
    	   }	    					    					
    	//}
    					
    	r =  r1	&& r2;	    	
    	
    	return r;
    }    	


/* ------------------------------------------------------------------------- */

	/** faz a leitura de instancias de BW_Group de acordo com consulta SQL
	 * enviada como parametro
	 * @param	String SQLtext	consulta SQL a ser executada
 	 * @return	Vector		vetor de BW_Group lidos
	 */
	public Vector readGroup (String SQLtext) {
        	Vector ret = null;
        	
    		try {
    			//garantindo seguranca?
    			con.setAutoCommit (false);	
    		
			Statement stmt = con.createStatement();			
			ResultSet results = stmt.executeQuery (SQLtext);
			//this.showWarnings ("readGroup");
			//System.out.println ("[DBG] ST_DatabaseImplementor.read: getting results");			
			boolean more = results.next ();
			ret = new Vector();
			
			while (more) {
    		        	int typeid = results.getInt ("typeid");
    		        	int objid = results.getInt ("objid");
    		        	String name = results.getString ("name");
    		        	String descr = results.getString ("description");
    		        	
    		        	BW_Group grp = new BW_Group (typeid,objid);
    		        	grp.setName (name);
    		        	grp.setDescription (descr);    		        	
    		        	ret.addElement (grp);
    				//System.out.println ("[DBG] ST_DatabaseImplementor.read: result readed");
    				
    				//le papeis e membros
    				this.readGroupPaperMember (grp);
    				
    				//passa para o proximo registro
    				 more = results.next ();
    				    				
    			} 	
    		
    		  //garantindo seguranca??
    		  con.commit();
   		  con.setAutoCommit (true); 	
   		
    		} catch ( SQLException ex ) {
         		this.catchSQLException("readGroup",ex);
     		} catch (java.lang.Exception ex) {
     			this.catchException("readGroup",ex);	   		
    		}          	
        	        	
		return ret;

	}

/* ------------------------------------------------------------------------- */


        /** faz a leitura de uma instancia de BW_Group
         *  @param 	BW_Group model	modelo do grupo a ser lido - deve ter o typeId e o ObjId setados
         *  @return	BW_Group	entidade lida
         */
        public BW_Group readGroup (BW_Group model) {
                BW_Group ret = null;
        	//System.out.println ("[DBG] ST_DatabaseImplementor.read: reading a Group");
        	
    		try {
    			String SQLtext = "SELECT * FROM t_group WHERE ";
			SQLtext += " typeid = "+model.getTypeId();
			SQLtext += " and objid = "+model.getObjId();		
			
			Vector v = this.readGroup (SQLtext);
			ret = (BW_Group) v.firstElement();

     		} catch (java.lang.Exception ex) {			         	
     		        //this.catchException("readGroup",ex);
    		}        	
        	
                return ret;
        }

/* ------------------------------------------------------------------------- */

        /** faz a leitura de todas as entidades BW_Group
         * @return	Vector	vetor de BW_Groups armazenados
         */
        public Vector readGroup () {
        	Vector ret = null;
        	
    		String SQLtext = "SELECT * FROM t_group";
		ret = this.readGroup (SQLtext);	
        	        	
		return ret;
        }

/* ------------------------------------------------------------------------- */


        /** grava um vetor de entidades BW_Group na base de dados
         * @param	Vector what	vetor de entidades BW_Group
         * @return	boolean	true se a operacao ocorreu bem,
         *			false caso contrario
         */
        public boolean writeGroup (Vector what) {
        	boolean ret = false;
        	
        	//System.out.println ("[DBG] ST_DatabaseImplementor.write: writing Groups");
        	
        	try {
        		//garantindo seguranca??
       			con.setAutoCommit (false);
       			
    			String SQLtext = "UPDATE t_group SET typeid = ? , objid = ? , name = ? , description = ? ";
    			SQLtext += " WHERE typeid = ? and objid = ? ";    			    			
    			PreparedStatement pstmt = con.prepareStatement(SQLtext);
    			 			
    			
    			for (int i=0; i<what.size(); i++) {
    				
    				BW_Group grp = (BW_Group) what.elementAt (i);
    				
    				//atualiza a tabela t_group
    				pstmt.setInt (1,grp.getTypeId());    				
    				pstmt.setInt (5,grp.getTypeId());
    				pstmt.setInt (2,grp.getObjId());
    				pstmt.setInt (6,grp.getObjId());
    				pstmt.setString (3,grp.getName());
    				pstmt.setString (4,grp.getDescription());
    			
    				int ResultCode = pstmt.executeUpdate();
    		     	
    				//se nada foi atualizado faz a insercao na tabela
    				if (ResultCode<=0) {
    					String SQLtext2 = new String ("INSERT INTO t_group ( typeid, objid, name, description )");
    					SQLtext2 += "VALUES ( ? , ? , ? , ? )";
    					pstmt = con.prepareStatement (SQLtext2);
    					
    					pstmt.setInt (1,grp.getTypeId());    				
	    				pstmt.setInt (2,grp.getObjId());
    	    				pstmt.setString (3,grp.getName());
    					pstmt.setString (4,grp.getDescription());    					
    					
    					ResultCode = pstmt.executeUpdate();
	    				if (ResultCode<=0)
    						ret = false;
    					else
    						ret = true;
    						    						
	    			}
    				else ret = true;
    				
    				//se conseguiu inserir/atualizar
    				if (ret) {
                                   ret = this.writeGroupPaperMember (grp);
    				}
	        	   //this.showWarnings ("writeGroup");	
	        	   //vai para o proximo elemento no vetor
    		        }
    		
    		 //seguranca??
    		 con.commit();
    		 con.setAutoCommit (true);
    		
    		} catch (SQLException ex) {
    		  	ret = false;
    		  	this.catchSQLException("writeGroup",ex);
    		} catch (Exception ex) {
			ret = false;
			this.catchException("writeGroup",ex);
    		}
        	
        	return ret;
        }


/* ------------------------------------------------------------------------- */



      //////////////// para o regitro -----------------------------------------

/* ------------------------------------------------------------------------- */
      /** faz a leitura de uma entidade BW_Register especifica
       * @param	BW_Register model	modelo de registro (typeid setado)
       * @param int objid		objid do Register a ser lido
       * @return	BW_Register	registro lido
       */
      public BW_Register readRegister (BW_Register model, int objid) {
      		BW_Register ret = null;
      		boolean readed = false;
      		
    		try {
    			//garantindo seguranca?
    			con.setAutoCommit (false);	
    		
    			String SQLtext = "SELECT event_typeid, event_classname FROM t_register WHERE ";
    			SQLtext += (" typeid = " + model.getTypeId() );
    			SQLtext += (" and objid = " + objid );
			Statement stmt = con.createStatement();
			
			ResultSet results = stmt.executeQuery (SQLtext);
			//this.showWarnings ("readRegister");

			boolean more = results.next ();
			
			BW_Register registro = new BW_Register();
			registro.setTypeId(model.getTypeId());
			registro.setObjId(model.getObjId());				
					
			while (more) {
    		        	int typeid = results.getInt ("event_typeid");
    		        	String classname = results.getString ("event_classname");    		        	
    		        	BW_Event evt = (BW_Event) Class.forName(classname).newInstance();
    		        	evt.setTypeId (typeid);    		        	
    		        	registro.insertEvent (evt);
    				
    				//passa para o proximo registro
    				 more = results.next ();    				    				
    				 readed = true;
    			} 	
    		    			
    		  //garantindo seguranca??
    		  con.commit();
   		  con.setAutoCommit (true); 	
   		  if (readed)
   		  	ret = registro;
   		
    		} catch ( SQLException ex ) {
         		this.catchSQLException("readRegister",ex);
     		} catch (java.lang.Exception ex) {			         	
     		        this.catchException("readRegister",ex);
    		}          	      		      		
      		
                return ret;
      }


/* ------------------------------------------------------------------------- */

      /** faz a leitura todas entidades BW_Register
       * @param	BW_Register model	modelo de registro (typeid setado)
       * @return	Vector		registros lidos
       */
      public Vector readRegister (BW_Register model) {
      		Vector ret = null;
  		Hashtable buf = new Hashtable ();
      		
    		try {
    			//garantindo seguranca?
    			con.setAutoCommit (false);	
    		
    			String SQLtext = "SELECT * FROM t_register WHERE ";
    			SQLtext += (" typeid = " + model.getTypeId() );
			Statement stmt = con.createStatement();
			
			ResultSet results = stmt.executeQuery (SQLtext);

    		   	//mostra os warnings da ultima consulta
    		   	this.showWarnings ("readRegister");
  			    						
			boolean more = results.next ();
											
			while (more) {
				int objid = results.getInt ("objid");
				
				BW_Register registro=null;
				//se registro ainda nao tinha aparecido
    		        	if (!buf.containsKey (new Integer ("objid"))) {
    		        		registro = new BW_Register();
					registro.setTypeId(model.getTypeId());
					registro.setObjId(objid);				
    		        	}
    		        	else {
    		        		registro = (BW_Register) buf.get(new Integer (objid));
    		        	}
    		        	
    		        	int event_typeid = results.getInt ("event_typeid");
    		        	String classname = results.getString ("event_classname");    		        	
    		        	BW_Event evt = (BW_Event) Class.forName(classname).newInstance();
    		        	evt.setTypeId (event_typeid);    		        	
    		        	registro.insertEvent (evt);
    				
    				//passa para o proximo registro
    				 more = results.next ();    				    				
    			} 	
    		    			
    		  //garantindo seguranca??
    		  con.commit();
   		  con.setAutoCommit (true); 	
   		
   		  Enumeration en = buf.elements();   		
   		  ret = new Vector();
   		  while (en.hasMoreElements()) {
   		  	ret.addElement (en.nextElement());
   		  }
   		
    		} catch ( SQLException ex ) {
         		this.catchSQLException("readRegister",ex);
     		} catch (java.lang.Exception ex) {	
     			this.catchException("readRegister",ex);     	
    		}          	
      		
      		
      		
                return ret;
      }


/* ------------------------------------------------------------------------- */


      /** grava um conjunto de entidades BW_Register colocadas em um vetor.
       * @param  	Vector what	vetor de entidades BW_Register
       * @return	boolean		true se a gravacao ocorreu ok,
       *				false, caso contrario
       */
       public boolean writeRegister (Vector what) {
        	boolean ret = true;
        	
        	try {
        		//garantindo seguranca??
       			con.setAutoCommit (false);
       			
       			//primeiro apaga as ocorrencias anterior
       			//para garantir que um evento removido do objeto seja removido!
    			String SQLtext= "DELETE FROM t_register ";
    			SQLtext += 	"   WHERE typeid = ? and objid = ? ";    			
    			PreparedStatement pstmt = con.prepareStatement(SQLtext);
    			
    			for (int i=0; i<what.size();i++) {
    				BW_Register registro = (BW_Register) what.elementAt (i);
    				pstmt.setInt (1,registro.getTypeId());
    				pstmt.setInt (2,registro.getObjId());
				pstmt.executeUpdate ();
    			}
    			    			    			
    			//insere todos os registro atualizados de novo
    			String SQLtext2 = new String ("INSERT INTO t_register ( typeid, objid, event_typeid, event_classname ) ");
    			SQLtext2 += "VALUES  ( ? , ? , ? , ? )";
    			pstmt = con.prepareStatement (SQLtext2);    			    			 			
    			
    			for (int i=0; i<what.size(); i++) {
    				
    			   BW_Register  registro = (BW_Register) what.elementAt (i);
    			   Hashtable ht = registro.getEvents();
    			   Enumeration en = ht.keys();
    			
    			   //para cada evento do registro    			
    			   while (en.hasMoreElements()) {    				
    				String st = (String) en.nextElement();
    				int separator =  st.indexOf ("-");
    				//System.out.println ("[DBG] ST_DatabaseImplementor.write: string "+st+
    				//		    " separator "+separator+" substring: "+st.substring (0,separator));
    				int typeid = Integer.parseInt (st.substring (separator+1,st.length()));
    				String classname = st.substring (0,separator);
    					
    				pstmt.setInt (1,registro.getTypeId());    				
    				pstmt.setInt (2,registro.getObjId());
    				pstmt.setInt (3,typeid);
    				pstmt.setString (4,classname);
    					
    				int ResultCode = pstmt.executeUpdate();
	    			if (ResultCode<=0)
    					ret = ret && false;
    				else
    					ret = ret && true;
    						    						
    				    			
    				//mostra os warnings da ultima consulta
    				this.showWarnings ("writeRegister");
	        	   	//vai para o proximo evento registrado
	        	   }
	        	   	   	
	        	   //vai para o proximo elemento no vetor
    		        }
    		
    		 //seguranca??
    		 con.commit();
    		 con.setAutoCommit (true);
    		
    		} catch (SQLException ex) {
    		  	ret = false;
    		  	this.catchSQLException ("writeRegister",ex);
    		} catch (Exception ex) {
			ret = false;
			this.catchException ("writeRegister",ex);
    		}
        	
        	return ret;

       }


/* ------------------------------------------------------------------------- */


  ///////////////////////////  para PROFILES ------------------------------------

/* ------------------------------------------------------------------------- */

  /** le os profiles de acordo com uma determinada consulta SQL.
    * @param	String SQLtext	consulta SQL a ser executada
    * @return 	Vector		vetor com os BW_Profile lidos
    */
   public Vector readProfile (String SQLtext) {
        	Vector ret = null;
    		
    		try {
			Statement stmt = con.createStatement();			
			ResultSet profile = stmt.executeQuery (SQLtext);
			
        	   	Vector v = new Vector();			
			boolean more = profile.next();
			while (more) {
			        int ptype =  profile.getInt ("typeid");
			        int pobj =  profile.getInt("objid");
			
			        SQLtext = "SELECT * FROM t_profile_event WHERE ";
				SQLtext += (" profile_typeid = "+ptype);
				SQLtext += (" and profile_objid = "+pobj);													
				ResultSet events = stmt.executeQuery (SQLtext);
			
				BW_Profile p = new BW_Profile();
				p.setTypeId (ptype);
				p.setObjId (pobj);				
				BW_TimeInterval ti = this.readTimeInterval(profile);
				if (ti!=null) p.setInterval (ti);
				
				//System.out.println ("[DBG] ST_DatabaseImplementor.read: readind events for the profile");
				boolean more2 = events.next();
				while (more2) {
				     int etype = events.getInt ("event_typeid");
				     String ecname = events.getString("event_classname");
				     BW_Event e = (BW_Event) Class.forName(ecname).newInstance();
				     e.setTypeId(etype);
				     e.setObjId(typeIdDefault.UNSET);
				     p.insertEvent (e);
				     more2 = events.next();
				}
				
				v.addElement (p);
				more = profile.next();
    			} 	
		
		  ret = v;
		     		
    		} catch ( SQLException ex ) {
         		this.catchSQLException("readProfile",ex);
     		} catch (java.lang.Exception ex) {	
     			this.catchException("readProfile",ex);		         	
    		}          	
    		
    		return ret;
   }


/* ------------------------------------------------------------------------- */

   /** faz a leitura de um grupo de profile de um determinado tipo, que
    * incluem um determinado perido.
    * @param	BW_Profile model	profile modelo, com o typeid desejado
    * @param	BW_TimeInterval	when	intervalo desejado
    * @return	Vector			vetor de BW_Profile que incluem o periodo pedido
    */
   public Vector readProfile (BW_Profile model, BW_TimeInterval when) {
        	Vector ret = null;
    		
    		String SQLtext = "SELECT * FROM t_profile WHERE ";
		SQLtext += (" typeid = "+model.getTypeId());
		SQLtext += (" and bg >= "+this.readBgTimeInterval(when));
		SQLtext += (" and ed <= "+this.readEdTimeInterval(when));
		//System.out.println ("[DBG] ST_DatabaseImplementor.read: "+SQLtext);
			
    		ret = this.readProfile (SQLtext);	
    		return ret;
   }


/* ------------------------------------------------------------------------- */

  /** le todos os profiles de um determinado tipo.
    * @param	BW_Profile model	profile modelo, com o typeid desejado setado
    * @return 	Vector			vetor com todos os BW_Profile lidos
    */
   public Vector readProfile (BW_Profile model) {
        	Vector ret = null;
    		
    		String SQLtext = "SELECT * FROM t_profile WHERE ";
		SQLtext += (" typeid = "+model.getTypeId());
			
		ret = this.readProfile (SQLtext);
    		
    		return ret;
   }


/* ------------------------------------------------------------------------- */
   /** le uma instancia especifica de profile, com o objid indicado
    * @param 	BW_Profile model	modelo de profile, com o typeid dsejado setado
    * @param	int objid		objid da instancia desejada
    * @return	BW_Profile		instancia lida, ou null se nenhuma instancia foi lida
    */
   public BW_Profile readProfile (BW_Profile model, int objid) {
        	BW_Profile ret = null;
    		
    		try {
    			String SQLtext = "SELECT * FROM t_profile WHERE ";
			SQLtext += (" typeid = "+model.getTypeId());
			SQLtext += (" and objid = "+objid);									
			Vector v = this.readProfile (SQLtext);
			ret = (BW_Profile) v.firstElement ();

     		} catch (java.lang.Exception ex) {	
     		//	this.catchException("readProfile",ex);		         	
    		}          	
    		
    		return ret;


   }


/* ------------------------------------------------------------------------- */

  /** escreve na base de dados um conjunto de profiles
   * @param	Vector	vetor de BW_Profiles a serem escritos
   * @return	boolean	true se a operacao ocorreu com sucesso, false caso contrario
   */
  public boolean writeProfile (Vector v) {
	
        	boolean ret = true;
        	
        	try {
        		
        		//System.out.println ("[DBG]ST_DatabaseImplementor.writeProfile: HERE");
       			//primeiro apaga as ocorrencias anterior
       			//para garantir que um evento removido do objeto seja removido!
    			String SQLtext= "DELETE FROM t_profile_event ";
    			SQLtext += 	"   WHERE profile_typeid = ? and profile_objid = ? ";    			
    			PreparedStatement delete = con.prepareStatement(SQLtext);
    			
    			SQLtext = "UPDATE t_profile SET bg = ? , ed = ? , tmz = ? , lcl = ? ";
    			SQLtext += "      WHERE typeid = ? and objid = ?";
    			PreparedStatement update = con.prepareStatement(SQLtext);
    			
    			SQLtext = "INSERT INTO t_profile_event ( profile_typeid, profile_objid, event_typeid, event_classname ) ";
    			SQLtext += "      VALUES ( ? , ? , ? , ? ) ";
    			PreparedStatement insert = con.prepareStatement(SQLtext);
    			
    			for (int i=0; i<v.size(); i++) {
    				
    			   BW_Profile p = (BW_Profile) v.elementAt (i);
    			
    			   //System.out.println ("[DBG]ST_DatabaseImplementor.writeProfile: writing interval "+p.getInterval());
    			
    			   //atualiza o profile simples
    			   update.setLong (1,this.readBgTimeInterval(p.getInterval()));
    			   update.setLong (2,this.readEdTimeInterval(p.getInterval()));    			    			
    			   update.setString (3,this.readTmzTimeInterval(p.getInterval()));
    			   update.setString (4,this.readLclTimeInterval(p.getInterval()));
    			   update.setInt (5,p.getTypeId());
    			   update.setInt (6,p.getObjId());
    			   int ResultCode = update.executeUpdate();
    			
    			   System.out.println ("[DBG]ST_DatabaseImplementor.writeProfile: writing BW_profile: ("+p.getTypeId()+","+p.getObjId()+") "+ResultCode);
    			
    			   //eh profile novo
    			   if (ResultCode<=0) {
    				String SQLtext2 = "INSERT INTO t_profile (bg, ed, tmz, lcl, typeid, objid )";
    				SQLtext2 += "      VALUES ( ? , ? , ? , ? , ? , ? )";
    				PreparedStatement update2 = con.prepareStatement(SQLtext2);
    			   	update2.setLong (1,this.readBgTimeInterval(p.getInterval()));
    			   	update2.setLong (2,this.readEdTimeInterval(p.getInterval()));    			    			
    			   	update2.setString (3,this.readTmzTimeInterval(p.getInterval()));
    			   	update2.setString (4,this.readLclTimeInterval(p.getInterval()));
    			   	update2.setInt (5,p.getTypeId());
    			   	update2.setInt (6,p.getObjId());
    			   	update2.executeUpdate();    				    				
    			   }

    			   //tira os eventos registrados nele
    			   delete.setInt (1,p.getTypeId());
    			   delete.setInt (2,p.getObjId());
    			   ResultCode = delete.executeUpdate();
    			   System.out.println ("[DBG]ST_DatabaseImplementor.writeProfile: deleting events "+ResultCode);
    			
    			       			
    			   //recoloca os atuais  			
    			   Enumeration en = p.getEvents().keys();    					
    			   while (en.hasMoreElements()) {
    			   	String evt = (String) en.nextElement();
    			   	int pos = evt.indexOf("-");
    			   	String ecname = evt.substring (0,pos);
    			   	int etype = Integer.parseInt (evt.substring(pos+1,evt.length()));
    			   	
    			   	insert.setInt (1,p.getTypeId());
    			   	insert.setInt (2,p.getObjId());
    			   	insert.setInt (3,etype);
    			   	insert.setString (4,ecname);
    			   	insert.executeUpdate();
	        	   }
	        	   	   	
	        	   //vai para o proximo elemento no vetor
    		        }
    		
    		            				    			
    			//mostra os warnings da ultima consulta
    			//this.showWarnings ("writeProfile");    		
	        	ret = true;	        	
    		} catch (SQLException ex) {
    		  	ret = false;
    		  	this.catchSQLException ("writeProfile",ex);
    		} catch (Exception ex) {
			ret = false;
			this.catchException ("writeProfile",ex);
    		}
        	
        	return ret;



  }


/* ------------------------------------------------------------------------- */

   /** le todos os awareness profile, de acordo com o modelo indicado, que
    * incluam o intervalo indicado.
    * @param 	BW_AwarenessProfile model	modelo de profile, com o typeid dsejado setado
    * @param	BW_TimeInterval interval	intervalo de interesse
    * @return	Vector		vetor de BW_AwarenessProfile com todas as instancias lidas
    */
   public Vector readAwarenessProfile (BW_AwarenessProfile model, BW_TimeInterval when) {
        	Vector ret = null;
        	Vector awprofiles = new Vector ();
    		
    		try {
    		  Vector profiles = this.readProfile ( (BW_Profile) model, when );    		
    		
    		  for (int j=0; j<profiles.size();j++) {
    		        BW_Profile p = (BW_Profile) profiles.elementAt(j);        	
        	   	BW_AwarenessProfile ap = new BW_AwarenessProfile (model.getTypeId(),p.getObjId());
        	   	ap = this.readAwarenessProfile (ap,p.getObjId());
        	   	awprofiles.addElement(ap);
        	   }	
		
		  if (!awprofiles.isEmpty())
		  	ret = awprofiles;
		
     		} catch (java.lang.Exception ex) {	
     			this.catchException("readAwarenessProfiles",ex);		         	
    		}          	
    		
    		return ret;

   }


/* ------------------------------------------------------------------------- */
   /** le todos os awareness profile de acordo com o modelo indicado
    * @param 	BW_AwarenessProfile model	modelo de profile, com o typeid dsejado setado
    * @return	Vector		vetor de BW_AwarenessProfile com todas as instancias lidas
    */
   public Vector readAwarenessProfile (BW_AwarenessProfile model) {
        	Vector ret = null;
        	Vector awprofiles = new Vector ();
    		
    		try {
    		  Vector profiles = this.readProfile ( (BW_Profile) model);    		
    		
    		  for (int j=0; j<profiles.size();j++) {
    		        BW_Profile p = (BW_Profile) profiles.elementAt(j);        	
        	   	BW_AwarenessProfile ap = new BW_AwarenessProfile (model.getTypeId(),p.getObjId());
        	   	ap = this.readAwarenessProfile (ap,p.getObjId());
        	   	awprofiles.addElement(ap);
        	   }	
		
		  if (!awprofiles.isEmpty())
		  	ret = awprofiles;
		
     		} catch (java.lang.Exception ex) {	
     			this.catchException("readAwarenessProfiles",ex);		         	
    		}          	
    		
    		return ret;

   }


/* ------------------------------------------------------------------------- */

   /** le uma instancia especifica de awareness profile, com o objid indicado
    * @param 	BW_AwarenessProfile model	modelo de profile, com o typeid dsejado setado
    * @param	int objid			objid da instancia desejada
    * @return	BW_AwarenessProfile		instancia lida
    */
   public BW_AwarenessProfile readAwarenessProfile (BW_AwarenessProfile model, int objid) {
        	BW_AwarenessProfile ret = null;
    		
    		try {
    			String SQLtext11 = "SELECT * FROM ";
			String SQLtext12 = " WHERE typeid = ? and objid = ? ";
			PreparedStatement stmt;
        	   	
        	   	BW_Profile p = this.readProfile (model,objid);
        	   	BW_AwarenessProfile ap = new BW_AwarenessProfile (model.getTypeId(),objid);
        	   	ap.setEvents (p.getEvents());
        	   	ap.setInterval (p.getInterval());     	   	
        	   	//System.out.println ("[DBG] ST_DatabaseImplementor.readAwarenessProfile: readed: "+ap);
        	   	
			stmt = con.prepareStatement(SQLtext11+" t_profile_personal "+SQLtext12);			
			stmt.setInt (1,model.getTypeId()); stmt.setInt (2,objid);
			ResultSet personal = stmt.executeQuery();			
			stmt = con.prepareStatement(SQLtext11+" t_profile_personal_paper "+SQLtext12);
			stmt.setInt (1,model.getTypeId()); stmt.setInt (2,objid);			
			ResultSet paper = stmt.executeQuery();
			
			boolean more = personal.next();
			while (more) {
				int ptype = personal.getInt ("profile_typeid");
				int pobj = personal.getInt ("profile_objid");
				System.out.println ("[DBG]ST_DatabaseImplementor.read: personal profile readed ("+ptype+","+pobj+")");
				BW_Profile pers = (BW_Profile) this.read (new BW_Profile (ptype,pobj),pobj);
				ap.insertPersonalProfile (pers);
				more = personal.next();
    			}
    			
			more = paper.next();
			while (more) {
				int ptype = paper.getInt ("profile_typeid");
				int pobj = paper.getInt ("profile_objid");
				int pptype = paper.getInt ("paper_typeid");
				int ppobj = paper.getInt ("paper_objid");
				BW_Profile pers = (BW_Profile) this.read (new BW_Profile (ptype,pobj),pobj);
				BW_Paper pap = (BW_Paper) this.read (new BW_Paper (pptype,ppobj),ppobj);
				
				System.out.println ("[DBG]ST_DatabaseImplementor.read: personal paper profile readed ("+ptype+","+pobj+","+pptype+","+ppobj+")");
				if (pap!=null)
					ap.insertPersonalPaperProfile (pers,pap);
				more = paper.next();
    			}    			
    			
    			//mostra os warnings da ultima consulta
    			//this.showWarnings ("readAwarenessProfile");
		
		  ret = ap;
		     		
    		} catch ( SQLException ex ) {
         		this.catchSQLException("readAwarenessProfile",ex);
     		} catch (java.lang.Exception ex) {	
     			this.catchException("readAwarenessProfiles",ex);		         	
    		}          	
    		
    		return ret;


   }


/* ------------------------------------------------------------------------- */

  /** escreve na base de dados um conjunto de awareness profiles
   * @param	Vector	what	vetor de BW_AwarenessProfiles a serem escritos
   * @return	boolean		true se a operacao ocorreu com sucesso, false caso contrario
   */
  public boolean writeAwarenessProfile (Vector what ) {
	
        	boolean ret = true;
        	
        	try {
       			//primeiro apaga as ocorrencias anterior
       			//para garantir que um evento removido do objeto seja removido!
    			String SQLtext11= "DELETE FROM ";
    			String SQLtext12= "   WHERE typeid = ? and objid = ? ";    			
    			PreparedStatement delete; // = con.prepareStatement(SQLtext);
    			
    			String SQLtext21 = "INSERT INTO ";
    			String SQLtext22 = " ( typeid, objid, profile_typeid, profile_objid ";
    			String SQLtext23 = " ) VALUES ( ? , ? , ? , ? ";
    			String SQLtext24 = " ) ";
    			PreparedStatement insert; // = con.prepareStatement(SQLtext);
    			
    			for (int i=0; i<what.size(); i++) {    				
    			   boolean RProf = true;
    			   boolean RPers = true;
    			   boolean RPap = true;
    			
    			   BW_AwarenessProfile p = (BW_AwarenessProfile) what.elementAt (i);
    			
    			   //atualiza o profile simples    			
    			   Vector vet = new Vector ();
    			   vet.addElement ((BW_Profile)p);
    			   RProf = RProf && this.writeProfile (vet);
    			   //System.out.println ("[DBG]ST_DatabaseImplementor.writeAwarenessProfile: writing PROFILE: "+RProf);
    			    		
    			   //se gravou o awareness profine na tabela t_profile	
    			   //grava os profiles pessoais
    			   if (RProf) {    			
    			   	//tira os profiles pessoais registrados nele    			   	
    			   	delete = con.prepareStatement(SQLtext11 + " t_profile_personal "+ SQLtext12);
    			   	delete.setInt (1,p.getTypeId());
    			   	delete.setInt (2,p.getObjId());
    			   	int rs = delete.executeUpdate();    			   	
    			        //System.out.println ("[DBG] ST_DatabaseImplementor.write: deleted from t_profile_personal "+rs);
    			
	    			//recoloca os atuais profiles pessoais
    			        Enumeration en = p.getPersonalProfile().elements();
    			   	while (en.hasMoreElements()) {
    			   		boolean RPers1 = true;
    			   		BW_Profile pp = (BW_Profile) en.nextElement();
    			   		vet.removeAllElements();
    			   		vet.addElement (pp);
    			   		RPers1 = RPers1 && this.write (pp);
    			   		//System.out.println ("[DBG]ST_DatabaseImplementor.writeAwarenessProfile: writing personal profile: "+RPers1);
    			   		if (RPers1) {
    			   			insert = con.prepareStatement(SQLtext21+" t_profile_personal "+SQLtext22
    			   							+SQLtext23+SQLtext24);
    			   			insert.setInt (1,p.getTypeId());
    			   			insert.setInt (2,p.getObjId());
    			   			insert.setInt (3,pp.getTypeId());
    			   			insert.setInt (4,pp.getObjId());
    			   			int RS = insert.executeUpdate();
    			   			//System.out.println ("[DBG]ST_DatabaseImplementor.writeAwarenessProfile: writing t_profile_personal "+RS);
    			   			if (RS<=0) RPers1 = RPers1 && false;
    			   			else  RPers1 = RPers1 && true;	
    			   		}
    			   		RPers = RPers && RPers1;    			   		
	        	        }        	
	        	    }
	        	
	        	
	        	    //se gravou o awareness profile na tabela t_profile
	        	    //grava os profiles pessoais por papel
	        	    if (RProf) {
    			   	//tira os profiles pessoais por papel registrados nele
    			   	delete = con.prepareStatement(SQLtext11 + " t_profile_personal_paper "+ SQLtext12);
    			   	delete.setInt (1,p.getTypeId());
    			   	delete.setInt (2,p.getObjId());
    			   	int rs = delete.executeUpdate();    			
    			   	//System.out.println ("[DBG]ST_DatabaseImplementor.writeAwarenessProfile: removing t_profile_paper: "+rs);    			   	   	
    			
	    			//recoloca os atuais profiles pessoais po papel
	    			Hashtable ht = p.getPersonalPaperProfile();
    			        Enumeration en = ht.keys();
    			        //System.out.println ("[DBG] ST_DatabaseImplementor.write: hash "+ht);
    			   	while (en.hasMoreElements()) {
    			   		boolean RPap1 = true;
    			   		BW_Paper paper = (BW_Paper) en.nextElement();
    			   		BW_Profile pp = (BW_Profile) ht.get (paper);
    			   		
    			   		vet.removeAllElements();
    			   		vet.addElement (pp);
    			   		    			   		
    			   		RPap1 = RPap1 && this.write (pp);
    			   		//System.out.println ("[DBG]ST_DatabaseImplementor.writeAwarenessProfile: writing personal paper Profile: "+RPap1);		
    			   		if (RPap1) {
    			   			insert = con.prepareStatement(SQLtext21+" t_profile_personal_paper "+SQLtext22
    			   							+", paper_typeid, paper_objid "+SQLtext23+", ?, ? "+SQLtext24);
    			   			insert.setInt (1,p.getTypeId());
    			   			insert.setInt (2,p.getObjId());
    			   			insert.setInt (3,pp.getTypeId());
    			   			insert.setInt (4,pp.getObjId());
    			   			insert.setInt (5,paper.getTypeId());
    			   			insert.setInt (6,paper.getObjId());
    			   			int RS = insert.executeUpdate();
    			   			//System.out.println ("[DBG]ST_DatabaseImplementor.writeAwarenessProfile: writing t_profile_personal_paper: "+RS);
    			   			if (RS<=0) RPap1 = RPap1 && false;
    			   			else  RPap1 = RPap1 && true;	
    			   		}
    			   		//grava o papel
    			   		if (RPap1) {
    			   			vet.removeAllElements();
    			   			vet.addElement (paper);    			   			
    			   			boolean r1 = this.write(vet);
    			   			RPap1 = RPap1 && r1;
    			   			//System.out.println ("[DBG]ST_DatabaseImplementor.writeAwarenessProfile: writing t_paper: "+RPap1);
    			   		}
    			   		
    			   		RPap = RPap && RPap1;
	        	        }	        	      	        	            		        	        	                	
	        	    }	
	        	          	   		        	       	
	        	   //vai para o proximo elemento no vetor
	        	   ret = RProf && RPap && RPers;
	        	   //System.out.println ("[DBG]ST_DatabaseImplementor.writeAwarenessProfile: ret: "+ret);
    		        }
    		
    		        //this.showWarnings ("readAwarenessProfile");    				    			
	        	ret = true && ret;
	        	
    		} catch (SQLException ex) {
    		  	ret = false;
    		  	this.catchSQLException ("writeAwarenessProfile",ex);
    		} catch (Exception ex) {
			ret = false;
			this.catchException ("writeAwarenessProfile",ex);
    		}
        	
        	return ret;
  }


/* ------------------------------------------------------------------------- */




/////////////////////////////////////////////////////////////////////////////////////////

   /////////////////////////////// METODOS ////////////////////////////////


   /////////////////// READS

/* ------------------------------------------------------------------------- */

        /** faz leitura de dados, com base em uma String com um statement
         * SQL passado por parametro.
	 * @param 	String what	o que deve ser lido (String SQL)
	 * @return	Vector elementos lidos - cada posicao do vetor possui
	 * uma hashtable de nome do campo - valor, correspondente a cada uma
	 * das tuplas resgatadas
	 */
	public Vector read (String what) {
    		Vector ret = new Vector();
    		
    		try {
    		   Statement stmt = con.createStatement();
    		   ResultSet results = stmt.executeQuery (what);
    		   ResultSetMetaData rsmd = results.getMetaData();    		    		
    		    		       		
    		   int nfields = rsmd.getColumnCount();
    		
    		   boolean more = results.next ();
    		
    		   //System.out.println ("[DBG] ST_DatabaseImplementor.read: resultado obtido");
    		
    		   while (more) {    		
    		   	Hashtable ht = new Hashtable();
    		   	
    		   	for (int i=1;i<=nfields;i++) {
    		   		String key = rsmd.getColumnName(i);
    		   		String type = rsmd.getColumnTypeName (i);
    		   		Object value = null;
    		   		
    		   		//determina o tipo
    		   		if (type.equals ("text")) {
    		   			value = (Object) results.getString(i);
    		   		} else if (type.equals ("int2") || type.equals ("int4")) {
    		   		        value = (Object) new Integer (results.getInt (i));
    		   		} else if (type.equals ("int8")) {
    		   			value = (Object) new Long (results.getLong (i));
    		   		} else if (type.equals ("float4") || type.equals ("float8")) {
    		   			value = (Object) new Double (results.getDouble (i));
    		   		} else if (type.equals ("bool") ) {
    		   			value = (Object) new Boolean (results.getBoolean (i));
    		   		} else if (type.equals ("time")) {
    		   			Time t = (results.getTime (i));
    		   			Calendar c = Calendar.getInstance(TimeZone.getDefault(),Locale.getDefault());
    		   			c.setTime (t);
    		   			value = (Object) c;
    		   		} else if (type.equals ("timestamp")) {
    		   			Timestamp t = results.getTimestamp(i);
    		   			Calendar c = Calendar.getInstance(TimeZone.getDefault(),Locale.getDefault());
    		   			c.setTime (t);
    		   			value = (Object) t;
    		   		} else {
    		   			//System.out.println ("READ NONE");
    		   			value = results.getObject(i);
    		   		}
    		   		
    		   		ht.put (key,value);    		   		    		   		    		   			
    		   	}    		   	
    		   	
    		   	ret.addElement (ht);
    		   	more = results.next ();
    		   }
    		
    		   //mostra os warnings da ultima consulta
    		   this.showWarnings ("read");
    		
    		} catch ( SQLException ex ) {
         		this.catchSQLException ("read",ex);
     		} catch (java.lang.Exception ex) {			         	
     		        this.catchException ("read",ex);
    		}
    		
    		return ret;		
	}
	
/* ------------------------------------------------------------------------- */	

	/** faz a leitura do Setup
	 * @param	ST_Setup set	setup desejado
	 * @return	ST_Setup	setup com os dados atualizados
	 *				null se a leitura nao foi possivel
	 */
	public ST_Setup read (ST_Setup set) {
    		ST_Setup ret = null;
    		
    		try {
    			Statement stmt = con.createStatement();
    			String SQLtext = "SELECT * FROM t_setup";
			ResultSet results = stmt.executeQuery (SQLtext);

    		   	//mostra os warnings da ultima consulta
    		   	this.showWarnings ("read");
  			    			
			//System.out.println ("[DBG] ST_DatabaseImplementor.read: getting results");			
			boolean more = results.next ();
			if (more) {
    		        	int timelife = results.getInt ("timelife");
    		        	boolean isserver = results.getBoolean ("isserver");
    		        	String server = results.getString ("server");
    		        	int lastobjid = results.getInt ("lastobjid");
    		        	int port = results.getInt ("port");
    				ret = new ST_Setup();
    				ret.setTimeLife(timelife);
    				ret.setIsServer(isserver);
    				ret.setServer(server);
    				ret.setLastObjId(lastobjid);
    				ret.setPort(port);
    				//System.out.println ("[DBG] ST_DatabaseImplementor.read: results readed");
    			} 	
    			
    		} catch ( SQLException ex ) {
         		this.catchSQLException ("read",ex);
     		} catch (java.lang.Exception ex) {	
     			this.catchException ("read",ex);		         	
    		}
    		
    		return ret;	
	}

/* ------------------------------------------------------------------------- */
	
	
	/** Le um conjunto de elementos de um determinado tipo
	 * (obrigatoriamente deve ser uma subclasse de BW_Element)
	 * em um determinado intervalo.
	 * @param	BW_Element what		tipo de dado a ser lido
	 * @param	BW_TimeInterval when	intervalo a ser lido
	 */
	 public Vector read (BW_Element what, BW_TimeInterval when) {
    		Vector ret = null;
    		
    		try {
    			Class cl = what.getClass();
    			
    			//Se eh um BW_Group
    			if (Class.forName("BW.kernel.BW_Group").isAssignableFrom (cl))
    				ret = this.readGroup();
    			//Se eh um BW_Register
    			else if (Class.forName("BW.kernel.BW_Register").isAssignableFrom (cl))
    				ret = this.readRegister((BW_Register)what);    				
    			//Se eh um BW_Event
    			else if (Class.forName("BW.kernel.BW_Event").isAssignableFrom (cl))
    				ret = this.readEvents ((BW_Event)what, when);    				
    			//se eh um Awarenessprofile	
    			else if (Class.forName("BW.kernel.BW_AwarenessProfile").isAssignableFrom (cl))
    				ret = this.readAwarenessProfile ((BW_AwarenessProfile)what, when);    				
    				//ret = null;
    			//se eh um Profile	
    			else if (Class.forName("BW.kernel.BW_Profile").isAssignableFrom (cl))
    				ret = this.readProfile ((BW_Profile)what, when);
    				//ret=null;    				
    			//se eh um papel
    			else if (Class.forName("BW.kernel.BW_Paper").isAssignableFrom (cl))
    				ret = this.readPaper ((BW_Paper)what);
    				//ret=null;    				
    			//se eh um membro
    			else if (Class.forName("BW.kernel.BW_Member").isAssignableFrom (cl))
    				ret = this.readMember ((BW_Member)what, when);
    				//ret=null;    				
    			//se nao eh nenhum dos prontos	
    			else {  System.out.println ("READ NONE");
    				ret = null;    			
    			}	   			    		
    		} catch (Exception ex) {     		
    		        this.catchException ("read",ex);
    		}
    		
    		return ret;		
	
	 }
		

/* ------------------------------------------------------------------------- */
	
	 /** Le todos os elementos de um determinado tipo
	  * @param	BW_Element what		tipo de dado a ser lido
	  * @return	Vector	todos os dados daquele tipo lidos
	  */
	 public Vector read (BW_Element what) {
    		Vector ret = null;
    		
    		try {
    			Class cl = what.getClass();
    			
    			//Se eh um BW_Group
    			if (Class.forName("BW.kernel.BW_Group").isAssignableFrom (cl))
    				ret = this.readGroup();
    			//Se eh um BW_Register
    			else if (Class.forName("BW.kernel.BW_Register").isAssignableFrom (cl))
    				ret = this.readRegister((BW_Register)what);    				
			//Se eh um BW_Event
    			else if (Class.forName("BW.kernel.BW_Event").isAssignableFrom (cl))
    				ret = this.readEvents ((BW_Event)what);    				
    			//se eh um Awarenessprofile	
    			else if (Class.forName("BW.kernel.BW_AwarenessProfile").isAssignableFrom (cl))
    				ret = this.readAwarenessProfile ((BW_AwarenessProfile)what);    				
    				//ret = null;
    			//se eh um Profile	
    			else if (Class.forName("BW.kernel.BW_Profile").isAssignableFrom (cl))
    				ret = this.readProfile ((BW_Profile)what);    				    				
    			//se eh um Paper
    			else if (Class.forName("BW.kernel.BW_Paper").isAssignableFrom (cl))
    				ret = this.readPaper ((BW_Paper)what);    				    				
    			//se eh um Member	
    			else if (Class.forName("BW.kernel.BW_Member").isAssignableFrom (cl)) {
    				//System.out.println ("[DBG] ST_DatabaseImplementor.read (BW_Element): reading members..");
    				ret = this.readMember ((BW_Member)what);    				    				
    				//ret = null;
    			}	
			//se nao eh nenhum dos prontos    				
    			else {	System.out.println ("READ NONE");
    			 	ret = null;    			    			
    			 }	
    		
    		} catch (Exception ex) {     		
    			this.catchException ("read",ex);
    		}
    		
    		return ret;		
	
	 }
	
	 /** Le uma unica instancia de um tipo de elemento
	  * @param 	BW_Element what	tipo de dado a ser lido
	  * @param	int objId	identificador do objeto a ser resgatado
	  * @return	BW_Element	objeto lido
	  */
	 public BW_Element read (BW_Element what, int objId) {
    		BW_Element ret = null;
    		
    		try {
    			Class cl = what.getClass();
    			
    			//Se eh um BW_Group
    			if (Class.forName("BW.kernel.BW_Group").isAssignableFrom (cl)) {
    				what.setObjId (objId);
    				ret = (BW_Element) this.readGroup((BW_Group)what);
    			}
    			//Se eh um BW_Register
    			else if (Class.forName("BW.kernel.BW_Register").isAssignableFrom (cl)) {
    				what.setObjId (objId);
    				ret = this.readRegister((BW_Register)what,objId);    				
    			}	
    			//Se eh um BW_Event
    			else if (Class.forName("BW.kernel.BW_Event").isAssignableFrom (cl)) {
    				what.setObjId (objId);
    				ret = this.readEvents ((BW_Event)what,objId);    				
    			}	
    			//se eh um Awarenessprofile	
    			else if (Class.forName("BW.kernel.BW_AwarenessProfile").isAssignableFrom (cl)) {
    				ret = this.readAwarenessProfile ((BW_AwarenessProfile)what,objId);
    			}	
    			//se eh um Profile	
    			else if (Class.forName("BW.kernel.BW_Profile").isAssignableFrom (cl)) {
    				ret = this.readProfile ((BW_Profile)what, objId);
    			}	
    			//se eh um Paper
    			else if (Class.forName("BW.kernel.BW_Paper").isAssignableFrom (cl)) {
    				ret = this.readPaper ((BW_Paper)what, objId);
    			}	
    			//se eh um Member	
    			else if (Class.forName("BW.kernel.BW_Member").isAssignableFrom (cl)) {
    				ret = this.readMember ((BW_Member)what, objId);
    			}	
                        //se nao eh nenhum dos prontos
    			else{	System.out.println ("READ NONE");
    			 	ret = null;    			    			
    		        }
    		} catch (Exception ex) {     		
    			this.catchException ("read",ex);
    		}
    		return ret;		
	
	 }

	
    /////////////////// WRITES
    	

/* ------------------------------------------------------------------------- */

        /** escreve os dados, descritos por uma String contendo um
         * statement SQL, na base de dados.
	 * @param 	String what	o que deve ser escrito
	 * @return 	boolean		true  se foi gravado com exito
	 *				false caso contrario
	 */
	public boolean write (String what) {
    		boolean ret = false;
    		
    		try {
    		   Statement stmt = con.createStatement();
    		   stmt.executeUpdate (what);
    		
    		   //mostra os warnings da ultima consulta
    		   this.showWarnings ("write");
    		   ret = true;    		    		
    		
    		} catch (SQLException ex){
        		ret = false;
        		this.catchSQLException ("write",ex);
    		} catch (Exception ex) {
			ret = false;
			this.catchException ("write",ex);
    		}
    		
    		return ret;				
	}


/* ------------------------------------------------------------------------- */

		
	/** escreve um conjunto de dados em meio permanente.
	 * @note 	Estes dados devem obrigatoriamente ser de uma mesma subclasse de BW_Element
	 * @param 	Vector data	Vetor de BW_Elements com os dados a serem gravados
	 * @return	boolean		true se a operacao teve sucesso,
	 *				false caso contrario
	 */
	public boolean write (Vector data) {
    		boolean ret = false;
    		
    		try {
    			Object o = data.firstElement();
    			Class cl = o.getClass();
    			
    			//Se eh um BW_Group
    			if (Class.forName("BW.kernel.BW_Group").isAssignableFrom (cl)) {
    				ret = this.writeGroup(data);
    			}	
    			//Se eh um BW_Register
    			else if (Class.forName("BW.kernel.BW_Register").isAssignableFrom (cl)) {
    				ret = this.writeRegister(data);
    			}	
    			//Se eh um BW_Event
    			else if (Class.forName("BW.kernel.BW_Event").isAssignableFrom (cl)) {
    				ret = this.writeEvents (data);
    			}	    			
    			//Se eh um BW_AwarenessProfile
    			else if (Class.forName("BW.kernel.BW_AwarenessProfile").isAssignableFrom (cl)) {
    			        ret = this.writeAwarenessProfile(data);
    				//ret = false;
    			}	    			
    			//Se eh um BW_Profile
    			else if (Class.forName("BW.kernel.BW_Profile").isAssignableFrom (cl)) {
    				ret = this.writeProfile (data);
    			}	    			
    			//Se eh um BW_Paper
    			else if (Class.forName("BW.kernel.BW_Paper").isAssignableFrom (cl)) {
    				ret = this.writePaper (data);
    			}	    			
    			//Se eh um BW_Member
    			else if (Class.forName("BW.kernel.BW_Member").isAssignableFrom (cl)) {
    				ret = this.writeMember (data);
    			}	    			
    			else 	ret = false;    			    			
    		
    		} catch (Exception ex) {     		
    			this.catchException ("write",ex);
    		}
        	return ret;				
	}

        /** escreve um unico elemento na base de dados
         * @param	BW_Element what	elemento a ser gravado
         * @return	boolean		true se tudo ocorreu bem
         *				false caso contrario
         */
       public boolean write (BW_Element what) {
    		boolean ret = false;
    		
    		try {
    			Class cl = what.getClass();
    			Vector v = new Vector ();
    			
    			//Se eh um BW_Group
    			if (Class.forName("BW.kernel.BW_Group").isAssignableFrom (cl)) {
    				v.addElement ((BW_Group)what);    				
    				ret = this.writeGroup(v);
    			}
    			//Se eh um BW_Register
    			else if (Class.forName("BW.kernel.BW_Register").isAssignableFrom (cl)) {
    				v.addElement ((BW_Register)what);
    				ret = this.writeRegister(v);
    			}	
    			//Se eh um BW_Event
    			else if (Class.forName("BW.kernel.BW_Event").isAssignableFrom (cl)) {
    				v.addElement ((BW_Event)what);
    				ret = this.writeEvents (v);
    			}	    			
    			//Se eh um BW_AwarenessProfile
    			else if (Class.forName("BW.kernel.BW_AwarenessProfile").isAssignableFrom (cl)) {
    				v.addElement ((BW_AwarenessProfile)what);
    				ret = this.writeAwarenessProfile (v);    				
    				ret = false;
    			}	    			
    			//Se eh um BW_Profile
    			else if (Class.forName("BW.kernel.BW_Profile").isAssignableFrom (cl)) {    			
    				v.addElement ((BW_Profile)what);
    				ret = this.writeProfile (v);
    			}	    			
    			//Se eh um BW_Paper
    			else if (Class.forName("BW.kernel.BW_Paper").isAssignableFrom (cl)) {
    				v.addElement ((BW_Paper)what);
    				ret = this.writePaper (v);
    			}	    			
    			//Se eh um BW_Member
    			else if (Class.forName("BW.kernel.BW_Member").isAssignableFrom (cl)) {
    				v.addElement ((BW_Member)what);
    				ret = this.writeMember (v);
    			}	    			
    			else {  System.out.println ("WRITE NONE");
    				ret = false;    			    			
    		        }
    		} catch (Exception ex) {     		
    			this.catchException ("write",ex);
    		}
    		    		
    		return ret;		

       }

/* ------------------------------------------------------------------------- */


       /** atualiza ST_Setup na base de dados
        * @param ST_Setup
        */
       public boolean write (ST_Setup set) {
    		boolean ret = false;
    		
    		try {
    			String SQLtext = "UPDATE t_setup SET server = ? , isserver = ? , timelife = ? ";    			
    			SQLtext += ", lastobjid = ? , port = ? ";
    			PreparedStatement pstmt = con.prepareStatement(SQLtext);
    			
    			pstmt.setString  (1,set.getServer());
    			pstmt.setBoolean (2,set.getIsServer());
    			pstmt.setInt     (3,set.getTimeLife());
    			pstmt.setInt     (4,set.getLastObjId());
    			pstmt.setInt     (5,set.getPort());
    			
    			int ResultCode = pstmt.executeUpdate();
    		     	
    			//se nada foi atualizado faz a insercao na tabela
    			if (ResultCode<=0) {
    				String SQLtext2 = new String ("INSERT INTO t_setup ( server, isserver, timelife, lastobjid, port )");
    				SQLtext2 += "VALUES ( ? , ? , ? , ? , ? )";
    				pstmt = con.prepareStatement (SQLtext2);
    				pstmt.setString  (1,set.getServer());
    				pstmt.setBoolean (2,set.getIsServer());
    				pstmt.setInt     (3,set.getTimeLife());			
	    			pstmt.setInt     (4,set.getLastObjId());
    				pstmt.setInt     (5,set.getPort());
    				ResultCode = pstmt.executeUpdate();
    				if (ResultCode<=0)
    					ret = false;
    				else
    					ret = true;
    			}
    			else ret = true;
    			
    			//mostra os warnings da ultima consulta
    			this.showWarnings ("write");
    		
    		} catch (SQLException ex) {    		
    		  	ret = false;
    		  	this.catchSQLException ("write",ex);
    		} catch (Exception ex) {
			ret = false;
			this.catchException ("write",ex);
    		}
    		
    		return ret;		
       }

/* ------------------------------------------------------------------------- */

			
   ////////////// OUTRAS OPERACOES COM A BASE

/* ------------------------------------------------------------------------- */


        /** faz a abertura da base de armazenamento
	 * @return	boolean		true se abriu a base corretamente
	 *				false caso contrario
	 */
	public boolean open () {
	   initDB (url,user,passwd);
	   return this.getStatus();
	}

/* ------------------------------------------------------------------------- */


        /** fecha e conclui as operacoes sobre a base de dados
	 * @return 	boolean		true  se foi fechado com exito
	 *				false caso contrario
	 */
	public boolean close () {
	    return this._close();
	}

/* ------------------------------------------------------------------------- */


 	/** retorna o status da base de dados
	 * @return	boolean		true - se a base esta aberta
	 *				false - se ela esta fechada
	 */
	public boolean getStatus () {
	    try {
	      if (con.isClosed()) {
	        this.status = false;
	      }
	    } catch (Exception ex){
	    	this.status = false;
	    }
	    return this.status;    	
	}

/* ------------------------------------------------------------------------- */


   protected void catchSQLException (String metode, SQLException ex) {	
        String msg = "[DBG] ST_DatabaseImplementor.";
        msg += metode;
        msg += ": SQLException caught --- ";
        System.out.println (msg);
	while (ex != null) {
		System.out.println ("SQLState: " + ex.getSQLState ());	
		System.out.println ("Message:  " + ex.getMessage ());
		System.out.println ("Vendor:   " + ex.getErrorCode ());
		ex.printStackTrace();
		ex = ex.getNextException ();
		System.out.println ("");
	}
	
	try {	
		if (!con.getAutoCommit())
			con.rollback();			         	    		
	} catch (SQLException e) {	
	}	
   }		


/* ------------------------------------------------------------------------- */
   		
		
   protected void catchException (String metode, Exception ex)  {		
	String msg = "[DBG]ST_DatabaseImplementor.";
	msg += metode;
	msg += ": Exception caught ---- ";	
	System.out.println (msg);	
	ex.printStackTrace ();	
	try {	
		if (!con.getAutoCommit()) 	
			con.rollback();			         	    		
	} catch (SQLException e) {		
	}
  }


/* ------------------------------------------------------------------------- */
   protected void showWarnings (String metode) throws SQLException {   		
     	//mostra os warnings da ultima consulta
       	 System.out.println ("[DBG]ST_DatabaseImplementor."+metode+":searching for warnings");
	 SQLWarning warn = con.getWarnings();
       	 //while (warn!= null) {
       	 if (warn!=null) {
       		System.out.println ("WARNING: "+warn);
       		warn = 	con.getWarnings();
	 }
	con.clearWarnings();    		
   }


   ////////////////////////////////////////////////////////////////////////////



}
