/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: COPSE.CLIENT
 * Classe:  CClient
\******************************************************************************/

package COPSE.CLIENT;

import java.io.*;
import java.util.*;
import java.net.*;


import COPSE.*;
import COPSE.CONTROL.*;
import FILE.*;
//import PROCESS.*;

/* ************************************** BW ******************************************************* */
import BW.*;
import BW.kernel.*;
/* ************************************** BW ******************************************************* */

public class CClient
    extends Thread
{
    protected CEventConnector         pServerConnector;

    protected CClientEventHandler     pEventHandler;


    protected CClientSessionManager   pSessionManager;
    protected Vector                  pSessionManagerListeners;

    protected CFile                   pFile;
    protected CFileController         pFileController;
    protected Vector                  pFileControllerListeners;

    protected CClientCollaborationManager   pCollabManager;


    protected int                     mConnectionId;
    protected int                     mUserId;
    protected String                  mUserName;
    protected int                     mServerPort;

    boolean                           mExit;
    Boolean                           mIsReady;


/* ************************************** BW ******************************************************* */
    protected String                  BWServer;
/* ************************************** BW ******************************************************* */


    protected static int GetToolId()        { return 0; }


    //* Constructors...
    public CClient( int userId, String host, int port )
        throws IOException
    {   // temporary
        this( userId, Integer.toString(userId), host, port );
    }

    public CClient( int userId, String name, String host, int port )
        throws IOException
    {
        super();


/* ************************************** BW ******************************************************* */
        BWServer = new String (host);
/* ************************************** BW ******************************************************* */

        mUserId = userId;
        mUserName = name;
        mExit = false;
        mIsReady = new Boolean(false);

        // Init Connection to Server
        System.err.println( "[DEBUG] CClient - Init connection to Server ..." );
        CreateConnectionToServer( host, port );
        mServerPort = port;

        pSessionManagerListeners = new Vector();
        pFileControllerListeners = new Vector();

        // Init Collaboration Manager
        pCollabManager = new CClientCollaborationManager( this );
        pCollabManager.start();
    }

    protected void CreateConnectionToServer ( String host, int port )
        throws IOException
    {
        System.err.println( "[DEBUG] CClient - creating client socket..." );
        Socket pSocket = new Socket( host, port );
        System.err.println( "[DEBUG] CClient - creating client socket object streams..." );
        pServerConnector = CSocketStreamConnector.New( pSocket, false );
        System.err.println( "[DEBUG] CClient - created client socket streams" );
    }






    //* Methods Called during thread running...
    protected void StartCommunicationToServer()
        throws IOException
    {

        // Start Communication to Process Server
        System.err.println( "[DEBUG] CClient - Sending 'open_connection' to Server ..." );
        CEvent event = new CEvent( CEvent.EvCOPSE_Connection_Open );
        event.AddArgument( new Integer( mUserId ) );
        event.AddArgument( new Integer( GetToolId() ) );
        event.AddArgument( mUserName );
        pServerConnector.Send( event );

        System.err.println( "[DEBUG] CClient - Receiving 'open_connection' from Server ..." );
        event = pServerConnector.Receive();
        System.err.println( "[DEBUG] CClient - Received an event from Server ..." );

        switch( event.GetType() )
        {
            case CEvent.EvCOPSE_Connection_Open:
                Integer conn = (Integer) event.GetArgumentAt(0);
                Integer uId  = (Integer) event.GetArgumentAt(1);
                Integer tId  = (Integer) event.GetArgumentAt(2);
                if( (uId.intValue() == mUserId) && (tId.intValue() == GetToolId()) )
                {
                    SetConnectionId( conn.intValue() );
                    if( !InitClientConnection() )
                        throw new IOException( "Error on starting client connection to the server" );
                }
                else
                    throw new IOException( "Error on starting client connection to the server" );


 /* ************************************** BW ******************************************************* */
		boolean BWStart = GW_BW_Mediator.Start(BWServer);
		System.out.println ("[DBG] COPSE.CLIENT.CClient.StartCommunicationToServer: "+
					" Starting GW_BW_Mediador "+BWStart );
		
		BW_Member BWMember = new BW_Member (typeIdDefault.BW_MEMBER,mUserId);
		BWMember.setLogin (mUserName);
		System.out.println ("[DBG] COPSE.CLIENT.CClient.StartCommunicationToServer: "+
					" Logging "+BWMember.getLogin());
		GW_BW_Mediator.UserLogin (BWMember);
					
		BW_Event BWEvent = new BW_Event (CEvent.EvCOPSE_Connection_Open,typeIdDefault.UNSET);
		BWEvent.setDescription ("Sessao de um usuario - " + mUserName);
		BWEvent.setDetails ("Usuario "+mUserName+" participou da sessao CUTE");		
		GW_BW_Mediator.BeginEvent (CEvent.EvCOPSE_Connection_Open,BWEvent);
		
 /* ************************************** BW ******************************************************* */

                break;

            case CEvent.EvCOPSE_Connection_Error:
                String msg = (String) event.GetArgumentAt(0);
                throw new IOException( msg );

            default:
                throw new IOException( "Invalid server answer to connection request!" );
        }
    }

    protected boolean InitClientConnection()
    {
        // Init Event Handler
        pEventHandler = CreateClientEventHandler();
        pEventHandler.start();

/* // Moved to Constructor
        // Init Collaboration Manager
        pCollabManager = new CClientCollaborationManager( this );
        pCollabManager.start();
*/
        return true;
    }

    protected CClientEventHandler  CreateClientEventHandler()
    {
        return new CClientEventHandler( this );
    }


    protected boolean SendInitialCommands()
    {
        CEvent event;
        try {
            // Init File and FileController
            event = new CEvent( CEvent.EvCOPSE_FileController_Load );
            pServerConnector.Send( event );
            // Processes an event just not to "overflow" the socket
            event = pServerConnector.Receive();
            pEventHandler.AddEvent( event );

            // Load Users in Session
            event = new CEvent( CEvent.EvCOPSE_SessionManager_Load );
            pServerConnector.Send( event );
            // Processes an event just not to "overflow" the socket
            event = pServerConnector.Receive();
            pEventHandler.AddEvent( event );

            // TODO:
            // Send initial event for Collaboration Handlers ...

        } catch ( IOException e )
        {
            return false;
        };
        return true;
    }




    //* Client Thread Execution
    public void run()
    {
        try {
            StartCommunicationToServer();
        } catch( IOException e )
        {
            e.printStackTrace( System.err );
            return;
        }

        SendInitialCommands();

        System.err.println( "DEBUG: Client Is Ready..." );

        synchronized( mIsReady )
        {
            mIsReady.notifyAll();
            mIsReady = new Boolean(true);
        }


        while( !mExit )
        {
            try{
                CEvent event = pServerConnector.Receive();
                pEventHandler.AddEvent( event );
            } catch(IOException ioe)
            {
                if( mExit )
                {
                    // Ok! Exit requested...
                    break;
                }
                else
                {
                    // TODO: Verify if the connection is still alive
                }
            }
            catch(Exception e) {}
        } // while( !mExit )
    }



    public void WaitUntilReady()
    {
        synchronized( mIsReady )
        {
            //while( !mIsReady.booleanValue() )
                try{
                    mIsReady.wait();
                } catch (Exception e) {}
        }
    }



    //* Save
    public void Save()
    {
        try{
            CEvent event = new CEvent( CEvent.EvCOPSE_FileController_Save );
            Send( event );

/* ************************************** BW ******************************************************* */

            BW_Event BWEvent  =  new BW_Event (CEvent.EvCOPSE_FileController_Save,typeIdDefault.UNSET);
            BWEvent.setDetails ("Diagrama de classes consolidado");
            GW_BW_Mediator.BeginEvent (CEvent.EvCOPSE_FileController_Save,BWEvent);
	    GW_BW_Mediator.EndEvent (CEvent.EvCOPSE_FileController_Save);
	
/* ************************************** BW ******************************************************* */
        } catch( Exception e ) {};
    }

    //* Exit
    public void Exit()
    {
        CEvent event = new CEvent( CEvent.EvCOPSE_Connection_Close );

        System.err.println( "[DEBUG] CClient - Exit: Sending Connection_Close..." );

/* ************************************** BW ******************************************************* */
	GW_BW_Mediator.EndEvent (CEvent.EvCOPSE_Connection_Open);
	
	BW_Member BWMember = new BW_Member (typeIdDefault.BW_MEMBER,mUserId);
	BWMember.setLogin (mUserName);	
	System.out.println ("[DBG] COPSE.CLIENT.CClient.Exit: LOGOUT "+BWMember.getLogin());
	GW_BW_Mediator.UserLogout(BWMember);
	
	boolean BWEnd = GW_BW_Mediator.Stop();
	System.out.println ("[DBG] COPSE.CLIENT.CClient.Exit: End Event "+BWEnd);
	

/* ************************************** BW ******************************************************* */

        try{
            pServerConnector.Send( event );
        } catch( Exception e ) {};
    }

    // This function should be called by the EventHandler thread, and not by
    // any other thread !!!
    public void _Exit()
    {
        System.err.println( "[DEBUG] CClient - _Exit: mExit == true" );
        mExit = true;
        pServerConnector.Close();

        System.err.println( "[DEBUG] CClient - Exiting Collaboration Manager..." );
        pCollabManager.Exit();

        System.err.println( "[DEBUG] CClient - Stopping CClient thread..." );
        this.stop();

        // At last, stop the current thread...
        System.err.println( "[DEBUG] CClient - Stopping EventHandler thread..." );
        pEventHandler.stop();
    }




    //* Collaboration Manager
    public CClientCollaborationManager GetCollaborationManager()
    {   return pCollabManager; }




    //* Session Manager
    public CClientSessionManager GetSessionManager()     { return pSessionManager; }
    public void                  SetSessionManager(CClientSessionManager smgr)
    {   pSessionManager = smgr; }

    public void     AddSessionManagerListener( CClientSessionManagerListener sessionListener )
    {   pSessionManagerListeners.addElement( sessionListener );    }

    public void     RemoveSessionManagerListener( CClientSessionManagerListener sessionListener )
    {   pSessionManagerListeners.removeElement( sessionListener );    }

    public Enumeration  GetSessionManagerListeners()    { return pSessionManagerListeners.elements(); }



    // * File Controller
    public CFileController      GetFileController()     { return pFileController; }
    public void                 SetFileController(CFileController ctrl)
    {
        pFileController = ctrl;
        pFile = pFileController.GetFile();
    }

    public void     AddFileControllerListener( CClientFileControllerListener fileListener )
    {   pFileControllerListeners.addElement( fileListener );    }

    public void     RemoveFileControllerListener( CClientFileControllerListener fileListener )
    {   pFileControllerListeners.removeElement( fileListener );    }

    public Enumeration  GetFileControllerListeners()    { return pFileControllerListeners.elements(); }



    //*  Client Basic objects...
    public int                  GetServerPort()         { return mServerPort; }
    public int                  GetConnectionId()       { return mConnectionId; }
    public int                  GetUserId()             { return mUserId; }
    public String               GetUserName()           { return mUserName; }


    protected   void    SetConnectionId (int connId)    { mConnectionId = connId; }
    public      void    SetUserName     (String name)   { mUserName = name; }


    //* Send Events to Server
    public void     Send( CEvent event )
        throws IOException
    {
        if( pServerConnector != null )
            pServerConnector.Send( event );
    }


    //*  Debug...
    public void DEBUG_write( PrintStream pStr )
    {
        pStr.println( "CClient [toolId=" + GetToolId() + "| userId="+mUserId+"| connId="+mConnectionId+"]" );

        pStr.println( "SessionManager:");
        pSessionManager.DEBUG_write( pStr );

        pStr.println( "FileController:");
        //pFileController.DEBUG_write( pStr );
    }


}


