/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * package: COPSE.CLIENT
 * Classe:  CClientEventHandler
\******************************************************************************/

package COPSE.CLIENT;

import java.io.*;
//import java.net.*;
import java.util.*;


import COPSE.*;
import COPSE.CONTROL.*;
import FILE.*;


public class CClientEventHandler
    extends CEventHandler
{
    protected CClient                       pClient;
    protected CClientSessionManager         pSessionManager;
    protected CClientCollaborationManager   pCollaborationManager;


    public CClientEventHandler( CClient client )
    {
        super();
        pClient = client;
        pCollaborationManager = pClient.GetCollaborationManager();
        pSessionManager = pClient.GetSessionManager();
    }


    protected final void NotifyFileControllerListeners( int elemId, CEvent event )
    {
        // Listeners
        Enumeration e = pClient.GetFileControllerListeners();
        while( e.hasMoreElements() )
        {
            CClientFileControllerListener listener = (CClientFileControllerListener) e.nextElement();
            FileControllerChanged( listener, elemId, event );
        }

    }

    protected   void FileControllerChanged( CClientFileControllerListener listener, int elemId, CEvent event )
    {
        int type = event.GetType();

        //*
        switch( type )
        {
            case CEvent.EvCOPSE_FileController_Load:
                listener.FileControllerLoaded();
                break;
/*
            case CEvent.EvCOPSE_FileController_Element_Lock:
                //listener.ElementLocked( elemId );
                //break;
            case CEvent.EvCOPSE_FileController_Element_Unlock:
                //listener.ElementUnlocked( elemId );
                //break;
            case CEvent.EvCOPSE_FileController_Element_New:
                //listener.ElementCreated( elemId );
                //break;
            case CEvent.EvCOPSE_FileController_Element_Delete:
                //listener.ElementDeleted( elemId );
*/
            default:
                listener.ElementStateChanged( elemId, event );
                break;

        }
        //*/
    }


    protected void ProcessEvent( CEvent event )
    {
        int type = event.GetType();
        try {

        switch( type )
        {
            case CEvent.EvCOPSE_Connection_Open:
                // New User Connection
                {
                    if( pSessionManager == null ) return; // ignore event...

                    System.err.println( "DEBUG: ClientEventHandler - New user connection" );
                    Integer conn = (Integer) event.GetArgumentAt(0);
                    Integer uId  = (Integer) event.GetArgumentAt(1);
                    Integer tId  = (Integer) event.GetArgumentAt(2);
                    String  name = (String)  event.GetArgumentAt(3);

                    CUserSession user = new CUserSession( uId.intValue(), conn.intValue() );
                    user.SetUserName( name );
                    CToolSession tool = pSessionManager.GetToolSession( tId.intValue() );
                    tool.AddUserSession( user );

                    // Listeners
                    Enumeration e = pClient.GetSessionManagerListeners();
                    while( e.hasMoreElements() )
                    {
                        CClientSessionManagerListener listener = (CClientSessionManagerListener) e.nextElement();
                        listener.ConnectionOpened( conn.intValue() );
                    }

                }
                break;

            case CEvent.EvCOPSE_Connection_Close:
                // User Disconnection
                {
                    Integer userId = (Integer) event.GetArgumentAt(0);
                    Integer toolId = (Integer) event.GetArgumentAt(1);

                    if( ( userId.intValue() == pClient.GetUserId() ) &&
                        ( toolId.intValue() == pClient.GetToolId() ) )
                    {
                        pClient._Exit();
                    }
                    else
                    {
                        CToolSession tool = pSessionManager.GetToolSession( toolId.intValue() );
                        CUserSession user = tool.FindUserSession( userId.intValue() );
                        tool.RemoveUserSession( user );

                        // Listeners
                        try {
                            Enumeration e = pClient.GetSessionManagerListeners();
                            while( e.hasMoreElements() )
                            {
                                CClientSessionManagerListener listener = (CClientSessionManagerListener) e.nextElement();
                                listener.ConnectionClosed( user.GetConnectionId() );
                            }
                        } catch( Exception exc ){};
                    }
                }
                break;



            // ....
            case CEvent.EvCOPSE_FileController_Element_Lock:
                {
                    int connId = ((Integer)event.GetArgumentAt(0)).intValue();
                    int elemId = ((Integer)event.GetArgumentAt(1)).intValue();

                    if( pClient.GetFileController().LockElement( connId, elemId ) )
                        NotifyFileControllerListeners( elemId, event );
                }
                break;

            case CEvent.EvCOPSE_FileController_Element_Unlock:
                {
                    int connId = ((Integer)event.GetArgumentAt(0)).intValue();
                    int elemId = ((Integer)event.GetArgumentAt(1)).intValue();

                    if( pClient.GetFileController().UnlockElement( connId, elemId ) )
                        NotifyFileControllerListeners( elemId, event );
                }
                break;

            case CEvent.EvCOPSE_FileController_Element_New:
                {
                    int connId = ((Integer)event.GetArgumentAt(0)).intValue();
                    CElement element = (CElement) event.GetArgumentAt(1);

                    pClient.GetFileController().NewElement( connId, element );
                    NotifyFileControllerListeners( element.GetId(), event );
                }
                break;

            case CEvent.EvCOPSE_FileController_Element_Delete:
                {
                    int connId = ((Integer)event.GetArgumentAt(0)).intValue();
                    int elemId = ((Integer)event.GetArgumentAt(1)).intValue();

                    if( pClient.GetFileController().CanRemoveElement( connId, elemId ) )
                    {   // Notify the application before removing it to allows some others needed cleaning operations...
                        NotifyFileControllerListeners( elemId, event );
                        pClient.GetFileController().DeleteElement( connId, elemId );
                    }
                }
                break;

            case CEvent.EvCOPSE_FileController_Element_Update:
                {
                    int connId = ((Integer)event.GetArgumentAt(0)).intValue();
                    CElement element = (CElement) event.GetArgumentAt(1);

                    if( pClient.GetFileController().UpdateElement( connId, element ) )
                        NotifyFileControllerListeners( element.GetId(), event );
                }
                break;





            case CEvent.EvCOPSE_FileController_Load:
                {
                    pClient.SetFileController( (CFileController) event.GetArgumentAt(0) );
                    NotifyFileControllerListeners( 0, event );

                    /*  / Listeners
                    Enumeration e = pClient.GetFileControllerListeners();
                    while( e.hasMoreElements() )
                    {
                        CClientFileControllerListener listener = (CClientFileControllerListener) e.nextElement();
                        listener.FileControllerLoaded();
                    }
                    /* */
                }
                break;

            case CEvent.EvCOPSE_SessionManager_Load:
                {
                    pSessionManager = (CClientSessionManager) event.GetArgumentAt(0);
                    pSessionManager.SetClientSession( pClient.GetConnectionId() );
                    pClient.SetSessionManager( pSessionManager );

                    // Listeners
                    Enumeration e = pClient.GetSessionManagerListeners();
                    while( e.hasMoreElements() )
                    {
                        CClientSessionManagerListener listener = (CClientSessionManagerListener) e.nextElement();
                        listener.SessionManagerLoaded();
                    }
                }
                break;

            case CEvent.EvCOPSE_Server_Exit:
                // Server is exiting... (no more communications will be possible)
                {
                    System.err.println( "DEBUG: ClientEventHandler - Server is exiting..." );
                    pClient._Exit();
                }
                break;

            default:
                // Verify if its a collaboration event
                if( (type >= CEvent.EvCollaboration_First) &&
                    (type <= CEvent.EvCollaboration_Last) )
                {
                    //System.err.println( "[DEBUG]         - It's a collaboration event..." );
                    if( pCollaborationManager == null )
                    {
                        if( (pCollaborationManager = pClient.GetCollaborationManager()) == null )
                            return;
                    }

                    //System.err.println( "[DEBUG]         - Sending to the CollaborationManager" );
                    pCollaborationManager.AddEvent( event );
                }
                else
                // Ignore the event
                {}
        }

        } catch( Exception otherException )
        {
            System.err.println( "[DEBUG] CClientEventHandler - Exception..." );
            otherException.printStackTrace( System.err );
        }
    }

}