/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * package: COPSE
 * Classe:  CSessionManager
\******************************************************************************/

package COPSE;

import java.io.*;
import java.util.*;


public class CSessionManager
    implements Serializable
{
    protected Vector    pToolSessions;

    public CSessionManager()
    {
        pToolSessions = new Vector();
    }

    public synchronized CToolSession FindToolSession( int toolId )
    {
        Enumeration e = pToolSessions.elements();
        while( e.hasMoreElements() )
        {
            CToolSession session = (CToolSession) e.nextElement();
            if( session.GetToolId() == toolId )
                return session;
        }
        return null;
    }

    public synchronized CToolSession GetToolSession( int toolId )
    {
        CToolSession session = FindToolSession( toolId );
        if( session == null )
        {
            session = new CToolSession( toolId );
            pToolSessions.addElement( session );
        }
        return session;
    }

    public synchronized void AddToolSession( CToolSession session )
    {
        if( session == null ) return;

        CToolSession toolSession = FindToolSession( session.GetToolId() );
        if( toolSession == null )
            pToolSessions.addElement( session );
    }

    public void RemoveToolSession( int toolId )
    {
        CToolSession toolSession = FindToolSession( toolId );
        if( toolSession != null )
            pToolSessions.removeElement( toolSession );
    }

    public Enumeration GetToolSessions()
    {
        return pToolSessions.elements();
    }

    public int GetTotalSessions()
    {
        return pToolSessions.size();
    }


    public void DEBUG_write( PrintStream pStr )
    {
        pStr.println( "CSessionManager:" );
        Enumeration e = pToolSessions.elements();
        while( e.hasMoreElements() )
        {
            CToolSession session = (CToolSession) e.nextElement();
            session.DEBUG_write( pStr );
        }
    }


    public CUserSession FindSessionByConnection( int connectionId )
    {
        Enumeration e = pToolSessions.elements();
        while( e.hasMoreElements() )
        {
            CToolSession toolSession = (CToolSession) e.nextElement();
            CUserSession userSession = toolSession.FindSessionByConnection( connectionId );
            if( userSession != null )
                return userSession;
        }
        return null;
    }

    public boolean RemoveSession( int connectionId )
    {
        Enumeration e = pToolSessions.elements();
        while( e.hasMoreElements() )
        {
            CToolSession toolSession = (CToolSession) e.nextElement();
            CUserSession userSession = toolSession.FindSessionByConnection( connectionId );
            if( userSession != null )
            {
                toolSession.RemoveUserSession( userSession );
                if( toolSession.GetTotalSessions() == 0 )
                    pToolSessions.removeElement( toolSession );
                return true;
            }
        }
        return false;
    }
}
