/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * package: COPSE.SERVER
 * Classe:  CServerCollaborationManager
\******************************************************************************/

package COPSE.SERVER;

import java.io.*;
import java.net.*;
import java.util.*;


import COPSE.*;
import COPSE.CONTROL.*;
import FILE.*;


public class CServerCollaborationManager
    extends  CEventHandler
{
    protected   CServer         pServer;

    protected   CServerCommunicationChatHandler     pChatHandler;
    protected   CServerAwarenessPointerHandler      pPointerHandler;
    protected   CServerAwarenessScreenHandler       pScreenHandler;

    public CServerCollaborationManager( CServer server )
    {
        pServer = server;

        pChatHandler    = new CServerCommunicationChatHandler(this);
        pPointerHandler = new CServerAwarenessPointerHandler(this);
        pScreenHandler  = new CServerAwarenessScreenHandler(this);

        pChatHandler.start();
        pPointerHandler.start();
        pScreenHandler.start();
    }

    public CServer  GetServer()     { return pServer; }

    public void ProcessEvent( CEvent event, CConnectionHandler handler )
    {
        int type = event.GetType();

        if( type == CEvent.EvCollaboration_Communication_Chat_Message )
            pChatHandler.AddEvent( event, handler );
        else
        if( (type >= CEvent.EvCollaboration_Awareness_Pointer_First) &&
            (type <= CEvent.EvCollaboration_Awareness_Pointer_Last) )
            pPointerHandler.AddEvent( event, handler  );
        else
        if( (type >= CEvent.EvCollaboration_Awareness_Screen_First) &&
            (type <= CEvent.EvCollaboration_Awareness_Screen_Last) )
            pScreenHandler.AddEvent( event, handler  );
        else
        if( type == CEvent.EvCollaboration_Communication_Call_Message )
        {
            try {
                int toUser = ( (Integer) event.GetArgumentAt(1) ).intValue();

                CUserConnectionHandler user = (CUserConnectionHandler) handler;
                CServerUserSession userSession = (CServerUserSession) user.GetUserSession();
                CServerToolSession tool = (CServerToolSession) userSession.GetToolSession();

                if( toUser == 0 )
                    tool.NotifyAllExcept( userSession, event );
                else
                    tool.NotifyUser( toUser, event );
                System.err.println( "[DEBUG] CServerCollaborationManager - Call Message sent to user..." );
            } catch( Exception e ) {}
        }

    }

    public void Exit()
    {
        pChatHandler.Exit();
        pPointerHandler.Exit();
        pScreenHandler.Exit();

        this.stop();
    }


}