/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * package: COPSE.SERVER
 * Classe:  CServerEventHandler
\******************************************************************************/

package COPSE.SERVER;

import java.io.*;
//import java.net.*;
import java.util.*;

//****************** Tests by Manu **********************
import BW.Log;
//*******************************************************

import COPSE.*;
import COPSE.CONTROL.*;
import FILE.*;

public class CServerEventHandler
    extends  CEventHandler
{
    protected CServer                           pServer;
    protected CServerSessionManager             pSessionManager;
    protected CServerCollaborationManager       pCollaborationManager;

    //****************** Tests by Manu **********************
    Log mLog;
    //*******************************************************

    public CServerEventHandler( CServer server )
    {
        super();
        pServer = server;
        pSessionManager = pServer.GetSessionManager();
        pCollaborationManager = pServer.GetCollaborationManager();

       //****************** Tests by Manu **********************
        this.BWregister();
       //*******************************************************

    }


    protected void ProcessEvent( CEvent event, CConnectionHandler handler )
    {
        System.err.println( "[DEBUG] CServerEventHandler: Event ["+event.GetType()+"]" );

       //****************** Tests by Manu **********************
        if (!this.BWnewEvent (event)) System.err.println("probs com o logs");
       //*******************************************************

        CUserConnectionHandler userHandler = null;
        if( handler instanceof CUserConnectionHandler )
            userHandler = (CUserConnectionHandler) handler;

        int type = event.GetType();
        switch( type )
        {
            case CEvent.EvCOPSE_Connection_Close:
                // User requested to close conection
                {
                    if( userHandler == null ) break;

                    int connId = handler.GetConnectionId();
                    CUserSession user = userHandler.GetUserSession();
                    CToolSession tool = user.GetToolSession();

                    CEvent ev = new CEvent( CEvent.EvCOPSE_Connection_Close );
                    ev.AddArgument( new Integer(user.GetUserId()) );
                    ev.AddArgument( new Integer(tool.GetToolId()) );
                    pSessionManager.NotifyAll( ev );

                    pSessionManager.RemoveSession( connId );

                    // Close the connectionHandler to client
                    userHandler.Close();

                    // Verify if there is anybody in this server...
                    /*
                    if( pSessionManager.GetTotalSessions() == 0 )
                    {}
                    */
                }
                break;



            // ....
            case CEvent.EvCOPSE_FileController_Element_Lock:
                // User Requested FileController to lock element
                {
                    int connId = handler.GetConnectionId(); // = event.GetArgumentAt(0)
                    int elemId = ((Integer)event.GetArgumentAt(1)).intValue();

                    if( pServer.GetFileController().LockElement( connId, elemId ) )
                        pSessionManager.NotifyAll( event );
                }
                break;

            case CEvent.EvCOPSE_FileController_Element_Unlock:
                // User Requested FileController to unlock element
                {
                    int connId = handler.GetConnectionId(); // = event.GetArgumentAt(0)
                    int elemId = ((Integer)event.GetArgumentAt(1)).intValue();

                    if( pServer.GetFileController().UnlockElement( connId, elemId ) )
                        pSessionManager.NotifyAll( event );
                }
                break;

            case CEvent.EvCOPSE_FileController_Element_New:
                {
                    int connId = handler.GetConnectionId(); // = event.GetArgumentAt(0)
                    CElement element = (CElement) event.GetArgumentAt(1);

                    pServer.GetFileController().NewElement( connId, element );
                    pSessionManager.NotifyAll( event );
                }
                break;

            case CEvent.EvCOPSE_FileController_Element_Delete:
                // User Requested FileController to unlock element
                {
                    int connId = handler.GetConnectionId(); // = event.GetArgumentAt(0)
                    int elemId = ((Integer)event.GetArgumentAt(1)).intValue();

                    if( pServer.GetFileController().DeleteElement( connId, elemId ) )
                        pSessionManager.NotifyAll( event );
                }
                break;

            case CEvent.EvCOPSE_FileController_Element_Update:
                {
                    int connId = handler.GetConnectionId(); // = event.GetArgumentAt(0)
                    CElement element = (CElement) event.GetArgumentAt(1);

                    if( pServer.GetFileController().UpdateElement( connId, element ) )
                        pSessionManager.NotifyAll( event );
                }
                break;


            case CEvent.EvCOPSE_FileController_Save:
                // User Requested to save File
                {
                    try {
                        pServer.GetFileController().GetFile().Save();
                    } catch(Exception e) {}
                }
                break;


            case CEvent.EvCOPSE_FileController_Load:
                // User Requested the FileController
                {
                    CEvent ev = new CEvent( type );
                    ev.AddArgument( pServer.GetFileController() );
                    try {
                        handler.Send( ev );
                    } catch( IOException e ){};
                }
                break;

            case CEvent.EvCOPSE_SessionManager_Load:
                // User Requested the SessionManager
                {
                    CEvent ev = new CEvent( type );
                    ev.AddArgument( pSessionManager.CloneAsClientSessionManager() );
                    try {
                        handler.Send( ev );
                    } catch( IOException e ){};
                }
                break;


            default: ;
                if( (type >= CEvent.EvCollaboration_First) &&
                    (type <= CEvent.EvCollaboration_Last) )
                {
                    System.err.println( "[DEBUG]         - It's a collaboration event..." );
                    if( pCollaborationManager == null )
                    {
                        if( (pCollaborationManager = pServer.GetCollaborationManager()) == null )
                            return;
                    }

                    System.err.println( "[DEBUG]         - Sending to the CollaborationManager" );
                    pCollaborationManager.AddEvent( event, handler );
                }
                else
                {
                    // Pass the event to everyone ?? or Ignore ??
                    //pSessionManager.NotifyAll( event );
                }
        }
    }

//****************** Tests by Manu **********************
   //faz o registro dos eventos - abre o log
   protected void BWregister () {
        this.mLog = new Log ("CServerEventHandlerLog");
   }

   //ocorrencia de um evento - escreve no log
   public boolean BWnewEvent (CEvent event) {
         mLog.write (this.getClass().getName()+": Event ["+event.GetType()+"]");
         Enumeration  en = event.GetArguments();
         while (en.hasMoreElements())
	    mLog.write ("------ Elemento:"+en.nextElement().toString() );
         return (mLog.getStatus());
   }
//*******************************************************



}
