/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: FILE
 * Classe:  CFile
\******************************************************************************/

package FILE;


import java.io.*;
import java.util.*;


/* ******

public CFileSystem



********/



public class CFile
    implements Serializable
{
    //** Attributes
    private String        mName;
    private String        mPath;
    private String        mDescription;

    // Compound of elements
    private int           mLastElemId;
    private Vector        pElements;

    // has links: (in->in) or (in->out)
    private int           mLastLinkId;
    private Vector        pLinks;

    // TODO: add other attributes here
    private CHistory      pHistory;
    private boolean       pModified;


    //** Constructors
    public CFile()
    {
        Reset( "", "" );
    }

    public CFile( String name, String path )
    {
        Reset( name, path );
    }

    private void Reset( String name, String path )
    {
        mName = name;
        mPath = path;
        mDescription = "";

        mLastElemId = 0;
        pElements = new Vector();

        mLastLinkId = 0;
        pLinks = new Vector();

        //...
        pHistory = new CHistory(this);
        pModified = true;
    }


    //** Methods to Attributes
    public void     SetName( String name )                  { mName = name; pModified = true; }
    public void     SetPath( String path )                  { mPath = path; pModified = true; }
    public void     SetDescription( String desc )           { mDescription = desc; pModified = true; }

    public String   GetName( )                              { return mName; }
    public String   GetPath( )                              { return mPath; }
    public String   GetDescription( )                       { return mDescription; }

    public CHistory GetHistory()                            { return pHistory; }

    public boolean  IsModified()                            { return pModified; }



    //**  Elements
    public Enumeration  GetElements()                       { return pElements.elements(); }


    public void     AddElement( CElement elem )
    {
        elem.SetId( ++mLastElemId );
        pElements.addElement( elem );

        pModified = true;
        // TODO: history... (not here, but who called this method)
    }

    public CElement     GetElement ( int id )
    {
        Enumeration e = pElements.elements();
        while( e.hasMoreElements() )
        {
            CElement elem = (CElement) e.nextElement();
            if( elem.GetId() == id )
                return elem;
        }
        return null;
    }

    public void  RemoveElement( CElement elem )
    {
        // remove elem or mark as deleted ??
        pElements.removeElement( elem );

        // remove links ???

        pModified = true;
        // TODO: history... (not here, but who called this method)
    }



    //**  Links
    public Enumeration  GetLinks()          { return pLinks.elements(); }

    public void     AddLink( CLink link )
    {
        link.SetId( ++mLastLinkId );
        pLinks.addElement( link );

        pModified = true;
        // TODO: history... (not here, but who called this method)
    }

    public CLink     GetLink ( int id )
        throws Exception
    {
        Enumeration e = pLinks.elements();
        while( e.hasMoreElements() )
        {
            CLink link = (CLink) e.nextElement();
            if( link.GetId() == id )
                return link;
        }
        throw new Exception();
    }

    public void  RemoveLink( CLink link )
    {
        // remove link or mark as deleted ??
        pLinks.removeElement( link );

        // remove links ???

        pModified = true;
        // TODO: history... (not here, but who called this method)
    }


/*

    //**  Load & Save
    public void Load ()
    {
        String fileName = mPath + "//" + mName;
        FileInputStream fInStr;
        ObjectInputStream in;

        try {
            fInStr = new FileInputStream( fileName );
            in = new ObjectInputStream( fInStr );
            readObject( in );
            fInStr.close();
            pModified = false;
        } catch( java.io.IOException e1 ) {
            System.out.println( "DEBUG: Error 1  [CFile].Load" );
            e1.printStackTrace();
        } catch( java.lang.ClassNotFoundException e2 ) {
            System.out.println( "DEBUG: Error 2  [CFile].Load" );
            e2.printStackTrace();
        }
    }

    public void  Save()
    {
        String fileName = mPath + "//" + mName;
        FileOutputStream fOutStr;
        ObjectOutputStream out;

        try {
            fOutStr = new FileOutputStream( fileName );
            out = new ObjectOutputStream( fOutStr );
            writeObject( out );
            fOutStr.close();
            pModified = false;
        }
        catch ( java.io.IOException e1 ) {
            System.out.println( "DEBUG: Error  [CFile].Save" );
        }
    }


/*
    //**  Serializable
    protected void writeObject(java.io.ObjectOutputStream out)
        throws IOException
    {
        out.writeObject( mName );
        out.writeObject( mPath );
        out.writeObject( mDescription );

        out.writeInt( mLastElemId );
        out.writeObject( pElements );

        out.writeInt( mLastLinkId );
        out.writeObject( pLinks );

        out.writeObject( pHistory );
    }

    protected void readObject(java.io.ObjectInputStream in)
        throws IOException, ClassNotFoundException
    {
        mName =         (String) in.readObject( );
        mPath =         (String) in.readObject( );
        mDescription =  (String) in.readObject( );

        mLastElemId =   in.readInt( );
        pElements =     (Vector) in.readObject( );

        mLastLinkId =   in.readInt( );
        pLinks =        (Vector) in.readObject( );

        pHistory =      (CHistory) in.readObject();
        pHistory.SetFile( this );

        // Setup Internal Structure After Reading
        //SetupAfterRead();
    }
/*
    private void SetupAfterRead()
    {
        Enumeration e = pElements.elements();
        while( e.hasMoreElements() )
        {
            CElement elem = (CElement) e.nextElement();

            if( elem instanceof CHistoryElement )
                pHistory._AddHistoryElement( (CHistoryElement) elem );
            else
                SetupElementAfterRead( elem );
        }
    }

    // To Be Overriden
    protected void SetupElementAfterRead( CElement elem ) {}
*/

    //** DEBUG
    public void DEBUG_write( PrintStream pStr )
    {
        pStr.println( "DEBUG: CFile");
        pStr.println( "       Name:         " + mName );
        pStr.println( "       Path:         " + mPath );
        pStr.println( "       Description:  " + mDescription );
        pStr.println( "       LastElemId:   " + mLastElemId );
        pStr.println( "       LastLinkId:   " + mLastLinkId );

        pHistory.DEBUG_write(pStr);

        Enumeration e = pElements.elements();
        while( e.hasMoreElements() )
        {
            CElement elem = (CElement) e.nextElement();
            elem.DEBUG_write( pStr );
        }

        e = pLinks.elements();
        while( e.hasMoreElements() )
        {
            CLink link = (CLink) e.nextElement();
            link.DEBUG_write( pStr );
        }


    }


    //**  Load & Save
    public static CFile Load (String name, String path)
        throws IOException, ClassNotFoundException
    {
        String fileName = path + "/" + name;

        FileInputStream fInStr = new FileInputStream( fileName );
        ObjectInputStream in = new ObjectInputStream( fInStr );

        CFile file = (CFile) in.readObject();
        file.SetName( name );
        file.SetPath( path );

        fInStr.close();
        return file;
    }

    public static void  Save( CFile file )
        throws IOException
    {
        String fileName = file.GetPath() + "/" /*File.separator*/ + file.GetName();

        FileOutputStream    fOutStr = new FileOutputStream( fileName );
        ObjectOutputStream  out     = new ObjectOutputStream( fOutStr );

        file.GetHistory().SetLastUpdate( new Date() );
        file.pModified = false;
        out.writeObject( file );

        fOutStr.close();
    }

    public static void  SaveAs( CFile file, String name, String path )
        throws IOException
    {
        String fileName = path + File.separator + name;

        FileOutputStream    fOutStr = new FileOutputStream( fileName );
        ObjectOutputStream  out     = new ObjectOutputStream( fOutStr );

        file.SetName( name );
        file.SetPath( path );
        file.GetHistory().SetLastUpdate( new Date() );
        file.pModified = false;
        out.writeObject( file );

        fOutStr.close();
    }



    public void Save()
        throws IOException
    {
        CFile.Save(this);
    }

}