/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: GRAPH
 * Classe:  CGrpLabel
\******************************************************************************/

package GRAPH;

import java.awt.*;


public class    CGrpLabel
    extends     CGrpObject
{
    protected   Point       pRefPoint;
    protected   Font        pFont;
    protected   int         mAlign;
    protected   String      mText;

    //protected   Rectangle   pRegion = null;

    public static final int LEFT   = 0;
    public static final int CENTER = 1;
    public static final int RIGHT  = 2;


    public CGrpLabel( Point point, String text, int align, Font font )
    {
        super();
        pRefPoint   = point;
        mText       = text;
        pFont       = font;
        mAlign      = align;
    }

    public CGrpLabel( Point point, String text )
    {
        this( point, text, LEFT, null );
    }

    public CGrpLabel( Point point, String text, int align )
    {
        this( point, text, align, null );
    }

    public CGrpLabel( Point point, String text, Font font )
    {
        this( point, text, LEFT, font );
    }


    public void     SetRefPoint( Point p )      { pRefPoint = p; }

    public void     SetText( String text )      { mText = text; }


    public Rectangle GetRegion( CGraphics g )
    {
        /*if( pRegion != null )
            return( pRegion ); */

        Font oldFont = null;

        if( pFont != null )
        {
            oldFont = g.getFont();
            g.setFont( pFont );
        }

        FontMetrics fm = g.getFontMetrics();
        int w = fm.stringWidth( mText );
        int h = fm.getHeight();

        int x;

        if( mAlign == RIGHT )
            x = pRefPoint.x - w;
        else
        if( mAlign == CENTER )
            x = pRefPoint.x - w/2;
        else
            x = pRefPoint.x;

        if( oldFont != null )
            g.setFont( oldFont );

        return( pRegion = new Rectangle( x, pRefPoint.y, w, h ) );
    }




    public void Draw( CGraphics g )
    {
        Font oldFont = null;
        if( pFont != null )
        {
            oldFont = g.getFont();
            g.setFont( pFont );
        }

        Color oldColor = null;
        if( pColor != null )
        {
            oldColor = g.getColor();
            g.setColor( pColor );
        }


        FontMetrics fm = g.getFontMetrics();
        int w = fm.stringWidth( mText );
        //int h = fm.getHeight();

        int x, y = pRefPoint.y + fm.getAscent();

        if( mAlign == RIGHT )
            x = pRefPoint.x - w;
        else
        if( mAlign == CENTER )
            x = pRefPoint.x - w/2;
        else
            x = pRefPoint.x;

        g.drawString( mText, x, y );

        if( oldColor != null )
            g.setColor( oldColor );

        if( oldFont != null )
            g.setFont( oldFont );
    }

}