/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: GRAPH
 * Classe:  CGrpTriangle
\******************************************************************************/

package GRAPH;

import java.awt.*;
//import java.io.*;


public class    CGrpTriangle
    extends     CGrpObject
{
    protected   CPolyline   pPoly;
    protected   int         mDirection;

    public static final int UP    = 0;
    public static final int DOWN  = 1;
    public static final int LEFT  = 2;
    public static final int RIGHT = 3;


    public CGrpTriangle ( Rectangle rect, int direction )
    {
        super();
        mDirection = direction;
        SetRegion( rect );
    }

    public void SetDirection( int direction )
    {
        if( mDirection == direction ) return;

        mDirection = direction;
        _AdjustPolyline();
    }

    public void SetRegion( Rectangle rect )
    {
        super.SetRegion( rect );
        _AdjustPolyline();
    }

    private void _AdjustPolyline()
    {
        pPoly = new CPolyline();

        switch( mDirection )
        {
            case DOWN:
                {
                    pPoly.AddPoint( new Point( pRegion.x, pRegion.y ) );
                    pPoly.AddPoint( new Point( pRegion.x + pRegion.width, pRegion.y ) );
                    pPoly.AddPoint( new Point( pRegion.x + pRegion.width/2, pRegion.y + pRegion.height) );
                    //pPoly.AddPoint( new Point( pRegion.x, pRegion.y ) );
                }
                break;

            case UP:
                {
                    pPoly.AddPoint( new Point( pRegion.x, pRegion.y + pRegion.height ) );
                    pPoly.AddPoint( new Point( pRegion.x + pRegion.width, pRegion.y + pRegion.height ) );
                    pPoly.AddPoint( new Point( pRegion.x + pRegion.width/2, pRegion.y) );
                    //pPoly.AddPoint( new Point( pRegion.x, pRegion.y + pRegion.height ) );
                }
                break;

            case RIGHT:
                {
                    pPoly.AddPoint( new Point( pRegion.x, pRegion.y ) );
                    pPoly.AddPoint( new Point( pRegion.x, pRegion.y  + pRegion.height) );
                    pPoly.AddPoint( new Point( pRegion.x + pRegion.width, pRegion.y + pRegion.height/2) );
                    //pPoly.AddPoint( new Point( pRegion.x, pRegion.y ) );
                }
                break;

            default: //( mDirection == LEFT )
                {
                    pPoly.AddPoint( new Point( pRegion.x + pRegion.width, pRegion.y ) );
                    pPoly.AddPoint( new Point( pRegion.x + pRegion.width, pRegion.y + pRegion.height ) );
                    pPoly.AddPoint( new Point( pRegion.x, pRegion.y + pRegion.height/2) );
                    //pPoly.AddPoint( new Point( pRegion.x + pRegion.width, pRegion.y ) );
                }
        }// switch
    }


    public void           Draw        (CGraphics g)
    {
        Color oldColor;

        if( IsFilled() )
        {
            oldColor = null;
            if( pFillColor != null )
            {
                oldColor = g.getColor();
                g.setColor( pFillColor );
            }
            g.fillPolygon( pPoly );

            if( oldColor != null )
                g.setColor( oldColor );
        }



        oldColor = null;
        if( pColor != null )
        {
            oldColor = g.getColor();
            g.setColor( pColor );
        }

        if( IsDashed() ) g.SetDashedPen();
        g.drawPolygon( pPoly );
        if( IsDashed() ) g.SetSolidPen();

        if( oldColor != null )
            g.setColor( oldColor );
    }

    public boolean Contains( int x, int y )
    {   return( pPoly.GetPolygon().contains(x,y) ); }

}
