/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: GRAPH
 * Classe:  CRectSelector
\******************************************************************************/

package GRAPH;

import  java.awt.*;
import  java.util.*;


public class    CRectSelector
    //implements  MouseMotionListener
{
    protected   Vector          pSelPoints;

    protected   Rectangle       pRect;

    protected   boolean         mMouseDown;
    protected   CSelectorPoint  pSelected;
    protected   Point           mInitialPoint;
    protected   Point           mLastPoint;

    public CRectSelector( Rectangle rect )
    {
        pRect = rect;

        _CreateRectSelectors();

        mMouseDown = false;
        pSelected = null;
        mInitialPoint = mLastPoint = null;
    }

    private void _CreateRectSelectors()
    {
        pSelPoints = new Vector();

        int x1, y1, x2, y2, x3, y3;
        x1 = pRect.x;       x2 = pRect.x + pRect.width;     x3 = pRect.x + (pRect.width/2);
        y1 = pRect.y;       y2 = pRect.y + pRect.height;    y3 = pRect.y + (pRect.height/2);

        pSelPoints.addElement( new CSelectorPoint( new Point(x3, y1), CSelectorPoint.N_RESIZE ) ); // 0
        pSelPoints.addElement( new CSelectorPoint( new Point(x3, y2), CSelectorPoint.S_RESIZE ) ); // 1
        pSelPoints.addElement( new CSelectorPoint( new Point(x2, y3), CSelectorPoint.E_RESIZE ) );
        pSelPoints.addElement( new CSelectorPoint( new Point(x1, y3), CSelectorPoint.W_RESIZE ) );

        pSelPoints.addElement( new CSelectorPoint( new Point(x2, y1), CSelectorPoint.NE_RESIZE ) ); // 4
        pSelPoints.addElement( new CSelectorPoint( new Point(x2, y2), CSelectorPoint.SE_RESIZE ) );
        pSelPoints.addElement( new CSelectorPoint( new Point(x1, y1), CSelectorPoint.NW_RESIZE ) );
        pSelPoints.addElement( new CSelectorPoint( new Point(x1, y2), CSelectorPoint.SW_RESIZE ) );
    }


    public void setBounds( Rectangle r )
    {
        pRect = r;
        _MovePoints();
    }

    public Rectangle getBounds()
    {
        Rectangle r = new Rectangle( pRect );
        //r.grow( 3, 3 );
        return r;
    }

    public boolean Contains( int x, int y )
    {
        Rectangle r = getBounds();
        r.grow(4,4);
        return r.contains(x,y);
    }


    public void Draw( CGraphics g )
    {
        g.setXORMode(Color.white);

        //g.SetDashedPen();
        g.RealDrawRect( pRect.x-1, pRect.y-1, pRect.width+2, pRect.height+2 );
        //g.SetSolidPen();

        Enumeration e = pSelPoints.elements();
        while( e.hasMoreElements() )
            ( (CSelectorPoint) e.nextElement() ).Draw(g);

        g.setPaintMode();
    }

    public Cursor GetCursor( int x, int y )
    {
        if( mMouseDown )
            if( pSelected != null )
                return pSelected.GetCursor();
            else
                return Cursor.getPredefinedCursor( Cursor.MOVE_CURSOR );

        if( Contains(x,y) )
        {
            Enumeration e = pSelPoints.elements();
            while( e.hasMoreElements() )
            {
                CSelectorPoint point = (CSelectorPoint) e.nextElement();
                if( point.Contains( x, y ) )
                    return point.GetCursor();
            }

            if( pRect.contains(x,y) )
                return Cursor.getPredefinedCursor( Cursor.MOVE_CURSOR );
        }

        return Cursor.getDefaultCursor();
    }

    public boolean IsMouseDown()        {    return mMouseDown;  }

    public boolean HasMoved()
    {
        if(mLastPoint == mInitialPoint)
            return false;

        if(mLastPoint != null)
            if( (mLastPoint.x==mInitialPoint.x) && (mLastPoint.y==mInitialPoint.y) )
                return false;

        return true;
    }

    public boolean MouseDown( CGraphics g, int x, int y )
    {
        if( !Contains(x,y) )    return false;

        // Verify if it is a selectorPoint
        pSelected = null;
        Enumeration e = pSelPoints.elements();
        while( (e.hasMoreElements()) && (pSelected==null) )
        {
            CSelectorPoint point = (CSelectorPoint) e.nextElement();
            if( point.Contains( x, y ) )
            {
                pSelected = point;
                //break;
            }
        }

        if( (pSelected != null) || (pRect.contains(x,y)) )
        {
            mMouseDown = true;
            mInitialPoint = mLastPoint = new Point(x,y);
            //g.setXORMode();
            return true;
        }

        return false;
    }


    private void _TranslatePoints( int dx, int dy )
    {
        Enumeration e = pSelPoints.elements();
        while( e.hasMoreElements() )
        {
            CSelectorPoint p = (CSelectorPoint) e.nextElement();
            Point newPoint = new Point( p.GetPoint() );
            newPoint.translate( dx, dy );
            p.SetPoint( newPoint );
        }
    }

    private void _MovePoints()
    {
        int x1, y1, x2, y2, x3, y3;
        x1 = pRect.x;       x2 = pRect.x + pRect.width;     x3 = pRect.x + (pRect.width/2);
        y1 = pRect.y;       y2 = pRect.y + pRect.height;    y3 = pRect.y + (pRect.height/2);

        Enumeration e = pSelPoints.elements();
        while( e.hasMoreElements() )
        {
            CSelectorPoint p = (CSelectorPoint) e.nextElement();
            int x, y;

            // X
            switch( p.GetType() )
            {
                case CSelectorPoint.W_RESIZE:
                case CSelectorPoint.NW_RESIZE:
                case CSelectorPoint.SW_RESIZE:
                    x = x1;
                    break;

                case CSelectorPoint.E_RESIZE:
                case CSelectorPoint.NE_RESIZE:
                case CSelectorPoint.SE_RESIZE:
                    x = x2;
                    break;

                case CSelectorPoint.N_RESIZE:
                case CSelectorPoint.S_RESIZE:
                default:
                    x = x3;
            }

            // Y
            switch( p.GetType() )
            {
                case CSelectorPoint.N_RESIZE:
                case CSelectorPoint.NE_RESIZE:
                case CSelectorPoint.NW_RESIZE:
                    y = y1;
                    break;

                case CSelectorPoint.S_RESIZE:
                case CSelectorPoint.SE_RESIZE:
                case CSelectorPoint.SW_RESIZE:
                    y = y2;
                    break;

                case CSelectorPoint.W_RESIZE:
                case CSelectorPoint.E_RESIZE:
                default:
                    y = y3;
            }

            p.SetPoint( new Point(x,y) );

        }// while hasMoreElements...

    }


    public void MouseMove( CGraphics g, int x, int y )
    {
        if( !mMouseDown )   return;

        int dx, dy;
        dx = x - mLastPoint.x;
        dy = y - mLastPoint.y;

        mLastPoint = new Point(x,y);

        // Hides itself
        Draw( g );

        if( pSelected != null )
        {   // Resize the rectangle

            // Horizontal
            switch( pSelected.GetType() )
            {
                case CSelectorPoint.W_RESIZE:
                case CSelectorPoint.NW_RESIZE:
                case CSelectorPoint.SW_RESIZE:
                    pRect.x       += dx;
                    pRect.width   -= dx;
                    break;

                case CSelectorPoint.E_RESIZE:
                case CSelectorPoint.NE_RESIZE:
                case CSelectorPoint.SE_RESIZE:
                    pRect.width   += dx;
            }

            // Vertical
            switch( pSelected.GetType() )
            {
                case CSelectorPoint.NE_RESIZE:
                case CSelectorPoint.NW_RESIZE:
                case CSelectorPoint.N_RESIZE:
                    pRect.y       += dy;
                    pRect.height  -= dy;
                    break;

                case CSelectorPoint.SE_RESIZE:
                case CSelectorPoint.SW_RESIZE:
                case CSelectorPoint.S_RESIZE:
                    pRect.height   += dy;
            }

            // Move Points
            _MovePoints();
        }

        else
        {   // the whole object was moved
            // Translate rectangle and points
            pRect.translate( dx, dy );
            _TranslatePoints( dx, dy );
        }

        // Shows itself
        Draw( g );
    }


    public void MouseUp ( CGraphics g, int x, int y )
    {
        if( !mMouseDown ) return;

        // To be sure the selector is well drawn
        MouseMove( g, x, y );

        // Save last Point to future use ???
        mLastPoint = new Point( x, y );

        // "clear" temporary variables
        mMouseDown = false;
        pSelected = null;

        // restore Paint Mode to CGraphics
        g.setPaintMode();
    }


}