/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: GRAPH
 * Classe:  CSelectorPointInRectPath
\******************************************************************************/

package GRAPH;

import java.awt.*;


public class    CSelectorPointInRectPath
    extends     CSelectorPoint
{
    protected   Rectangle       pRectPath   = null,
                                pIn         = null,
                                pOut        = null;
    protected   CSidePosition   pSidePosition;

    protected   boolean         mPointAtCorner;

    public CSelectorPointInRectPath( Rectangle rect, CSidePosition sidePos )
    {
        super( new Point(0,0) );

        pSidePosition = new CSidePosition( sidePos );

        SetRectPath( rect );
        SetPoint( pSidePosition.GetPoint( pRectPath ) );

    }


    public CSidePosition    GetSidePosition()    {   return pSidePosition; }

    public void SetRectPath( Rectangle rect )
    {
        pRectPath = new Rectangle( rect );
        pIn = new Rectangle( rect );
        pIn.grow( -3, -3 );

        pOut = new Rectangle( rect );
        pOut.grow( 3, 3 );
    }


    public void SetPoint( Point p )
    {
        if( pRectPath == null ) return;

        if( pIn.contains(p) || !pOut.contains(p) )
            return;

        boolean top     = 3 >= Math.abs( pRectPath.y - p.y );
        boolean bottom  = 3 >= Math.abs( pRectPath.y + pRectPath.height - p.y );
        boolean left    = 3 >= Math.abs( pRectPath.x - p.x );
        boolean right   = 3 >= Math.abs( pRectPath.x + pRectPath.width - p.x );


        mPointAtCorner = false;

        if( top )
        {
            if( left )
            {
                pSidePosition.Set( CSidePosition.TOP, 0 );
                mPointAtCorner = true;
            }
            else
            if( right )
            {
                pSidePosition.Set( CSidePosition.RIGHT, 0 );
                mPointAtCorner = true;
            }

            else
                pSidePosition.Set( CSidePosition.TOP, p.x - pRectPath.x );
        }

        else
        if( bottom )
        {
            if( left )
            {
                pSidePosition.Set( CSidePosition.BOTTOM, 0 );
                mPointAtCorner = true;
            }

            else
            if( right )
            {
                pSidePosition.Set( CSidePosition.RIGHT, pRectPath.height );
                mPointAtCorner = true;
            }

            else
                pSidePosition.Set( CSidePosition.BOTTOM, p.x - pRectPath.x );
        }

        else
        if( left )
            pSidePosition.Set( CSidePosition.LEFT, p.y - pRectPath.y );

        else // right
            pSidePosition.Set( CSidePosition.RIGHT, p.y - pRectPath.y );

        UpdateCursor();

        super.SetPoint( pSidePosition.GetPoint( pRectPath ) );
    }


    public void SetType( int type ) {}


    public void UpdateCursor()
    {
        if( mPointAtCorner )
            super.SetType( MOVE );

        else
        {
            switch( pSidePosition.GetSide() )
            {
                case CSidePosition.TOP:
                case CSidePosition.BOTTOM:
                    super.SetType( E_RESIZE );
                    break;

                //case
                default:
                    super.SetType( N_RESIZE );
            }
        }
    }

}