/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: TOOL.PROCESS.CLIENT
 * Classe:  ProcessControlPanel
\******************************************************************************/

package TOOL.PROCESS.CLIENT;


import  java.awt.*;
import  java.awt.event.*;


import  COPSE.*;
import  COPSE.CLIENT.*;
import  COPSE.CLIENT.GUI.*;
import  COPSE.CONTROL.*;
import  FILE.CElement;
import  PROCESS2.*;
import  TOOL.PROCESS.*;


public class    COPEActivityControl
    extends     CGUIControl
    implements  ActionListener          //, ItemListener  //, WindowListener ...
{
    //* Tool Manager
    protected   ProcessGUIToolManager   pToolManager;

    //* Controller of the Selected Element
    protected   CElementController  pSelectedElement;

    //* Menus
    protected   Menu                pMainMenu;
    protected   MenuItem            mnNew;
    protected   MenuItem            mnEdit;
    protected   MenuItem            mnDelete;
//    protected   MenuItem            mnRename;

/*
    protected   Menu                pRoleMenu;
    protected   MenuItem            mnNewRole;
    protected   MenuItem            mnRole;
    protected   MenuItem            mnVisualMinimum;
*/

    //* Dialogs
    protected   COPEDlgActivity     pDlgEdit;
    protected   boolean             mLockRequested;
    protected   boolean             mIsEditing;



    public COPEActivityControl  ( ProcessGUIToolManager tool )
    {
        super( tool.GetGUIManager() );
        pToolManager = tool;
        pSelectedElement = null;

        pDlgEdit = null;
        mLockRequested = false;
        mIsEditing = false;
    }


    public void SetEnabled( boolean enabled )
    {
        mIsEnabled = enabled;

        if( !enabled )
        {
            SetSelectedElement( null );
            /* */
            pSelectedElement = null;
            if( pDlgEdit != null )
                CloseDlgEdit( false );
            mLockRequested = false;
            mIsEditing = false;
            /* */
        }

        ChangeMenuSettings();
    }



    public void     AddMenu( MenuBar menuBar )
    {
        pMainMenu  = new Menu( "Activity" );

        mnNew       = new MenuItem( "New..." );
        mnEdit      = new MenuItem( "Edit..." );
        mnDelete    = new MenuItem( "Delete" );

/*
        pVisualMenu = new Menu( "Visual" );
        mnVisualTotal       = new CheckboxMenuItem( "Total" );
        mnVisualPartial     = new CheckboxMenuItem( "Partial" );
        mnVisualMinimum     = new CheckboxMenuItem( "Minimum" );

        pVisualMenu.add( mnVisualTotal );
        pVisualMenu.add( mnVisualPartial );
        pVisualMenu.add( mnVisualMinimum );
*/

        pMainMenu.add( mnNew );
        pMainMenu.add( mnEdit );
        pMainMenu.add( mnDelete );
        pMainMenu.addSeparator();
        //pMainMenu.add( pVisualMenu );


        mnNew.addActionListener( this );
        mnEdit.addActionListener( this );
        mnDelete.addActionListener( this );

/*
        mnVisualTotal.addItemListener( this );
        mnVisualPartial.addItemListener( this );
        mnVisualMinimum.addItemListener( this );
*/

        ChangeMenuSettings();

        menuBar.add( pMainMenu );
    }



    public synchronized void SetSelectedElement( CElement element )
    {
        // Close the edit dialog if it's open...
        if( mLockRequested )
        {
            try {
                CEvent event = new CEvent( CEvent.EvCOPSE_FileController_Element_Unlock );
                event.AddArgument( new Integer( pToolManager.GetClient().GetSessionManager().GetClientSession().GetConnectionId() ) );
                event.AddArgument( new Integer( pSelectedElement.GetElement().GetId() ) );
                pGUI.Send( event );
            } catch( Exception e ) {}

            mLockRequested = false;
        }
        else
        if( mIsEditing ) //pDlgEdit != null )
        {
            CloseDlgEdit( false );
        }

        if( element != null )
            pSelectedElement = pToolManager.GetClient().GetFileController().FindElementController( element.GetId() );
        else
            pSelectedElement = null;
        ChangeMenuSettings();
    }

    public CElement     GetSelectedElement()
    {
        return pSelectedElement.GetElement();
    }

/*
    public CObjClass GetSelectedClass()
    {
        if( pSelectedElement == null )
            return null;

        return (CObjClass) pSelectedElement.GetElementController().GetElement();
    }
*/

    public boolean  IsWaitingForLock()
    {   return mLockRequested; }

    public synchronized void     ReceiveLockConfirmation()
    {
        if( mLockRequested )
        {
            pDlgEdit = new COPEDlgActivity( pGUI, false, this );
            //pDlgEdit = new Dialog( pGUI.GetMainWindow(), "COPE - Test Dialog" );
            pDlgEdit.setVisible( true );
            mIsEditing = true;
            mLockRequested = false;
        }
    }


    public void     CloseDlgEdit(boolean ok)
    {
        if( ok )
        {
            // Send event to update class...
            /* */
            if( pDlgEdit.HasChanged() )
            {
                try {
                    CEvent ev = new CEvent( CEvent.EvCOPSE_FileController_Element_Update );
                    ev.AddArgument( new Integer( pToolManager.GetClient().GetSessionManager().GetClientSession().GetConnectionId()  ) );
                    ev.AddArgument( pDlgEdit.GetNewActivity() );
                    pGUI.Send( ev );
                } catch( Exception e ) {}
            }
            /* */
        }


        // Unlock class
        try {
            CEvent ev = new CEvent( CEvent.EvCOPSE_FileController_Element_Unlock );
            ev.AddArgument( new Integer( pToolManager.GetClient().GetSessionManager().GetClientSession().GetConnectionId()  ) );
            ev.AddArgument( new Integer( pSelectedElement.GetElement().GetId() ) );
            pGUI.Send( ev );

            mIsEditing = false;
            pDlgEdit.setVisible( false );
            pDlgEdit.dispose();
            pDlgEdit = null;
        } catch( Exception e) {}
    }


    public void ChangeMenuSettings()
    {
        pMainMenu.setEnabled( IsEnabled() );
        if( IsEnabled() )
        {
            if( pSelectedElement != null )
            {
                //CVisualElementController elemController = pSelectedElement.GetElementController();

                // Edit and Delete...
                boolean isLocked = pSelectedElement.IsLocked();
                mnEdit.setEnabled( !isLocked );
                mnDelete.setEnabled( !isLocked );

                /*
                pVisualMenu.setEnabled( true );

                int visual = elemController.GetVisual();
                mnVisualTotal.setState( visual == 2 );
                mnVisualPartial.setState( visual == 1 );
                mnVisualMinimum.setState( visual == 0 );
                */

                return;
            }

            mnEdit.setEnabled( false );
            mnDelete.setEnabled( false );
            //pVisualMenu.setEnabled( false );
        }
    }


    public void actionPerformed( ActionEvent event )
    {
        Object source = event.getSource();

        if( source == mnNew )
        {
            //pToolManager.GetCanvas().CreateNewClass();
            if( pSelectedElement != null )
            {
                CElement elem = pSelectedElement.GetElement();
                if( elem instanceof CAbstractActivity )
                {
                    COPEDlgActivityNew dlg = new COPEDlgActivityNew( pToolManager, (CAbstractActivity)elem );
                    dlg.setVisible( true );
                }
            }
        }
        else

        if( source == mnEdit )
        {
            //*
            try {
                CEvent ev = new CEvent( CEvent.EvCOPSE_FileController_Element_Lock );
                ev.AddArgument( new Integer( pToolManager.GetClient().GetSessionManager().GetClientSession().GetConnectionId()  ) );
                ev.AddArgument( new Integer( pSelectedElement.GetElement().GetId() ) );
                pGUI.Send( ev );
                mLockRequested = true;
            } catch(Exception e) {}

            /* * /
            if( pDlgEdit == null )
                pDlgEdit = new COPEDlgActivity( pGUI, false, this );

            pDlgEdit.setVisible(true);
            /* */
        }

        else
        if( source == mnDelete )
        {
            try {
                CEvent ev = new CEvent( CEvent.EvCOPSE_FileController_Element_Delete );
                ev.AddArgument( new Integer( pToolManager.GetClient().GetSessionManager().GetClientSession().GetConnectionId()  ) );
                ev.AddArgument( new Integer( pSelectedElement.GetElement().GetId() ) );
                pGUI.Send( ev );
            } catch(Exception e) {}
        }

    }

}