/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: TOOL.PROCESS
 * Classe:  CProcessFileController
\******************************************************************************/

package TOOL.PROCESS;

import  java.util.*;

import  FILE.*;
import  PROCESS2.*;

import  GRAPH.*;

import  COPSE.CONTROL.*;

/*************************************************************************
 *  Class CProcessFileController
 *************************************************************************/
public class CProcessFileController
    extends  CVisualFileController
{
    public CProcessFileController( CFile file )
    {   super( file );   }

    public synchronized boolean  CanRemoveElement ( int connId, CElementController control )
    {
        CElement elem = control.GetElement();

        if( elem instanceof CAbstractActivity )
            return _CanRemoveActivity( connId, control, (CAbstractActivity)elem );

        // Else ...
        return super.CanRemoveElement( connId, control );
    }


    private boolean _CanRemoveActivity( int connId, CElementController control, CAbstractActivity activity )
    {
        boolean ok = true;

        if( !(activity instanceof CActivity) )
            return false;

        CActivity act = (CActivity)activity;

        //temporary: Always return FALSE
        if( act.GetState() >= CActivityState.AS_INITIATED )
        //if( act.GetState() != CActivityState.AS_INITIATED )
            return false;


        // Verify the own object ... (activity)
        ok = super.CanRemoveElement( connId, control );

        if( ok )
        {
            // Verify Flows / Roles / ...
            Enumeration e = act.GetFlows();
            while( ok  &&  e.hasMoreElements() )
            {
                CFlow flow = (CFlow) e.nextElement();
                CElementController flowControl = FindElementController( flow.GetId() );
                ok = super.CanRemoveElement( connId, flowControl );
            }

            /*
            e = act.GetRoles();
            while( ok  &&  e.hasMoreElements() )
            {
                CRole role = (CRole) e.nextElement();
                CElementController roleControl = FindElementController( role.GetId() );
                ok = super.CanRemoveElement( connId, roleControl );
            }
            */

            // Subactivities...
        }

        return ok;
    }








    public synchronized boolean  DeleteElement( int connId, int elemId )
    {
        // Set To Invisible


        CElementController control = FindElementController( elemId );
        if( control == null )   return false;

        CElement elem = control.GetElement();

        /*
        if( elem instanceof CAbstractActivity )
            return _DeleteClass( connId, control, (CObjClass)elem );

        else
        if( elem instanceof CObjRelation )
            return _DeleteRelation( connId, control, (CObjRelation)elem );
        */

        return false;
        // Else ...
        //return super.DeleteElement( connId, elemId );
    }

/*
    private synchronized boolean  _DeleteRelation( int connId, CElementController control, CObjRelation relat )
    {
        if( !CanRemoveElement( connId, control ) )
            return false;

        // Remove references in classes
        Enumeration e = relat.GetExtremities();
        while( e.hasMoreElements() )
        {
            CObjRelationExtremity extr = (CObjRelationExtremity) e.nextElement();
            CObjClass aclass = extr.GetClass();
            aclass.RemoveRelation( relat );
        }

        // Remove Relation
        pFile.RemoveElement( relat );
        pElementCtrls.removeElement( control );
        return true;
    }



    private void _RemoveElements_( int connId, Enumeration e )
    {
        while( e.hasMoreElements() )
        {
            CElement element = (CElement) e.nextElement();

            //DeleteElement( connId, element.GetId() );
            //*
            CElementController control = FindElementController( element.GetId() );

            pFile.RemoveElement( element );
            pElementCtrls.removeElement( control );
            /* * /
        }
    }



    private synchronized boolean  _DeleteClass( int connId, CElementController control, CObjClass aclass )
    {
        if( !_CanRemoveClass( connId, control, aclass ) )
            return false;

        // Delete Relationships...
        //_RemoveElements_( connId, aclass.GetRelations() );

        ///*
        Vector tempRel = new Vector();
        Enumeration e = aclass.GetRelations();
        while( e.hasMoreElements() )
            tempRel.addElement( e.nextElement() );

        e = tempRel.elements();
        while( e.hasMoreElements() )
        {
            CObjRelation relation = (CObjRelation) e.nextElement();
            CElementController relControl = FindElementController( relation.GetId() );

            _DeleteRelation( connId, relControl, relation );
            // pFile.RemoveElement( relation );
            // pElementCtrls.removeElement( relControl );
        }
        //  * /

        // Delete Attributes and Methods
        _RemoveElements_( connId, aclass.GetAttributes() );
        _RemoveElements_( connId, aclass.GetMethods() );

        // Delete Class
        pFile.RemoveElement( aclass );
        pElementCtrls.removeElement( control );

        return true;
    }
*/





    public synchronized boolean  UpdateElement( int connId, CElement elemUpdate )
    {
        System.err.println( "[DEBUG] CProcessFileController - UpdateElement..." );
        CElementController control = FindElementController( elemUpdate.GetId() );
        if( control == null ) return false;

        if( control.GetLockerConnectionId() != connId )
            return false;

        /*
        if( elemUpdate instanceof CActivity )
            return _UpdateActivity( connId, control, (CActivity) elemUpdate );
        /*
        if( elemUpdate instanceof CObjRelation )
            return _UpdateRelation( connId, control, (CObjRelation) elemUpdate );
        */
        control.GetElement().UpdateFrom( elemUpdate );
        return true;
    }



/*
    private synchronized boolean _UpdateRelation( int connId, CElementController control, CObjRelation newRelation )
    {
        System.err.println( "[DEBUG] Before update..." );

        System.err.println( "  Actual relation:");
        control.GetElement().DEBUG_write( System.err );

        System.err.println( "  Requested relation:");
        newRelation.DEBUG_write( System.err );

        System.err.println( "  Updating... " );
        control.GetElement().UpdateFrom( newRelation );

        System.err.println( "  After update, current relation..." );
        control.GetElement().DEBUG_write( System.err );
        System.err.println( "  Requested relation:");
        newRelation.DEBUG_write( System.err );

        return true;
    }
*/


/*
    private synchronized boolean _UpdateClass( int connId, CElementController control, CObjClass nClass )
    {
        CObjClass oldClass = (CObjClass) control.GetElement();
        CObjClass newClass = nClass.CloneWithoutRelations();

        Enumeration e;
        Vector vector;

        // Remove invalid attributes and change editted
        vector = new Vector();
        e = oldClass.GetAttributes();
        while( e.hasMoreElements() )
            vector.addElement( e.nextElement() );

        e = vector.elements();
        while( e.hasMoreElements() )
        {
            CObjAttribute oldAttr = (CObjAttribute) e.nextElement();
            CObjAttribute newAttr = newClass.FindAttribute( oldAttr.GetId() );

            if( newAttr == null )
            {
                // Remove Attribute oldAttr...
                //DeleteElement( connId, oldAttr.GetId() );
                super.DeleteElement( connId, oldAttr.GetId() );
                oldClass.RemoveAttribute( oldAttr );
            }
            else
            {
                // Change Editted Attributes
                oldAttr.CopyFrom( newAttr );
            }
        }


        // Add new attributes...
        e = newClass.GetAttributes();
        while( e.hasMoreElements() )
        {
            CObjAttribute attr = (CObjAttribute) e.nextElement();
            if( attr.GetId() == 0 )
            {
                // Add Attribute attr...
                NewElement( connId, attr );
                oldClass.AddAttribute( attr );
            }
        }



        // Remove invalid methods
        vector = new Vector();
        e = oldClass.GetMethods();
        while( e.hasMoreElements() )
            vector.addElement( e.nextElement() );

        e = vector.elements();
        while( e.hasMoreElements() )
        {
            CObjMethod oldMethod = (CObjMethod) e.nextElement();
            CObjMethod newMethod = newClass.FindMethod( oldMethod.GetId() );

            if( newMethod == null )
            {
                // Remove method ...
                DeleteElement( connId, oldMethod.GetId() );
                oldClass.RemoveMethod( oldMethod );
            }
            else
            {
                // Change Editted Methods
                oldMethod.CopyFrom( newMethod );
            }
        }


        // Add new method...
        e = newClass.GetMethods();
        while( e.hasMoreElements() )
        {
            CObjMethod method = (CObjMethod) e.nextElement();
            if( method.GetId() == 0 )
            {
                // Add method ...
                NewElement( connId, method );
                oldClass.AddMethod( method );
            }
        }

        // Copy attributes of CObjClass
        oldClass.CopyInfoFrom( newClass );
        return true;
    }
*/


}
