/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: GRAPH
 * Classe:  CRelationSelector
\******************************************************************************/

package TOOL.UML;


import  java.awt.*;
import  java.util.*;

import  FILE.OBJ.*;
import  GRAPH.*;


public class    CRelationSelector
{
    protected   Vector          pSelPoints;

    protected   CSelectorPointInRectPath
                                pSelPointA, pSelPointB; //+

    protected   CObjRelation    pRelation; //+

    protected   CPolyline       pPolyline;
    protected   int             mSelPointIndex;

    protected   boolean         mMouseDown;
    protected   CSelectorPoint  pSelected;
    protected   Point           mInitialPoint;
    protected   Point           mLastPoint;

    public CRelationSelector( CObjRelation relation )
    {
        pRelation = relation;
        _CreatePointsAndSelectors();

        mMouseDown = true;
        pSelected = null;
        mInitialPoint = mLastPoint = null;
    }


    private void _CreatePointsAndSelectors()
    {
        // Create Polyline
        pPolyline = new CPolyline();
        pSelPoints = new Vector();

        // - First Point
        CObjRelationExtremity extr;
        CSidePosition sidePos;
        Rectangle rect;

        extr = pRelation.GetExtremity(0);
        sidePos = extr.GetSidePosition();

        rect = extr.GetClass().GetRegion();
        pPolyline.AddPoint( sidePos.GetPoint(rect) );

        pSelPointA = new CSelectorPointInRectPath( rect, sidePos );
        pSelPoints.addElement( pSelPointA );

        // - Internal Points
        Enumeration e = pRelation.GetInternalPoints();
        while( e.hasMoreElements() )
        {
            Point p = new Point( (Point) e.nextElement() );
            pPolyline.AddPoint( p );
            pSelPoints.addElement( new CSelectorPoint( p, CSelectorPoint.MOVE ) );
        }

        // - Last Point
        extr = pRelation.GetExtremity(1);
        sidePos = extr.GetSidePosition();

        rect = extr.GetClass().GetRegion();
        pPolyline.AddPoint( sidePos.GetPoint(rect) );

        pSelPointB = new CSelectorPointInRectPath( rect, sidePos );
        pSelPoints.addElement( pSelPointB );

    }

    public void Update()
    {
        _CreatePointsAndSelectors();
    }

    public Rectangle getBounds()
    {
        return pPolyline.GetBounds();
    }


    public CSidePosition    GetExtremitySidePosition( int extr )
    {
        if( extr == 0 )
            return pSelPointA.GetSidePosition();
        else
            return pSelPointB.GetSidePosition();
    }


    public CPolyline GetPolyline()
    {
        return new CPolyline( pPolyline );
    }

    public boolean Contains( int x, int y )
    {
        if( pPolyline.Contains(x,y) )
            return true;

        Enumeration e = pSelPoints.elements();
        while( e.hasMoreElements() )
        {
            CSelectorPoint point = (CSelectorPoint) e.nextElement();
            if( point.Contains( x, y ) )
                return true;
        }

        return false;
    }

    public void Draw( CGraphics g )
    {
        g.setXORMode(Color.darkGray);

        //g.SetDashedPen();
        g.RealDrawPolyline( pPolyline ); // ???
        //g.SetSolidPen();

        Enumeration e = pSelPoints.elements();
        while( e.hasMoreElements() )
            ( (CSelectorPoint) e.nextElement() ).Draw(g);

        g.setPaintMode();
    }

    public Cursor GetCursor( int x, int y )
    {
        if( mMouseDown )
            if( pSelected != null )
                return pSelected.GetCursor();
            else
                return Cursor.getPredefinedCursor( Cursor.MOVE_CURSOR );

        if( Contains(x,y) )
        {
            //return Cursor.getPredefinedCursor( Cursor.MOVE_CURSOR );
//*
            Enumeration e = pSelPoints.elements();
            while( e.hasMoreElements() )
            {
                CSelectorPoint point = (CSelectorPoint) e.nextElement();
                if( point.Contains( x, y ) )
                    return point.GetCursor();
            }

            //if( pPolyline.contains(x,y) )
                return Cursor.getPredefinedCursor( Cursor.MOVE_CURSOR );
/* */
        }

        return Cursor.getDefaultCursor();
    }

    public boolean IsMouseDown()        {    return mMouseDown;  }

    public boolean HasMoved()
    {
        if(mLastPoint == mInitialPoint)
            return false;

        if(mLastPoint != null)
            if( (mLastPoint.x==mInitialPoint.x) && (mLastPoint.y==mInitialPoint.y) )
                return false;

        return true;
    }


    public boolean MouseDown( CGraphics g, int x, int y )
    {
        if( !Contains(x,y) )    return false;

        // Verify if it is a selectorPoint
        pSelected = null;
        mSelPointIndex = -1;
        Enumeration e = pSelPoints.elements();
        while( (e.hasMoreElements()) && (pSelected==null) )
        {
            CSelectorPoint point = (CSelectorPoint) e.nextElement();
            mSelPointIndex++;

            if( point.Contains( x, y ) )
            {
                pSelected = point;
                //break;
            }
        }

        if( (pSelected != null) || (pPolyline.Contains(x,y)) )
        {
            mMouseDown = true;
            mInitialPoint = mLastPoint = new Point(x,y);
            //g.setXORMode();
            return true;
        }

        return false;
    }


    private void _TranslatePoints( int dx, int dy )
    {
        Enumeration e = pSelPoints.elements();
        while( e.hasMoreElements() )
        {
            CSelectorPoint p = (CSelectorPoint) e.nextElement();
            Point newPoint = new Point( p.GetPoint() );
            newPoint.translate( dx, dy );
            p.SetPoint( newPoint );
        }
    }

/*
    private void _MovePoints()
    {
        Enumeration e1 = pSelPoints.elements();
        Enumeration e2 = pPolyline.GetPoints();

        while( e1.hasMoreElements() )
        {
            CSelectorPoint selPoint = (CSelectorPoint) e1.nextElement();
            Point point = (Point) e2.nextElement();

            selPoint.SetPoint( new Point(point) );
        }
    }
*/


    public void MouseMove( CGraphics g, int x, int y )
    {
        if( !mMouseDown )   return;

        int dx, dy;
        dx = x - mLastPoint.x;
        dy = y - mLastPoint.y;

        mLastPoint = new Point(x,y);

        // Hides itself
        Draw( g );

        if( pSelected != null )
        {   // Move Point
            pSelected.SetPoint( mLastPoint );
            Point newPoint = pSelected.GetPoint();

            Point p = pPolyline.GetPointAt( mSelPointIndex );
            p.x = newPoint.x;
            p.y = newPoint.y;
        }

        else
        {   // the whole object was moved
            // Translate polyline and points
            _TranslatePoints( dx, dy );

            pPolyline.Translate( dx, dy );
            pPolyline.SetFirstPoint( pSelPointA.GetPoint() );
            pPolyline.SetLastPoint ( pSelPointB.GetPoint() );
            pPolyline.Update();
        }

        // Shows itself
        Draw( g );
    }


    public void MouseUp ( CGraphics g, int x, int y )
    {
        if( !mMouseDown ) return;

        // To be sure the selector is well drawn
        MouseMove( g, x, y );

        // Save last Point to future use ???
        mLastPoint = new Point( x, y );

        // "clear" temporary variables
        mMouseDown = false;
        pSelected = null;

        // restore Paint Mode to CGraphics
        g.setPaintMode();
    }


}
