/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * package: TOOL.UML.SERVER
 * Classe: UMLServerEventHandler
\******************************************************************************/

package TOOL.UML.SERVER;

import  java.awt.Rectangle;

import  COPSE.*;
import  COPSE.CONTROL.*;
import  COPSE.SERVER.*;
import  FILE.*;
import  FILE.OBJ.*;
import  GRAPH.CPolyline;
import  GRAPH.CSidePosition;
import  TOOL.UML.CUMLFileController;



/* ****************************************************************************
 * class UMLServerEventHandler
 ******************************************************************************/
public class    UMLServerEventHandler
    extends     CServerEventHandler
{
    protected   CVisualFileController   pFileController = null;

    public UMLServerEventHandler( CServer server )
    {   super( server );  }

    protected void ProcessEvent( CEvent event, CConnectionHandler handler )
    {
        CUserConnectionHandler userHandler = null;
        if( handler instanceof CUserConnectionHandler ) // Of course it's true, but ...
            userHandler = (CUserConnectionHandler) handler;

        if( pFileController == null )
            pFileController = (CVisualFileController) pServer.GetFileController();


        int type = event.GetType();

        switch( type )
        {
            case CEvent.EvCOPSE_FileController_Element_Visual:
                {
                    int elemId = ((Integer)event.GetArgumentAt(0)).intValue();
                    int visual = ((Integer)event.GetArgumentAt(1)).intValue();

                    pFileController.SetElementVisual( elemId, visual );
                    pSessionManager.NotifyAllExcept( (CServerUserSession)userHandler.GetUserSession(), event );
                }
                break;

            case CEvent.EvUMLEditor_Class_Position:
                {
                    int classId = ((Integer) event.GetArgumentAt(0)).intValue();
                    Rectangle r = (Rectangle) event.GetArgumentAt(1);

                    CElement elem = pFileController.GetFile().GetElement( classId );
                    if( elem instanceof CObjClass )
                    {
                        CObjClass aclass = (CObjClass) elem;
                        aclass.SetRegion( r );
                        pSessionManager.NotifyAll( event );
                    }
                }
                break;

            case CEvent.EvUMLEditor_Relation_Position:
                {
                    int relationId = ((Integer) event.GetArgumentAt(0)).intValue();
                    CSidePosition ext0 = (CSidePosition) event.GetArgumentAt(1);
                    CSidePosition ext1 = (CSidePosition) event.GetArgumentAt(2);
                    CPolyline poly = (CPolyline) event.GetArgumentAt(3);

                    CElement elem = pFileController.GetFile().GetElement( relationId );
                    if( elem instanceof CObjRelation )
                    {
                        CObjRelation relation = (CObjRelation) elem;

                        relation.GetExtremity(0).SetSidePosition( ext0 );
                        relation.GetExtremity(1).SetSidePosition( ext1 );
                        relation.SetInternalPoints( poly.GetPoints() );

                        pSessionManager.NotifyAll( event );
                    }
                }
                break;

            case CEvent.EvCOPSE_FileController_Element_New:
                {
                    CElement elem = (CElement) event.GetArgumentAt(1);
                    if( elem instanceof CObjRelation )
                    {
                        CObjRelation rel = (CObjRelation) elem;

                        Integer i0, i1;
                        i0 = (Integer) event.GetArgumentAt( 2 );
                        i1 = (Integer) event.GetArgumentAt( 3 );


                        CObjClass c0, c1;
                        try {
                            c0 = (CObjClass) pFileController.GetFile().GetElement( i0.intValue() );
                            c1 = (CObjClass) pFileController.GetFile().GetElement( i1.intValue() );
                        } catch ( Exception e )
                        { return; }


                        // Now it's ok to add relation...
                        super.ProcessEvent( event, handler );


                        // After adding the relation, stablish the links to classes...

                        CSidePosition s0, s1;
                        s0 = (CSidePosition) event.GetArgumentAt( 4 );
                        s1 = (CSidePosition) event.GetArgumentAt( 5 );

                        // Creating Extremities
                        CObjRelationExtremity extr = new CObjRelationExtremity( rel, c0 );
                        extr.SetSidePosition( s0 );

                        extr = new CObjRelationExtremity( rel, c1 );
                        extr.SetSidePosition( s1 );

                        return;
                    } // if newElement instanceof CObjRelation
                }
                break;

        }


        super.ProcessEvent( event, handler );

    }
}
