/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Project Server Application
 * Classe: COPSE
\******************************************************************************/


import java.io.*;
import java.net.*;
import java.util.*;


import COPSE.*;
import COPSE.SERVER.*;

import AGENT.*;


public class COPSEServer
    extends Thread
{
    protected   ServerSocket                    pServerSocket;

    protected   COPSEServerSessionManager       pSessionManager;

    protected   COPSEServerEventHandler         pEventHandler;
    protected   COPSEServerEventDispatcher      pEventDispatcher;

    protected   COPSEServerProjectManager       pProjectManager;

    protected   boolean                         mExit;


    public COPSEServer( String project, String host, int port )
        throws Exception
    {
        super();

        mExit = false;

        pSessionManager = new COPSEServerSessionManager();

        pEventHandler = new COPSEServerEventHandler( this );
        pEventHandler.start();
        pEventDispatcher = new COPSEServerEventDispatcher( this, pEventHandler );
        pEventDispatcher.start();

        try {
            pServerSocket = new ServerSocket( (new Integer(port)).intValue() );
        } catch( Exception e )
        {   throw e; }

        try {
            pProjectManager = new COPSEServerProjectManager( /*this,*/ project, host, port );
        } catch( Exception e )
        {   throw e;  }
    }


    public COPSEServerSessionManager GetSessionManager()    { return pSessionManager; }
    public COPSEServerProjectManager GetProjectManager()    { return pProjectManager; }


    public void run()
    {
        // Define the serverSocket accept timeout
        try {
            pServerSocket.setSoTimeout( 1 );
        } catch( SocketException e ){}

        mExit = false;
        while( !mExit )
        {
            Socket slaveSocket = null;

            // Wait for next connection request
            try {
                slaveSocket = pServerSocket.accept();
            } catch( Exception ex ) {};

            if( slaveSocket != null )
                _ConnectUser( slaveSocket );    // Try to open connection after verifying the user login
        }

        //pEventDispatcher.stop();

        // Close Project
        //pProjectManager.Close();

    }


    private void _ConnectUser( Socket socket )
    {
        CSocketStreamConnector pConnector;

        boolean     ok = false;
        Integer     userId = new Integer(0);

        try {
            // Create the Connector to the COPSE Client
            pConnector = CSocketStreamConnector.New( socket );

            // Load Initial Event
            CEvent event = pConnector.Receive();

            if( event.GetType() == CEvent.EvCOPSE_Connection_Open )
            {
                // Get User Identification
                String login    = (String) event.GetArgumentAt(0);
                String password = (String) event.GetArgumentAt(1);

                // Verify User login and password
                COPSEServerUsersManager usersManager = pProjectManager.GetUsersManager();
                CMember member = usersManager.FindUserByLogin( login );

                // if they are ok
                if( (member != null) && (member.CheckPassword(password)) )
                {
                    // Notify all COPSEClients before adding the user in the SessionManager
                    event = new CEvent( CEvent.EvCOPSE_Connection_Open );
                    event.AddArgument( new Integer(member.GetId()) );
                    pConnector.Send( event ); // To enforce this being the first event
                    pSessionManager.NotifyAll( event ); // Notify others before adding the user in the SessionManager

                    // Verify if the user is already in session
                    COPSEServerSession session = (COPSEServerSession) pSessionManager.FindSession( member.GetId() );
                    if( session == null )
                    {
                        session = new COPSEServerSession( member.GetId() );
                        pSessionManager.AddSession( session );
                    }

                    // Create the ConnectionHandler
                    COPSEServerConnectionHandler pConnection;
//                    pConnection = new COPSEServerConnectionHandler( pConnector, pEventHandler, session );
                    pConnection = new COPSEServerConnectionHandler( pConnector, pEventDispatcher, session );
                    pConnection.start();

                    System.out.println( "[DEBUG] COPSEServer: User '"+login+"'["+member.GetId()+"] logged..." );
                }
                else
                {
                    String msg;
                    if( member == null )
                         msg = "Invalid Login!";
                    else msg = "Invalid Password!";

                    event = new CEvent( CEvent.EvCOPSE_Connection_Error );
                    event.AddArgument( msg );
                    pConnector.Send( event );

                    pConnector.Close();

                    System.out.println( "[DEBUG] COPSEServer: Refused user login - "+msg );
                }


            } // if( EvCOPSE_Connection_Open )
            else
            {
                event = new CEvent( CEvent.EvCOPSE_Connection_Error );
                event.AddArgument( "Invalid OpenConnection Event"  );
                pConnector.Send( event );
                pConnector.Close();
                System.out.println( "[DEBUG] COPSEServer: Invalid OpenConnection Event!" );
            }

        }
        catch( Exception e ) {}
    }


    public void Exit()
    {
        // stop accepting new connections ...
        // then, the current thread will stop without calling "stop" method
        mExit = true;

        // Exit All servers...
        // During their exiting, the files will be saved/closed
        pProjectManager.ExitAllServers();
        // Now, there is no more servers !!!

        // Close all connections with COPSEClients
        CEvent event = new CEvent( CEvent.EvCOPSE_Server_Exit );
        pSessionManager.NotifyAll( event );
        pSessionManager.CloseAllConnectionHandlers();

        // Close/Save Project Special Files
        pProjectManager.Close();

        // Close de ServerSocket
        try{
            pServerSocket.close();
        } catch( Exception e ) {};

        // Stop Event Dispatcher
        pEventDispatcher.stop();

        // Stop the COPSEServerEventHandler thread (the current thread!)
        pEventHandler.stop();
    }

}

